@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: linux.cc,v 1.9 2008/02/05 22:33:33 sshwarts Exp $
+// $Id: linux.cc,v 1.10 2009/01/30 21:47:42 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 #include <stdio.h>
@@ -46,14 +46,14 @@ void bx_dbg_info_linux_command (void)
 
 class syscall_names_t {
 #define MAX_SYSCALLS 200
-  char *syscall_names_linux[MAX_SYSCALLS];
+  const char *syscall_names_linux[MAX_SYSCALLS];
 public:
-  syscall_names_t () { init (); }
-  void init ();
-  char *get_name (int num);
+  syscall_names_t() { init (); }
+  void init();
+  const char *get_name(int num);
 };
 
-void syscall_names_t::init ()
+void syscall_names_t::init()
 {
   for (int i=0; i<MAX_SYSCALLS; i++) {
     syscall_names_linux[i] = "<unknown syscall>";
@@ -71,13 +71,9 @@ void syscall_names_t::init ()
 #endif
 }
 
-char *syscall_names_t::get_name (int n)
+const char *syscall_names_t::get_name(int n)
 {
-  static char buf[64];
-  if (n < 0 || n > N_SYSCALLS) {
-    sprintf (buf, "syscall %d out of range", n);
-    return buf;
-  }
+  if (n < 0 || n > N_SYSCALLS) return 0;
   return syscall_names_linux[n];
 }
 
@@ -86,8 +82,11 @@ syscall_names_t syscall_names;
 void bx_dbg_linux_syscall (unsigned which_cpu)
 {
   Bit32u eax = BX_CPU(which_cpu)->get_reg32(BX_32BIT_REG_EAX);
-  char *name = syscall_names.get_name (eax);
-  fprintf (stderr, "linux system call %s (#%d)\n", name, eax);
+  const char *name = syscall_names.get_name(eax);
+  if (name)
+    fprintf (stderr, "linux system call %s (#%d)\n", name, eax);
+  else
+    fprintf (stderr, "system call (#%d) is out of range\n", eax);
 }
 
 #endif /* if BX_DEBUGGER */
