@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: symbols.cc,v 1.11 2008/03/30 14:32:14 sshwarts Exp $
+// $Id: symbols.cc,v 1.15.2.1 2009/06/07 07:49:09 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,70 +22,66 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 /////////////////////////////////////////////////////////////////////////
 
 #include "bochs.h"
 #include "cpu/cpu.h"
 
 #if BX_DEBUGGER
-#if !((BX_HAVE_HASH_MAP || BX_HAVE_HASH_MAP_H) && (BX_HAVE_SET || BX_HAVE_SET_H))
+#if !((BX_HAVE_MAP || BX_HAVE_MAP_H) && (BX_HAVE_SET || BX_HAVE_SET_H))
 
-static char *BX_HAVE_HASH_MAP_ERR = "context not implemented because BX_HAVE_HASH_MAP=0\n";
+static const char BX_HAVE_MAP_ERR[] = "context not implemented because BX_HAVE_MAP=0\n";
 
-char* bx_dbg_symbolic_address(Bit32u context, Bit32u eip, Bit32u base)
+const char* bx_dbg_symbolic_address(Bit32u context, Bit32u eip, Bit32u base)
 {
   static bx_bool first = true;
   if (first) {
-    dbg_printf(BX_HAVE_HASH_MAP_ERR);
+    dbg_printf(BX_HAVE_MAP_ERR);
     first = false;
   }
   return "unk. ctxt";
 }
 
-char* bx_dbg_symbolic_address_16bit(Bit32u eip, Bit32u cs)
+const char* bx_dbg_symbolic_address_16bit(Bit32u eip, Bit32u cs)
 {
   // just prints an error anyway
   return bx_dbg_symbolic_address (0,0,0);
 }
 
-int bx_dbg_symbol_command(char* filename, bx_bool global, Bit32u offset)
+int bx_dbg_symbol_command(const char* filename, bx_bool global, Bit32u offset)
 {
-  dbg_printf(BX_HAVE_HASH_MAP_ERR);
+  dbg_printf(BX_HAVE_MAP_ERR);
   return -1;
 }
 
-void bx_dbg_info_symbols_command(char *Symbol)
+void bx_dbg_info_symbols_command(const char *Symbol)
 {
-  dbg_printf(BX_HAVE_HASH_MAP_ERR);
+  dbg_printf(BX_HAVE_MAP_ERR);
 }
 
-int bx_dbg_lbreakpoint_symbol_command(char *Symbol)
+int bx_dbg_lbreakpoint_symbol_command(const char *Symbol)
 {
-  dbg_printf(BX_HAVE_HASH_MAP_ERR);
+  dbg_printf(BX_HAVE_MAP_ERR);
   return -1;
 }
 
-Bit32u bx_dbg_get_symbol_value(char *Symbol)
+Bit32u bx_dbg_get_symbol_value(const char *Symbol)
 {
   return 0;
 }
 
-char* bx_dbg_disasm_symbolic_address(Bit32u eip, Bit32u base)
+const char* bx_dbg_disasm_symbolic_address(Bit32u eip, Bit32u base)
 {
   return 0;
 }
 
-#else   /* if BX_HAVE_HASH_MAP == 1 */
+#else   /* if BX_HAVE_MAP == 1 */
 
-/* Haven't figured out how to port this code to OSF1 cxx compiler.
-   Until a more portable solution is found, at least make it easy
-   to disable the template code:  just set BX_HAVE_HASH_MAP=0
-   in config.h */
-#if BX_HAVE_HASH_MAP
-#include <hash_map>
-#elif BX_HAVE_HASH_MAP_H
-#include <hash_map.h>
+#if BX_HAVE_MAP
+#include <map>
+#elif BX_HAVE_MAP_H
+#include <map.h>
 #endif
 
 #if BX_HAVE_SET
@@ -94,18 +90,27 @@ char* bx_dbg_disasm_symbolic_address(Bit32u eip, Bit32u base)
 #include <set.h>
 #endif
 
-using namespace std;
-
 struct symbol_entry_t
 {
-  symbol_entry_t (Bit32u _start = 0, char* _name = 0)
+  symbol_entry_t (Bit32u _start = 0, const char* _name = 0)
   {
     start = _start;
-    name = _name;
+    name = _name ? strdup(_name) : 0;
   }
+  ~symbol_entry_t()
+  {
+    if (name)
+      free(name);
+  }
+
+  void trim_quotes(void);
 
   char* name;
   Bit32u start;
+
+private:
+  symbol_entry_t(const symbol_entry_t&);  // No definition
+  void operator=(const symbol_entry_t &); // No definition
 };
 
 struct lt_symbol_entry_t
@@ -126,8 +131,9 @@ struct lt_rsymbol_entry_t
 
 struct context_t
 {
-  typedef set<symbol_entry_t*,lt_symbol_entry_t>  sym_set_t;
-  typedef set<symbol_entry_t*,lt_rsymbol_entry_t> rsym_set_t;
+  typedef std::set<symbol_entry_t*,lt_symbol_entry_t>  sym_set_t;
+  typedef std::set<symbol_entry_t*,lt_rsymbol_entry_t> rsym_set_t;
+  typedef std::map<Bit32u,context_t*> map_t;
 
   context_t (Bit32u);
  ~context_t();
@@ -136,104 +142,107 @@ struct context_t
   symbol_entry_t* get_symbol_entry(Bit32u);
   symbol_entry_t* get_symbol_entry(const char *Symbol) const;
   void add_symbol(symbol_entry_t*);
-  const sym_set_t* get_all_symbols() const {return syms;}
-  const rsym_set_t* get_all_rsymbols() const {return rsyms;}
+  const sym_set_t* get_all_symbols() const {return &m_syms;}
+  const rsym_set_t* get_all_rsymbols() const {return &m_rsyms;}
 
-private:
-  static hash_map<int,context_t*>* map;
+  friend class context_dtor_t;
+protected:
+  static map_t s_map;
   // Forvard references (find name by address)
-  sym_set_t* syms;
+  sym_set_t m_syms;
   // Reverse references (find address by name)
-  rsym_set_t* rsyms;
-  Bit32u id;
+  rsym_set_t m_rsyms;
+  Bit32u m_id;
 };
 
-hash_map<int,context_t*>* context_t::map = new hash_map<int,context_t*>;
+context_t::map_t context_t::s_map;
 
-context_t::context_t (Bit32u _id)
+context_t::context_t (Bit32u id)
 {
-  id = _id;
-  syms = new sym_set_t;
-  rsyms = new rsym_set_t;
-  (*map)[id] = this;
+  m_id = id;
+  s_map[m_id] = this;
 }
 
 context_t::~context_t()
 {
-  if(syms) {
-    sym_set_t::iterator iter;
-    for(iter=syms->begin();iter!=syms->end();++iter)
-    if(*iter)
-      delete *iter;
-  }
-
-  if(rsyms) {
-    rsym_set_t::iterator iter;
-    for(iter=rsyms->begin();iter!=rsyms->end();++iter)
-      if(*iter)
-        delete *iter;
+  while (!m_syms.empty())
+  {
+    sym_set_t::iterator it = m_syms.begin();
+    symbol_entry_t *sym = *it;
+    m_syms.erase(it);
+    delete sym;
   }
 }
 
 context_t* context_t::get_context(Bit32u i)
 {
-  return (*map)[i];
+  return s_map[i];
 }
 
 symbol_entry_t* context_t::get_symbol_entry(Bit32u ip)
 {
-  symbol_entry_t probe;
-  probe.start = ip;
+  symbol_entry_t probe(ip, 0);
   // find the first symbol whose address is greater than ip.
-  if (syms->empty ()) return 0;
-  sym_set_t::iterator iter = syms->upper_bound(&probe);
+  if (m_syms.empty ())
+      return 0;
+  sym_set_t::iterator iter = m_syms.upper_bound(&probe);
 
-  if (iter == syms->end()) { // No symbol found
+  if (iter == m_syms.end()) { // No symbol found
     return 0;
   }
 
-  return *(--iter);
+  --iter;
+  if(iter == m_syms.end())
+      return 0;
+  return *iter;
 }
 
 symbol_entry_t* context_t::get_symbol_entry(const char *Symbol) const
 {
-  symbol_entry_t probe;
-  probe.name=(char *)Symbol;
-
-  if (rsyms->empty())
+  if (m_rsyms.empty())
     return 0;
 
+  symbol_entry_t probe(0, Symbol);
   rsym_set_t::const_iterator iter;
-  iter=rsyms->find(&probe);
-  if(iter==rsyms->end()) // No symbol found
+  iter=m_rsyms.find(&probe);
+  if(iter==m_rsyms.end()) // No symbol found
     return 0;
   return *iter;
 }
 
 void context_t::add_symbol(symbol_entry_t* sym)
 {
-  syms->insert(sym);
-  rsyms->insert(sym);
+  m_syms.insert(sym);
+  m_rsyms.insert(sym);
 }
 
-Bit32u bx_dbg_get_symbol_value(char *Symbol)
+void symbol_entry_t::trim_quotes(void)
+{
+  size_t len = strlen(name);
+  if (name[0] == '\"' && name[len-1] == '\"')
+  {
+    memmove(name, name+1, len);
+    name[len-2] = 0;
+  }
+}
+
+Bit32u bx_dbg_get_symbol_value(const char *Symbol)
 {
   context_t* cntx = context_t::get_context(0);
   if(!cntx) // Context not found
     return 0;
 
-  if (Symbol[0]=='\"') Symbol++;
-  int len = strlen(Symbol);
-  if (Symbol[len - 1] == '\"') Symbol[len - 1] = '\0';
+  symbol_entry_t s(0, Symbol);
+  s.trim_quotes();
 
-  symbol_entry_t* sym=cntx->get_symbol_entry(Symbol);
+  symbol_entry_t* sym=cntx->get_symbol_entry(s.name);
   if(!sym) // Symbol not found
     return 0;
 
   return sym->start;
 }
 
-char* bx_dbg_symbolic_address(Bit32u context, Bit32u eip, Bit32u base)
+const char* bx_dbg_symbolic_address(Bit32u context, Bit32u eip, Bit32u base)
 {
   static char buf[80];
 #if 0
@@ -265,7 +274,7 @@ char* bx_dbg_symbolic_address(Bit32u context, Bit32u eip, Bit32u base)
   return buf;
 }
 
-char* bx_dbg_disasm_symbolic_address(Bit32u eip, Bit32u base)
+const char* bx_dbg_disasm_symbolic_address(Bit32u eip, Bit32u base)
 {
   static char buf[80];
 
@@ -284,7 +293,7 @@ char* bx_dbg_disasm_symbolic_address(Bit32u eip, Bit32u base)
   return buf;
 }
 
-char* bx_dbg_symbolic_address_16bit(Bit32u eip, Bit32u cs)
+const char* bx_dbg_symbolic_address_16bit(Bit32u eip, Bit32u cs)
 {
   // in 16-bit code, the segment selector and offset are combined into a
   // 20-bit linear address = (segment selector<<4) + offset.
@@ -293,29 +302,26 @@ char* bx_dbg_symbolic_address_16bit(Bit32u eip, Bit32u cs)
   return bx_dbg_symbolic_address (0, eip+(cs<<4), 0);
 }
 
-int bx_dbg_symbol_command(char* filename, bx_bool global, Bit32u offset)
+int bx_dbg_symbol_command(const char* filename, bx_bool global, Bit32u offset)
 {
-  if (filename[0] == '"')
-    filename++;
-  int len = strlen(filename);
-  if (filename[len - 1] == '"')
-    filename[len - 1] = '\0';
+  symbol_entry_t file(0, filename);
+  file.trim_quotes();
 
   // Install symbols in correct context (page table)
   // The file format should be
   // address symbol (example '00002afe _StartLoseNT')
 
-  context_t* cntx = (global) ? context_t::get_context(0)
-      : context_t::get_context((BX_CPU(dbg_cpu)->cr3) >> 12);
+  Bit32u context_id = (global) ? 0 : (BX_CPU(dbg_cpu)->cr3) >> 12;
+
+  context_t* cntx = context_t::get_context(context_id);
 
   if (!cntx) {
-    cntx = (global) ? new context_t(0)
-       : new context_t((BX_CPU(dbg_cpu)->cr3) >> 12);
+    cntx = new context_t(context_id);
   }
 
-  FILE* fp = fopen(filename, "rt"); // 't' is need for win32, unixes simply ignore it
+  FILE* fp = fopen(file.name, "rt"); // 't' is need for win32, unixes simply ignore it
   if (!fp) {
-    dbg_printf ("Could not open symbol file '%s'\n", filename);
+    dbg_printf ("Could not open symbol file '%s'\n", file.name);
     return -1;
   }
 
@@ -325,7 +331,7 @@ int bx_dbg_symbol_command(char* filename, bx_bool global, Bit32u offset)
 
   while (fgets(buf, sizeof(buf), fp)) {
     // handle end of line (before error messages)
-    len = strlen(buf);
+    int len = strlen(buf);
     bool whole_line = (buf[len - 1] == '\n');
     if (whole_line)
       buf[len - 1] = 0;
@@ -336,14 +342,14 @@ int bx_dbg_symbol_command(char* filename, bx_bool global, Bit32u offset)
 
     if (!isspace(*sym_name)) {
       if (*sym_name == 0)
-        dbg_printf("%s:%d: missing symbol name\n", filename, line_num);
+        dbg_printf("%s:%d: missing symbol name\n", file.name, line_num);
       else
-        dbg_printf("%s:%d: syntax error near '%s'\n", filename, line_num, sym_name);
+        dbg_printf("%s:%d: syntax error near '%s'\n", file.name, line_num, sym_name);
       return -1;
     }
     ++sym_name;
 
-    symbol_entry_t* sym = new symbol_entry_t(addr + offset, strdup(sym_name));
+    symbol_entry_t* sym = new symbol_entry_t(addr + offset, sym_name);
     cntx->add_symbol(sym);
 
     // skip the rest of long line
@@ -359,7 +365,7 @@ int bx_dbg_symbol_command(char* filename, bx_bool global, Bit32u offset)
   return 0;
 }
 
-// chack if s1 is prefix of s2
+// check if s1 is prefix of s2
 static bool bx_dbg_strprefix(const char *s1, const char *s2)
 {
   if(!s1 || !s2)
@@ -372,7 +378,7 @@ static bool bx_dbg_strprefix(const char *s1, const char *s2)
   return strncmp(s1, s2, len)==0;
 }
 
-void bx_dbg_info_symbols_command(char *Symbol)
+void bx_dbg_info_symbols_command(const char *Symbol)
 {
   context_t* cntx = context_t::get_context(0);
 
@@ -389,20 +395,17 @@ void bx_dbg_info_symbols_command(char *Symbol)
       dbg_printf ("Symbols not loaded\n");
       return;
     }
-    // remove leading and trailing quotas
-    if (Symbol[0]=='\"') Symbol++;
-    int len = strlen(Symbol);
-    if (Symbol[len - 1] == '\"') Symbol[len - 1] = '\0';
 
-    symbol_entry_t probe;
-    probe.name=Symbol;
+    symbol_entry_t probe(0, Symbol);
+    // remove leading and trailing quotas
+    probe.trim_quotes();
     context_t::rsym_set_t::const_iterator iter;
     iter=rsyms->lower_bound(&probe);
 
-    if(iter==rsyms->end() || !bx_dbg_strprefix(Symbol, (*iter)->name))
+    if(iter==rsyms->end() || !bx_dbg_strprefix(probe.name, (*iter)->name))
       dbg_printf ("No symbols found\n");
     else {
-      for(;iter!=rsyms->end() && bx_dbg_strprefix(Symbol, (*iter)->name);++iter) {
+      for(;iter!=rsyms->end() && bx_dbg_strprefix(probe.name, (*iter)->name);++iter) {
         dbg_printf ("%08x: %s\n", (*iter)->start, (*iter)->name);
       }
     }
@@ -423,18 +426,18 @@ void bx_dbg_info_symbols_command(char *Symbol)
   }
 }
 
-int bx_dbg_lbreakpoint_symbol_command(char *Symbol)
+int bx_dbg_lbreakpoint_symbol_command(const char *symbol)
 {
   context_t* cntx = context_t::get_context(0);
   if(!cntx) {
     dbg_printf ("Global context not available\n");
     return -1;
   }
-  if (Symbol[0]=='\"') Symbol++;
-  int len = strlen(Symbol);
-  if (Symbol[len - 1] == '\"') Symbol[len - 1] = '\0';
+  symbol_entry_t probe(0, symbol);
+  // remove leading and trailing quotes
+  probe.trim_quotes();
 
-  const symbol_entry_t* sym=cntx->get_symbol_entry(Symbol);
+  const symbol_entry_t* sym=cntx->get_symbol_entry(probe.name);
   if(sym)
     return bx_dbg_lbreakpoint_command(bkRegular, sym->start);
   dbg_printf ("Symbol not found\n");
