@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: apic.cc,v 1.105 2007/12/07 10:59:18 sshwarts Exp $
+// $Id: apic.cc,v 1.128.2.1 2009/06/07 07:49:10 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (c) 2002 Zwane Mwaikambo, Stanislav Shwartsman
@@ -16,7 +16,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 //
 /////////////////////////////////////////////////////////////////////////
 
@@ -29,21 +29,20 @@
 
 #define LOG_THIS this->
 
-#define APIC_BROADCAST_PHYSICAL_DESTINATION_MODE (APIC_MAX_ID)
+#define BX_CPU_APIC(i) (&(BX_CPU(i)->lapic))
 
 #define BX_LAPIC_FIRST_VECTOR	0x10
 #define BX_LAPIC_LAST_VECTOR	0xfe
 
 ///////////// APIC BUS /////////////
 
-int apic_bus_deliver_interrupt(Bit8u vector, Bit8u dest, Bit8u delivery_mode, Bit8u dest_mode, bx_bool level, bx_bool trig_mode)
+int apic_bus_deliver_interrupt(Bit8u vector, Bit8u dest, Bit8u delivery_mode, bx_bool logical_dest, bx_bool level, bx_bool trig_mode)
 {
   if(delivery_mode == APIC_DM_LOWPRI)
   {
-     if(dest_mode == 0) {
+     if(! logical_dest) {
        // I/O subsytem initiated interrupt with lowest priority delivery
-       // mode is not supported in physical destination mode
-//     BX_ERROR(("Ignoring lowest priority interrupt in physical dest mode !"));
+       // which is not supported in physical destination mode
        return 0;
      }
      else {
@@ -52,9 +51,9 @@ int apic_bus_deliver_interrupt(Bit8u vector, Bit8u dest, Bit8u delivery_mode, Bi
   }
 
   // determine destination local apics and deliver
-  if(dest_mode == 0) {
-    if(dest == APIC_BROADCAST_PHYSICAL_DESTINATION_MODE)
-    {
+  if(! logical_dest) {
+    // physical destination mode
+    if((dest & APIC_ID_MASK) == APIC_MAX_ID) {
        return apic_bus_broadcast_interrupt(vector, delivery_mode, trig_mode, APIC_MAX_ID);
     }
     else {
@@ -69,7 +68,7 @@ int apic_bus_deliver_interrupt(Bit8u vector, Bit8u dest, Bit8u delivery_mode, Bi
 
        return 0;
     }
-  } 
+  }
   else {
     // logical destination mode
     if(dest == 0) return 0;
@@ -145,7 +144,7 @@ int apic_bus_broadcast_interrupt(Bit8u vector, Bit8u delivery_mode, bx_bool trig
 
 static void apic_bus_broadcast_eoi(Bit8u vector)
 {
-  bx_devices.ioapic->receive_eoi(vector);
+  DEV_ioapic_receive_eoi(vector);
 }
 
 #endif
@@ -166,127 +165,35 @@ void apic_bus_broadcast_smi(void)
 
 ////////////////////////////////////
 
-bx_generic_apic_c::bx_generic_apic_c(bx_phy_address base)
+bx_local_apic_c::bx_local_apic_c(BX_CPU_C *mycpu, unsigned id)
+  : base_addr(BX_LAPIC_BASE_ADDR), cpu(mycpu)
 {
-  put("APIC?");
-  settype(APICLOG);
-  id = APIC_UNKNOWN_ID;
-  set_base(base);
-}
+  apic_id = id;
+  BX_ASSERT(apic_id < APIC_MAX_ID);
 
-void bx_generic_apic_c::set_base(bx_phy_address newbase)
-{
-  newbase &= (~0xfff);
-  base_addr = newbase;
-  if (id != APIC_UNKNOWN_ID)
-    BX_INFO(("relocate APIC id=%d to 0x%08x", id, newbase));
-}
+  char buffer[16];
+  sprintf(buffer, "APIC%x", apic_id);
+  put(buffer);
 
-void bx_generic_apic_c::set_id(Bit8u newid) 
-{
-  BX_INFO(("set APIC ID to %d", newid));
-  id = newid;
-}
-
-bx_bool bx_generic_apic_c::is_selected(bx_phy_address addr, unsigned len)
-{
-  if((addr & ~0xfff) == get_base()) {
-    if((addr & 0xf) != 0)
-      BX_INFO(("warning: misaligned APIC access. addr=0x%08x, len=%d", addr, len));
-    return 1;
-  }
-  return 0;
-}
-
-void bx_generic_apic_c::read(bx_phy_address addr, void *data, unsigned len)
-{
-  if((addr & ~0x3) != ((addr+len-1) & ~0x3)) {
-    BX_PANIC(("APIC read at address 0x%08x spans 32-bit boundary !", addr));
-    return;
-  }
-  Bit32u value;
-  read_aligned(addr & ~0x3, &value);
-  if(len == 4) { // must be 32-bit aligned
-    *((Bit32u *)data) = value;
-    return;
-  }
-  // handle partial read, independent of endian-ness
-  value >>= (addr&3)*8;
-  if (len == 1)
-    *((Bit8u *) data) = value & 0xff;
-  else if (len == 2)
-    *((Bit16u *)data) = value & 0xffff;
-  else
-    BX_PANIC(("Unsupported APIC read at address 0x%08x, len=%d", addr, len));
-}
-
-void bx_generic_apic_c::write(bx_phy_address addr, void *data, unsigned len)
-{
-  if((addr & ~0x3) != ((addr+len-1) & ~0x3)) {
-    BX_PANIC(("APIC write at address 0x%08x spans 32-bit boundary !", addr));
-    return;
-  }
-  bx_phy_address addr_aligned = addr & ~0x3;
-  if(len == 4) { // must be 32-bit aligned
-    write_aligned(addr_aligned, (Bit32u*) data);
-    return;
-  }
-  // partial write to the apic register, need to update some bytes 
-  // and do not touch the others, i.e. to do RMW operation
-  Bit32u value;
-  read_aligned(addr_aligned, &value);  // apic read has no side effects
-  // handle partial write, independent of endian-ness
-  unsigned shift = (addr&3)*8;
-  if (len == 1) {
-    value &= ~(0xff << shift);
-    value |= (*((Bit8u *) data) << shift);
-  }
-  else if (len == 2) {
-    value &= ~(0xffff << shift);
-    value |= (*((Bit16u *)data) << shift);
-  }
-  else {
-    BX_PANIC(("Unsupported APIC write at address 0x%08x, len=%d", addr, len));
-  }
-  write_aligned(addr_aligned, &value);
-}
-
-bx_local_apic_c::bx_local_apic_c(BX_CPU_C *mycpu)
-  : bx_generic_apic_c(BX_LAPIC_BASE_ADDR), cpu(mycpu), cpu_id(cpu->which_cpu())
-{
-  // KPL: Register a non-active timer for use when the timer is started.
+  // Register a non-active timer for use when the timer is started.
   timer_handle = bx_pc_system.register_timer_ticks(this,
-            BX_CPU(0)->local_apic.periodic_smf, 0, 0, 0, "lapic");
+            BX_CPU(0)->lapic.periodic_smf, 0, 0, 0, "lapic");
   timer_active = 0;
 
-  reset();
-	
-  INTR = 0;
+  reset(BX_RESET_HARDWARE);
 }
 
-void bx_local_apic_c::reset() 
-{
-  /* same as INIT but also sets arbitration ID and APIC ID */
-  init();
-}
-
-void bx_local_apic_c::init()
+void bx_local_apic_c::reset(unsigned type)
 {
   int i;
 
-  bx_generic_apic_c::init();
-
-  BX_INFO(("local apic in %s initializing",
-     (cpu && cpu->name) ? cpu->name : "?"));
-
   // default address for a local APIC, can be moved
   base_addr = BX_LAPIC_BASE_ADDR;
   error_status = shadow_error_status = 0;
-  log_dest = 0;
+  ldr = 0;
   dest_format = 0xf;
   icr_hi = 0;
   icr_lo = 0;
-  log_dest = 0;
   task_priority = 0;
 
   for(i=0; i<BX_LAPIC_MAX_INTS; i++) {
@@ -307,60 +214,146 @@ void bx_local_apic_c::init()
     lvt[i] = 0x10000;	// all LVT are masked
   }
 
-  spurious_vector  = 0xff;   // software disabled(bit 8)
+  // split spurious vector register to 3 fields
+  spurious_vector = 0xff;
   software_enabled = 0;
-  focus_disable    = 0;
+  focus_disable = 0;
+
+  mode = BX_APIC_XAPIC_MODE;
+
+  INTR = 0;
 }
 
-void bx_local_apic_c::set_id(Bit8u newid)
+void bx_local_apic_c::set_base(bx_phy_address newbase)
 {
-  bx_generic_apic_c::set_id(newid);
-  sprintf(cpu->name, "CPU apicid=%02x",(Bit32u)id);
-  if(id < APIC_MAX_ID) {
-    char buffer[16];
-    sprintf(buffer, "APIC%x", id);
-    put(buffer);
-    settype(CPU0LOG + id);
-    sprintf(buffer, "CPU%x", id);
-    cpu->put(buffer);
-  } else {
-    BX_INFO(("naming convention for apics requires id=0-%d only", APIC_MAX_ID));
-  }
-  if(BX_CPU_LEVEL<2)
-    BX_INFO(("8086"));
-  else
-    BX_INFO(("80%d86", BX_CPU_LEVEL));
+  mode = (newbase >> 10) & 3;
+  newbase &= ~((bx_phy_address) 0xfff);
+  base_addr = newbase;
+  BX_INFO(("allocate APIC id=%d (MMIO %s) to 0x" FMT_PHY_ADDRX,
+    apic_id, (mode == BX_APIC_XAPIC_MODE) ? "enabled" : "disabled", newbase));
 }
 
+bx_bool bx_local_apic_c::is_selected(bx_phy_address addr)
+{
+  if (mode != BX_APIC_XAPIC_MODE) return 0;
+
+  if((addr & ~0xfff) == base_addr) {
+    if((addr & 0xf) != 0)
+      BX_INFO(("warning: misaligned APIC access. addr=0x" FMT_PHY_ADDRX, addr));
+    return 1;
+  }
+  return 0;
+}
+
+void bx_local_apic_c::read(bx_phy_address addr, void *data, unsigned len)
+{
+  if((addr & ~0x3) != ((addr+len-1) & ~0x3)) {
+    BX_PANIC(("APIC read at address 0x" FMT_PHY_ADDRX " spans 32-bit boundary !", addr));
+    return;
+  }
+  Bit32u value = read_aligned(addr & ~0x3);
+  if(len == 4) { // must be 32-bit aligned
+    *((Bit32u *)data) = value;
+    return;
+  }
+  // handle partial read, independent of endian-ness
+  value >>= (addr&3)*8;
+  if (len == 1)
+    *((Bit8u *) data) = value & 0xff;
+  else if (len == 2)
+    *((Bit16u *)data) = value & 0xffff;
+  else
+    BX_PANIC(("Unsupported APIC read at address 0x" FMT_PHY_ADDRX ", len=%d", addr, len));
+}
+
+void bx_local_apic_c::write(bx_phy_address addr, void *data, unsigned len)
+{
+  if (len != 4) {
+    BX_PANIC(("APIC write with len=%d (should be 4)", len));
+    return;
+  }
+
+  if(addr & 0xf) {
+    BX_PANIC(("APIC write at unaligned address 0x" FMT_PHY_ADDRX, addr));
+    return;
+  }
+
+  write_aligned(addr, *((Bit32u*) data));
+}
+
+#define BX_LAPIC_ID                   0x020
+#define BX_LAPIC_VERSION              0x030
+#define BX_LAPIC_TPR                  0x080
+#define BX_LAPIC_ARBITRATION_PRIORITY 0x090
+#define BX_LAPIC_PPR                  0x0A0
+#define BX_LAPIC_EOI                  0x0B0
+#define BX_LAPIC_LDR                  0x0D0
+#define BX_LAPIC_DESTINATION_FORMAT   0x0E0
+#define BX_LAPIC_SPURIOUS_VECTOR      0x0F0
+#define BX_LAPIC_ISR1                 0x100
+#define BX_LAPIC_ISR2                 0x110
+#define BX_LAPIC_ISR3                 0x120
+#define BX_LAPIC_ISR4                 0x130
+#define BX_LAPIC_ISR5                 0x140
+#define BX_LAPIC_ISR6                 0x150
+#define BX_LAPIC_ISR7                 0x160
+#define BX_LAPIC_ISR8                 0x170
+#define BX_LAPIC_TMR1                 0x180
+#define BX_LAPIC_TMR2                 0x190
+#define BX_LAPIC_TMR3                 0x1A0
+#define BX_LAPIC_TMR4                 0x1B0
+#define BX_LAPIC_TMR5                 0x1C0
+#define BX_LAPIC_TMR6                 0x1D0
+#define BX_LAPIC_TMR7                 0x1E0
+#define BX_LAPIC_TMR8                 0x1F0
+#define BX_LAPIC_IRR1                 0x200
+#define BX_LAPIC_IRR2                 0x210
+#define BX_LAPIC_IRR3                 0x220
+#define BX_LAPIC_IRR4                 0x230
+#define BX_LAPIC_IRR5                 0x240
+#define BX_LAPIC_IRR6                 0x250
+#define BX_LAPIC_IRR7                 0x260
+#define BX_LAPIC_IRR8                 0x270
+#define BX_LAPIC_ESR                  0x280
+#define BX_LAPIC_LVT_CMCI             0x2F0
+#define BX_LAPIC_ICR_LO               0x300
+#define BX_LAPIC_ICR_HI               0x310
+#define BX_LAPIC_LVT_TIMER            0x320
+#define BX_LAPIC_LVT_THERMAL          0x330
+#define BX_LAPIC_LVT_PERFMON          0x340
+#define BX_LAPIC_LVT_LINT0            0x350
+#define BX_LAPIC_LVT_LINT1            0x360
+#define BX_LAPIC_LVT_ERROR            0x370
+#define BX_LAPIC_TIMER_INITIAL_COUNT  0x380
+#define BX_LAPIC_TIMER_CURRENT_COUNT  0x390
+#define BX_LAPIC_TIMER_DIVIDE_CFG     0x3E0
+#define BX_LAPIC_SELF_IPI             0x3F0
+
 // APIC write: 4 byte write to 16-byte aligned APIC address
-void bx_local_apic_c::write_aligned(bx_phy_address addr, Bit32u *data)
+void bx_local_apic_c::write_aligned(bx_phy_address addr, Bit32u value)
 {
-  BX_DEBUG(("%s: LAPIC write 0x%08x to address %08x", cpu->name, *data, addr));
   BX_ASSERT((addr & 0xf) == 0);
-  addr &= 0xff0;
-  Bit32u value = *data;
-  switch(addr) {
-    case 0x20: // local APIC id
-      id = (value>>24) & APIC_ID_MASK;
-      break;
-    case 0x80: // task priority
+  unsigned apic_reg = addr & 0xff0;
+  BX_DEBUG(("LAPIC write 0x%08x to register 0x%04x", value, apic_reg));
+  switch(apic_reg) {
+    case BX_LAPIC_TPR: // task priority
       set_tpr(value & 0xff);
       break;
-    case 0xb0: // EOI
+    case BX_LAPIC_EOI: // EOI
       receive_EOI(value);
       break;
-    case 0xd0: // logical destination
-      log_dest = (value >> 24) & APIC_ID_MASK;
-      BX_DEBUG(("set logical destination to %02x", log_dest));
+    case BX_LAPIC_LDR: // logical destination
+      ldr = (value >> 24) & APIC_ID_MASK;
+      BX_DEBUG(("set logical destination to %08x", ldr));
       break;
-    case 0xe0: // destination format
+    case BX_LAPIC_DESTINATION_FORMAT:
       dest_format = (value >> 28) & 0xf;
       BX_DEBUG(("set destination format to %02x", dest_format));
       break;
-    case 0xf0: // spurious interrupt vector
+    case BX_LAPIC_SPURIOUS_VECTOR:
       write_spurious_interrupt_register(value);
       break;
-    case 0x280: // error status reg
+    case BX_LAPIC_ESR: // error status reg
       // Here's what the IA-devguide-3 says on p.7-45:
       // The ESR is a read/write register and is reset after being written to
       // by the processor. A write to the ESR must be done just prior to
@@ -368,67 +361,70 @@ void bx_local_apic_c::write_aligned(bx_phy_address addr, Bit32u *data)
       error_status = shadow_error_status;
       shadow_error_status = 0;
       break;
-    case 0x300: // interrupt command reg 0-31
+    case BX_LAPIC_ICR_LO: // interrupt command reg 0-31
       icr_lo = value & ~(1<<12);  // force delivery status bit = 0(idle)
       send_ipi();
       break;
-    case 0x310: // interrupt command reg 31-63
+    case BX_LAPIC_ICR_HI: // interrupt command reg 31-63
       icr_hi = value & 0xff000000;
       break;
-    case 0x320: // LVT Timer Reg
-      lvt[APIC_LVT_TIMER] = value & 0x300ff;
-      if(! software_enabled) lvt[APIC_LVT_TIMER] |= 0x10000;
-      break;
-    case 0x330: // LVT Thermal Monitor
-      lvt[APIC_LVT_THERMAL] = value & 0x107ff;
-      if(! software_enabled) lvt[APIC_LVT_THERMAL] |= 0x10000;
-      break;
-    case 0x340: // LVT Performance Counter
-      lvt[APIC_LVT_PERFORM] = value & 0x107ff;
-      if(! software_enabled) lvt[APIC_LVT_PERFORM] |= 0x10000;
-      break;
-    case 0x350: // LVT LINT0 Reg
-      lvt[APIC_LVT_LINT0] = value & 0x1a7ff;
-      if(! software_enabled) lvt[APIC_LVT_LINT0] |= 0x10000;
-      break;
-    case 0x360: // LVT Lint1 Reg
-      lvt[APIC_LVT_LINT1] = value & 0x1a7ff;
-      if(! software_enabled) lvt[APIC_LVT_LINT1] |= 0x10000;
-      break;
-    case 0x370: // LVT Error Reg
-      lvt[APIC_LVT_ERROR] = value & 0x100ff;
-      if(! software_enabled) lvt[APIC_LVT_ERROR] |= 0x10000;
-      break;
-    case 0x380: // initial count for timer
+    case BX_LAPIC_LVT_TIMER:   // LVT Timer Reg
+    case BX_LAPIC_LVT_THERMAL: // LVT Thermal Monitor
+    case BX_LAPIC_LVT_PERFMON: // LVT Performance Counter
+    case BX_LAPIC_LVT_LINT0:   // LVT LINT0 Reg
+    case BX_LAPIC_LVT_LINT1:   // LVT LINT1 Reg
+    case BX_LAPIC_LVT_ERROR:   // LVT Error Reg
+      {
+         static Bit32u lvt_mask[] = {
+           0x000310ff, /* TIMER */
+           0x000117ff, /* THERMAL */
+           0x000117ff, /* PERFMON */
+           0x0001f7ff, /* LINT0 */
+           0x0001f7ff, /* LINT1 */
+           0x000110ff  /* ERROR */
+         };
+         unsigned lvt_entry = (apic_reg - BX_LAPIC_LVT_TIMER) >> 4;
+         lvt[lvt_entry] = value & lvt_mask[lvt_entry];
+         if(! software_enabled) lvt[lvt_entry] |= 0x10000;
+         break;
+      }
+    case BX_LAPIC_TIMER_INITIAL_COUNT:
       set_initial_timer_count(value);
       break;
-    case 0x3e0: // timer divide configuration
+    case BX_LAPIC_TIMER_DIVIDE_CFG:
       // only bits 3, 1, and 0 are writable
       timer_divconf = value & 0xb;
       set_divide_configuration(timer_divconf);
       break;
     /* all read-only registers go here */
-    case 0x30: // local APIC version
-    case 0x90: // arbitration priority
-    case 0xa0: // processor priority
+    case BX_LAPIC_ID:      // local APIC id
+    case BX_LAPIC_VERSION: // local APIC version
+    case BX_LAPIC_ARBITRATION_PRIORITY:
+    case BX_LAPIC_PPR:     // processor priority
     // ISRs not writable
-    case 0x100: case 0x110: case 0x120: case 0x130:
-    case 0x140: case 0x150: case 0x160: case 0x170:
+    case BX_LAPIC_ISR1: case BX_LAPIC_ISR2:
+    case BX_LAPIC_ISR3: case BX_LAPIC_ISR4:
+    case BX_LAPIC_ISR5: case BX_LAPIC_ISR6:
+    case BX_LAPIC_ISR7: case BX_LAPIC_ISR8:
     // TMRs not writable
-    case 0x180: case 0x190: case 0x1a0: case 0x1b0:
-    case 0x1c0: case 0x1d0: case 0x1e0: case 0x1f0:
+    case BX_LAPIC_TMR1: case BX_LAPIC_TMR2:
+    case BX_LAPIC_TMR3: case BX_LAPIC_TMR4:
+    case BX_LAPIC_TMR5: case BX_LAPIC_TMR6:
+    case BX_LAPIC_TMR7: case BX_LAPIC_TMR8:
     // IRRs not writable
-    case 0x200: case 0x210: case 0x220: case 0x230:
-    case 0x240: case 0x250: case 0x260: case 0x270:
-      // current count for timer
-    case 0x390:
+    case BX_LAPIC_IRR1: case BX_LAPIC_IRR2:
+    case BX_LAPIC_IRR3: case BX_LAPIC_IRR4:
+    case BX_LAPIC_IRR5: case BX_LAPIC_IRR6:
+    case BX_LAPIC_IRR7: case BX_LAPIC_IRR8:
+    // current count for timer
+    case BX_LAPIC_TIMER_CURRENT_COUNT:
       // all read-only registers should fall into this line
-      BX_INFO(("warning: write to read-only APIC register 0x%02x", addr));
+      BX_INFO(("warning: write to read-only APIC register 0x%x", apic_reg));
       break;
     default:
       shadow_error_status |= APIC_ERR_ILLEGAL_ADDR;
       // but for now I want to know about it in case I missed some.
-      BX_PANIC(("APIC register %08x not implemented", addr));
+      BX_PANIC(("APIC register %x not implemented", apic_reg));
   }
 }
 
@@ -438,7 +434,7 @@ void bx_local_apic_c::send_ipi(void)
   int dest_shorthand = (icr_lo >> 18) & 3;
   int trig_mode = (icr_lo >> 15) & 1;
   int level = (icr_lo >> 14) & 1;
-  int dest_mode = (icr_lo >> 11) & 1;
+  int logical_dest = (icr_lo >> 11) & 1;
   int delivery_mode = (icr_lo >> 8) & 7;
   int vector = (icr_lo & 0xff);
   int accepted = 0;
@@ -457,7 +453,7 @@ void bx_local_apic_c::send_ipi(void)
 
   switch(dest_shorthand) {
   case 0:  // no shorthand, use real destination value
-    accepted = apic_bus_deliver_interrupt(vector, dest, delivery_mode, dest_mode, level, trig_mode);
+    accepted = apic_bus_deliver_interrupt(vector, dest, delivery_mode, logical_dest, level, trig_mode);
     break;
   case 1:  // self
     trigger_irq(vector, trig_mode);
@@ -481,7 +477,7 @@ void bx_local_apic_c::send_ipi(void)
 
 void bx_local_apic_c::write_spurious_interrupt_register(Bit32u value)
 {
-  BX_DEBUG(("write %08x to spurious interrupt register", value));
+  BX_DEBUG(("write of %08x to spurious interrupt register", value));
 
 #ifdef BX_IMPLEMENT_XAPIC
   spurious_vector = value & 0xff;
@@ -502,159 +498,150 @@ void bx_local_apic_c::write_spurious_interrupt_register(Bit32u value)
 
 void bx_local_apic_c::receive_EOI(Bit32u value)
 {
-  BX_DEBUG(("%s: Wrote 0x%x to EOI", cpu->name, value));
+  BX_DEBUG(("Wrote 0x%x to EOI", value));
   int vec = highest_priority_int(isr);
   if(vec < 0) {
     BX_DEBUG(("EOI written without any bit in ISR"));
   } else {
-      if((Bit32u) vec != spurious_vector) {
-        BX_DEBUG(("%s: local apic received EOI, hopefully for vector 0x%02x", cpu->name, vec));
-        isr[vec] = 0; 
-        if(tmr[vec]) {
-            apic_bus_broadcast_eoi(vec);
-            tmr[vec] = 0;
-        }
-        service_local_apic();
-      }
+    if ((Bit32u) vec != spurious_vector) {
+       BX_DEBUG(("local apic received EOI, hopefully for vector 0x%02x", vec));
+       isr[vec] = 0;
+       if(tmr[vec]) {
+           apic_bus_broadcast_eoi(vec);
+           tmr[vec] = 0;
+       }
+       service_local_apic();
+    }
   }
 
-  if(bx_dbg.apic)
-      print_status();
+  if(bx_dbg.apic) print_status();
 }
 
-void bx_local_apic_c::startup_msg(Bit32u vector)
+void bx_local_apic_c::startup_msg(Bit8u vector)
 {
-  if(cpu->debug_trap & BX_DEBUG_TRAP_SPECIAL) {
-    cpu->debug_trap &= ~BX_DEBUG_TRAP_SPECIAL;
-    cpu->eip_reg.dword.eip = 0;
-    cpu->load_seg_reg(&cpu->sregs[BX_SEG_REG_CS], vector*0x100);
-    BX_INFO(("%s started up at %04X:%08X by APIC", cpu->name, vector*0x100, cpu->eip_reg.dword.eip));
-  } else {
-    BX_INFO(("%s started up by APIC, but was not halted at the time", cpu->name));
-  }
+  cpu->deliver_SIPI(vector);
 }
 
 // APIC read: 4 byte read from 16-byte aligned APIC address
-void bx_local_apic_c::read_aligned(bx_phy_address addr, Bit32u *data)
+Bit32u bx_local_apic_c::read_aligned(bx_phy_address addr)
 {
-  BX_DEBUG(("%s: LAPIC read from address %08x", cpu->name, addr));
   BX_ASSERT((addr & 0xf) == 0);
-  *data = 0;  // default value for unimplemented registers
-  bx_phy_address addr2 = addr & 0xff0;
-  switch(addr2) {
-  case 0x20: // local APIC id
-    *data = (id) << 24; break;
-  case 0x30: // local APIC version
-    *data = BX_LAPIC_VERSION_ID; break;
-  case 0x80: // task priority
-    *data = task_priority & 0xff; break;
-  case 0x90: // arbitration priority
-    *data = get_apr(); break;
-  case 0xa0: // processor priority
-    *data = get_ppr(); break;
-  case 0xb0: // EOI
+  Bit32u data = 0;  // default value for unimplemented registers
+  unsigned apic_reg = addr & 0xff0;
+  BX_DEBUG(("LAPIC read from register 0x%04x", apic_reg));
+  switch(apic_reg) {
+  case BX_LAPIC_ID:      // local APIC id
+    data = apic_id << 24; break;
+  case BX_LAPIC_VERSION: // local APIC version
+    data = BX_LAPIC_VERSION_ID; break;
+  case BX_LAPIC_TPR:     // task priority
+    data = task_priority & 0xff; break;
+  case BX_LAPIC_ARBITRATION_PRIORITY:
+    data = get_apr(); break;
+  case BX_LAPIC_PPR:     // processor priority
+    data = get_ppr(); break;
+  case BX_LAPIC_EOI:     // EOI
     /*
      * Read-modify-write operations should operate without generating
      * exceptions, and are used by some operating systems to EOI.
      * The results of reads should be ignored by the OS.
      */
     break;
-  case 0xd0: // logical destination
-    *data = (log_dest & APIC_ID_MASK) << 24; break;
-  case 0xe0: // destination format
-    *data = ((dest_format & 0xf) << 24) | 0x0fffffff; break;
-  case 0xf0: // spurious interrupt vector
+  case BX_LAPIC_LDR:     // logical destination
+    data = (ldr & APIC_ID_MASK) << 24; break;
+  case BX_LAPIC_DESTINATION_FORMAT:
+    data = ((dest_format & 0xf) << 28) | 0x0fffffff; break;
+  case BX_LAPIC_SPURIOUS_VECTOR:
     {
       Bit32u reg = spurious_vector;
       if(software_enabled) reg |= 0x100;
       if(focus_disable) reg |= 0x200;
-      *data = reg; 
+      data = reg;
     }
     break;
-  case 0x100: case 0x110:
-  case 0x120: case 0x130:
-  case 0x140: case 0x150:
-  case 0x160: case 0x170:
+  case BX_LAPIC_ISR1: case BX_LAPIC_ISR2:
+  case BX_LAPIC_ISR3: case BX_LAPIC_ISR4:
+  case BX_LAPIC_ISR5: case BX_LAPIC_ISR6:
+  case BX_LAPIC_ISR7: case BX_LAPIC_ISR8:
     {
-      unsigned index = (addr2 - 0x100) << 1;
+      unsigned index = (apic_reg - BX_LAPIC_ISR1) << 1;
       Bit32u value = 0, mask = 1;
       for(int i=0;i<32;i++) {
         if(isr[index+i]) value |= mask;
         mask <<= 1;
       }
-      *data = value;
+      data = value;
     }
     break;
-  case 0x180: case 0x190:
-  case 0x1a0: case 0x1b0:
-  case 0x1c0: case 0x1d0:
-  case 0x1e0: case 0x1f0:
+  case BX_LAPIC_TMR1: case BX_LAPIC_TMR2:
+  case BX_LAPIC_TMR3: case BX_LAPIC_TMR4:
+  case BX_LAPIC_TMR5: case BX_LAPIC_TMR6:
+  case BX_LAPIC_TMR7: case BX_LAPIC_TMR8:
     {
-      unsigned index = (addr2 - 0x180) << 1;
+      unsigned index = (apic_reg - BX_LAPIC_TMR1) << 1;
       Bit32u value = 0, mask = 1;
       for(int i=0;i<32;i++) {
         if(tmr[index+i]) value |= mask;
         mask <<= 1;
       }
-      *data = value;
+      data = value;
     }
     break;
-  case 0x200: case 0x210:
-  case 0x220: case 0x230:
-  case 0x240: case 0x250:
-  case 0x260: case 0x270:
+  case BX_LAPIC_IRR1: case BX_LAPIC_IRR2:
+  case BX_LAPIC_IRR3: case BX_LAPIC_IRR4:
+  case BX_LAPIC_IRR5: case BX_LAPIC_IRR6:
+  case BX_LAPIC_IRR7: case BX_LAPIC_IRR8:
     {
-      unsigned index = (addr2 - 0x200) << 1;
+      unsigned index = (apic_reg - BX_LAPIC_IRR1) << 1;
       Bit32u value = 0, mask = 1;
       for(int i=0;i<32;i++) {
         if(irr[index+i]) value |= mask;
         mask <<= 1;
       }
-      *data = value;
+      data = value;
     }
     break;
-  case 0x280: // error status reg
-    *data = error_status; break;
-  case 0x300: // interrupt command reg  0-31
-    *data = icr_lo; break;
-  case 0x310: // interrupt command reg 31-63
-    *data = icr_hi; break;
-  case 0x320: // LVT Timer Reg
-  case 0x330: // LVT Thermal Monitor
-  case 0x340: // LVT Performance Counter
-  case 0x350: // LVT LINT0 Reg
-  case 0x360: // LVT Lint1 Reg
-  case 0x370: // LVT Error Reg
+  case BX_LAPIC_ESR:     // error status reg
+    data = error_status; break;
+  case BX_LAPIC_ICR_LO:  // interrupt command reg  0-31
+    data = icr_lo; break;
+  case BX_LAPIC_ICR_HI:  // interrupt command reg 31-63
+    data = icr_hi; break;
+  case BX_LAPIC_LVT_TIMER:   // LVT Timer Reg
+  case BX_LAPIC_LVT_THERMAL: // LVT Thermal Monitor
+  case BX_LAPIC_LVT_PERFMON: // LVT Performance Counter
+  case BX_LAPIC_LVT_LINT0:   // LVT LINT0 Reg
+  case BX_LAPIC_LVT_LINT1:   // LVT Lint1 Reg
+  case BX_LAPIC_LVT_ERROR:   // LVT Error Reg
     {
-      int index = (addr2 - 0x320) >> 4;
-      *data = lvt[index];
+      int index = (apic_reg - BX_LAPIC_LVT_TIMER) >> 4;
+      data = lvt[index];
       break;
     }
-  case 0x380: // initial count for timer
-    *data = timer_initial;
+  case BX_LAPIC_TIMER_INITIAL_COUNT: // initial count for timer
+    data = timer_initial;
     break;
-  case 0x390: // current count for timer
+  case BX_LAPIC_TIMER_CURRENT_COUNT: // current count for timer
     if(timer_active==0) {
-      *data = timer_current;
+      data = timer_current;
     } else {
-      Bit64u delta64;
-      Bit32u delta32;
-      delta64 = (bx_pc_system.time_ticks() - ticksInitial) / timer_divide_factor;
-      delta32 = (Bit32u) delta64;
+      Bit64u delta64 = (bx_pc_system.time_ticks() - ticksInitial) / timer_divide_factor;
+      Bit32u delta32 = (Bit32u) delta64;
       if(delta32 > timer_initial)
         BX_PANIC(("APIC: R(curr timer count): delta < initial"));
       timer_current = timer_initial - delta32;
-      *data = timer_current;
+      data = timer_current;
     }
     break;
-  case 0x3e0: // timer divide configuration
-    *data = timer_divconf; 
+  case BX_LAPIC_TIMER_DIVIDE_CFG:   // timer divide configuration
+    data = timer_divconf;
     break;
   default:
-    BX_INFO(("APIC register %08x not implemented", addr));
+    BX_INFO(("APIC register %08x not implemented", apic_reg));
   }
 
-  BX_DEBUG(("%s: read from APIC address %08x = %08x", cpu->name, addr, *data));
+  BX_DEBUG(("read from APIC address 0x" FMT_PHY_ADDRX " = %08x", addr, data));
+  return data;
 }
 
 int bx_local_apic_c::highest_priority_int(Bit8u *array)
@@ -677,11 +664,11 @@ void bx_local_apic_c::service_local_apic(void)
   if (first_irr < 0) return;   // no interrupts, leave INTR=0
   int first_isr = highest_priority_int(isr);
   if (first_isr >= 0 && first_irr <= first_isr) {
-    BX_DEBUG(("local apic(%s): not delivering int 0x%02x because int 0x%02x is in service", cpu->name, first_irr, first_isr));
+    BX_DEBUG(("lapic(%d): not delivering int 0x%02x because int 0x%02x is in service", apic_id, first_irr, first_isr));
     return;
   }
   if(((Bit32u)(first_irr) & 0xf0) <= (task_priority & 0xf0)) {
-    BX_DEBUG(("local apic(%s): not delivering int 0x%02X because task_priority is 0x%02X", cpu->name, first_irr, task_priority));
+    BX_DEBUG(("lapic(%d): not delivering int 0x%02X because task_priority is 0x%02X", apic_id, first_irr, task_priority));
     return;
   }
   // interrupt has appeared in irr. Raise INTR. When the CPU
@@ -701,19 +688,19 @@ bx_bool bx_local_apic_c::deliver(Bit8u vector, Bit8u delivery_mode, Bit8u trig_m
     trigger_irq(vector, trig_mode);
     break;
   case APIC_DM_SMI:
-    BX_PANIC(("Delivery of SMI still not implemented !"));
+    BX_INFO(("Deliver SMI"));
     cpu->deliver_SMI();
     return 1;
   case APIC_DM_NMI:
-    BX_PANIC(("Delivery of NMI still not implemented !"));
+    BX_INFO(("Deliver NMI"));
     cpu->deliver_NMI();
     return 1;
   case APIC_DM_INIT:
-    BX_DEBUG(("Deliver INIT IPI"));
-    init();
+    BX_INFO(("Deliver INIT IPI"));
+    cpu->deliver_INIT();
     break;
   case APIC_DM_SIPI:
-    BX_DEBUG(("Deliver Start Up IPI"));
+    BX_INFO(("Deliver Start Up IPI"));
     startup_msg(vector);
     break;
   case APIC_DM_EXTINT:
@@ -727,10 +714,10 @@ bx_bool bx_local_apic_c::deliver(Bit8u vector, Bit8u delivery_mode, Bit8u trig_m
   return 1;
 }
 
-void bx_local_apic_c::trigger_irq(unsigned vector, unsigned trigger_mode, bx_bool bypass_irr_isr)
+void bx_local_apic_c::trigger_irq(Bit8u vector, unsigned trigger_mode, bx_bool bypass_irr_isr)
 {
-  BX_DEBUG(("Local apic on %s: trigger interrupt vector=0x%x", cpu->name, vector));
-  
+  BX_DEBUG(("trigger interrupt vector=0x%02x", vector));
+
   if(vector > BX_LAPIC_LAST_VECTOR || vector < BX_LAPIC_FIRST_VECTOR) {
     shadow_error_status |= APIC_ERR_RX_ILLEGAL_VEC;
     BX_INFO(("bogus vector %#x, ignoring ...", vector));
@@ -754,9 +741,9 @@ service_vector:
   service_local_apic();
 }
 
-void bx_local_apic_c::untrigger_irq(unsigned vector, unsigned trigger_mode)
+void bx_local_apic_c::untrigger_irq(Bit8u vector, unsigned trigger_mode)
 {
-  BX_DEBUG(("Local apic on %s: untrigger interrupt vector=0x%x", cpu->name, vector));
+  BX_DEBUG(("untrigger interrupt vector=0x%02x", vector));
   // hardware says "no more".  clear the bit.  If the CPU hasn't yet
   // acknowledged the interrupt, it will never be serviced.
   BX_ASSERT(irr[vector] == 1);
@@ -768,13 +755,14 @@ Bit8u bx_local_apic_c::acknowledge_int(void)
 {
   // CPU calls this when it is ready to service one interrupt
   if(!INTR)
-    BX_PANIC(("%s: acknowledged an interrupt, but INTR=0", cpu->name));
+    BX_PANIC(("APIC %d acknowledged an interrupt, but INTR=0", apic_id));
+
   BX_ASSERT(INTR);
   int vector = highest_priority_int(irr);
   if (vector < 0) goto spurious;
   if((vector & 0xf0) <= get_ppr()) goto spurious;
   BX_ASSERT(irr[vector] == 1);
-  BX_DEBUG(("%s: acknowledge_int returning vector 0x%x", cpu->name, vector));
+  BX_DEBUG(("acknowledge_int() returning vector 0x%02x", vector));
   irr[vector] = 0;
   isr[vector] = 1;
   if(bx_dbg.apic) {
@@ -794,7 +782,7 @@ spurious:
 
 void bx_local_apic_c::print_status(void)
 {
-  BX_INFO(("%s local apic: status is {:", cpu->name));
+  BX_INFO(("lapic %d: status is {:", apic_id));
   for(int vec=0; vec<BX_LAPIC_MAX_INTS; vec++) {
     if(irr[vec] || isr[vec]) {
       BX_INFO(("vec 0x%x: irr=%d, isr=%d", vec,(int)irr[vec],(int)isr[vec]));
@@ -803,14 +791,28 @@ void bx_local_apic_c::print_status(void)
   BX_INFO(("}"));
 }
 
-bx_bool bx_local_apic_c::match_logical_addr(Bit8u address) 
+bx_bool bx_local_apic_c::match_logical_addr(Bit8u address)
 {
-  if(dest_format != 0xf) {
-    BX_PANIC(("bx_local_apic_c::match_logical_addr: cluster model addressing not implemented"));
+  bx_bool match = 0;
+
+  if (dest_format == 0xf) {
+    // flat model
+    match = ((address & ldr) != 0);
+    BX_DEBUG(("comparing MDA %02x to my LDR %02x -> %s", address,
+      ldr, match ? "Match" : "Not a match"));
   }
-  bx_bool match = ((address & log_dest) != 0);
-  BX_DEBUG(("%s: comparing MDA %02x to my LDR %02x -> %s", cpu->name,
-    address, log_dest, match? "Match" : "Not a match"));
+  else if (dest_format == 0) {
+    // cluster model
+    if (address == 0xff) // broadcast all
+      return 1;
+
+    if ((unsigned)(address & 0xf0) == (unsigned)(ldr & 0xf0))
+      match = ((address & ldr & 0x0f) != 0);
+  }
+  else {
+    BX_PANIC(("bx_local_apic_c::match_logical_addr: unsupported dest format 0x%x", dest_format));
+  }
+
   return match;
 }
 
@@ -818,20 +820,15 @@ Bit8u bx_local_apic_c::get_ppr(void)
 {
   int ppr = highest_priority_int(isr);
 
-  if((ppr < 0) || ((task_priority & 0xF0) >= ((Bit32u) ppr & 0xF0))) 
+  if((ppr < 0) || ((task_priority & 0xF0) >= ((Bit32u) ppr & 0xF0)))
     ppr = task_priority;
-  else 
+  else
     ppr &= 0xF0;
 
   return ppr;
 }
 
-Bit8u bx_local_apic_c::get_tpr(void)
-{
-  return task_priority;
-}
-
-void  bx_local_apic_c::set_tpr(Bit8u priority)
+void bx_local_apic_c::set_tpr(Bit8u priority)
 {
   if(priority < task_priority) {
     task_priority = priority;
@@ -844,13 +841,14 @@ void  bx_local_apic_c::set_tpr(Bit8u priority)
 Bit8u bx_local_apic_c::get_apr(void)
 {
   Bit32u tpr  = (task_priority >> 4) & 0xf;
-  Bit32u isrv = (highest_priority_int(isr) >> 4) & 0xf;
-  Bit32u irrv = (highest_priority_int(irr) >> 4) & 0xf;
+  int first_isr = highest_priority_int(isr);
+  if (first_isr < 0) first_isr = 0;
+  int first_irr = highest_priority_int(irr);
+  if (first_irr < 0) first_irr = 0;
+  Bit32u isrv = (first_isr >> 4) & 0xf;
+  Bit32u irrv = (first_irr >> 4) & 0xf;
   Bit8u  apr;
 
-  if(isrv < 0) isrv = 0;
-  if(irrv < 0) irrv = 0;
-
   if((tpr >= irrv) && (tpr > isrv)) {
     apr = task_priority & 0xff;
   }
@@ -879,7 +877,7 @@ void bx_local_apic_c::periodic_smf(void *this_ptr)
 void bx_local_apic_c::periodic(void)
 {
   if(!timer_active) {
-    BX_ERROR(("%s: bx_local_apic_c::periodic called, timer_active==0", cpu->name));
+    BX_ERROR(("bx_local_apic_c::periodic called, timer_active==0"));
     return;
   }
 
@@ -892,12 +890,12 @@ void bx_local_apic_c::periodic(void)
       trigger_irq(timervec & 0xff, APIC_EDGE_TRIGGERED);
     }
     else {
-      BX_DEBUG(("%s: local apic timer LVT masked", cpu->name));
+      BX_DEBUG(("local apic timer LVT masked"));
     }
     // Reload timer values.
     timer_current = timer_initial;
     ticksInitial = bx_pc_system.time_ticks(); // Take a reading.
-    BX_DEBUG(("%s: local apic timer(periodic) triggered int, reset counter to 0x%08x", cpu->name, timer_current));
+    BX_DEBUG(("local apic timer(periodic) triggered int, reset counter to 0x%08x", timer_current));
   }
   else {
     // one-shot mode
@@ -907,10 +905,10 @@ void bx_local_apic_c::periodic(void)
       trigger_irq(timervec & 0xff, APIC_EDGE_TRIGGERED);
     }
     else {
-      BX_DEBUG(("%s: local apic timer LVT masked", cpu->name));
+      BX_DEBUG(("local apic timer LVT masked"));
     }
     timer_active = 0;
-    BX_DEBUG(("%s: local apic timer(one-shot) triggered int", cpu->name));
+    BX_DEBUG(("local apic timer(one-shot) triggered int"));
     bx_pc_system.deactivate_timer(timer_handle);
   }
 }
@@ -921,8 +919,8 @@ void bx_local_apic_c::set_divide_configuration(Bit32u value)
   // move bit 3 down to bit 0.
   value = ((value & 8) >> 1) | (value & 3);
   BX_ASSERT(value >= 0 && value <= 7);
-  timer_divide_factor = (value==7)? 1 : (2 << value);
-  BX_INFO(("%s: set timer divide factor to %d", cpu->name, timer_divide_factor));
+  timer_divide_factor = (value==7) ? 1 : (2 << value);
+  BX_INFO(("set timer divide factor to %d", timer_divide_factor));
 }
 
 void bx_local_apic_c::set_initial_timer_count(Bit32u value)
@@ -959,15 +957,15 @@ void bx_local_apic_c::register_state(bx_param_c *parent)
   bx_list_c *lapic = new bx_list_c(parent, "local_apic", 25);
 
   BXRS_HEX_PARAM_SIMPLE(lapic, base_addr);
-  BXRS_HEX_PARAM_SIMPLE(lapic, id);
+  BXRS_HEX_PARAM_SIMPLE(lapic, apic_id);
+  BXRS_HEX_PARAM_SIMPLE(lapic, mode);
   BXRS_HEX_PARAM_SIMPLE(lapic, spurious_vector);
   BXRS_PARAM_BOOL(lapic, software_enabled, software_enabled);
   BXRS_PARAM_BOOL(lapic, focus_disable, focus_disable);
   BXRS_HEX_PARAM_SIMPLE(lapic, task_priority);
-  BXRS_HEX_PARAM_SIMPLE(lapic, spurious_vector);
-  BXRS_HEX_PARAM_SIMPLE(lapic, log_dest);
+  BXRS_HEX_PARAM_SIMPLE(lapic, ldr);
   BXRS_HEX_PARAM_SIMPLE(lapic, dest_format);
-  
+
   bx_list_c *ISR = new bx_list_c(lapic, "isr", BX_LAPIC_MAX_INTS);
   bx_list_c *TMR = new bx_list_c(lapic, "tmr", BX_LAPIC_MAX_INTS);
   bx_list_c *IRR = new bx_list_c(lapic, "irr", BX_LAPIC_MAX_INTS);
