@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: arith32.cc,v 1.78 2008/05/06 05:06:32 sshwarts Exp $
+// $Id: arith32.cc,v 1.83 2009/01/16 18:18:58 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,7 +22,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA B 02110-1301 USA
 /////////////////////////////////////////////////////////////////////////
 
 #define NEED_CPU_REG_SHORTCUTS 1
@@ -54,9 +54,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_EdGdM(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32, sum_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op2_32 = BX_READ_32BIT_REG(i->nnn());
   sum_32 = op1_32 + op2_32;
   write_RMW_virtual_dword(sum_32);
@@ -64,33 +64,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_EdGdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_32(op1_32, op2_32, sum_32);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_EdGdR(bxInstruction_c *i)
-{
-  Bit32u op1_32, op2_32, sum_32;
-
-  op1_32 = BX_READ_32BIT_REG(i->rm());
-  op2_32 = BX_READ_32BIT_REG(i->nnn());
-  sum_32 = op1_32 + op2_32;
-  BX_WRITE_32BIT_REGZ(i->rm(), sum_32);
-
-  SET_FLAGS_OSZAPC_ADD_32(op1_32, op2_32, sum_32);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_GdEdM(bxInstruction_c *i)
-{
-  Bit32u op1_32, op2_32, sum_32;
-
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  op1_32 = BX_READ_32BIT_REG(i->nnn());
-  op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-  sum_32 = op1_32 + op2_32;
-
-  BX_WRITE_32BIT_REGZ(i->nnn(), sum_32);
-
-  SET_FLAGS_OSZAPC_ADD_32(op1_32, op2_32, sum_32);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_GdEdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32, sum_32;
@@ -121,9 +94,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_EdGdM(bxInstruction_c *i)
 
   Bit32u op1_32, op2_32, sum_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op2_32 = BX_READ_32BIT_REG(i->nnn());
   sum_32 = op1_32 + op2_32 + temp_CF;
   write_RMW_virtual_dword(sum_32);
@@ -131,36 +104,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_EdGdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_32(op1_32, op2_32, sum_32, BX_LF_INSTR_ADD_ADC32(temp_CF));
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_EdGdR(bxInstruction_c *i)
-{
-  bx_bool temp_CF = getB_CF();
-
-  Bit32u op1_32, op2_32, sum_32;
-
-  op1_32 = BX_READ_32BIT_REG(i->rm());
-  op2_32 = BX_READ_32BIT_REG(i->nnn());
-  sum_32 = op1_32 + op2_32 + temp_CF;
-  BX_WRITE_32BIT_REGZ(i->rm(), sum_32);
-
-  SET_FLAGS_OSZAPC_32(op1_32, op2_32, sum_32, BX_LF_INSTR_ADD_ADC32(temp_CF));
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_GdEdM(bxInstruction_c *i)
-{
-  bx_bool temp_CF = getB_CF();
-
-  Bit32u op1_32, op2_32, sum_32;
-
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  op1_32 = BX_READ_32BIT_REG(i->nnn());
-  op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-  sum_32 = op1_32 + op2_32 + temp_CF;
-  BX_WRITE_32BIT_REGZ(i->nnn(), sum_32);
-
-  SET_FLAGS_OSZAPC_32(op1_32, op2_32, sum_32, BX_LF_INSTR_ADD_ADC32(temp_CF));
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_GdEdR(bxInstruction_c *i)
 {
   bx_bool temp_CF = getB_CF();
@@ -194,9 +137,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_EdGdM(bxInstruction_c *i)
 
   Bit32u op1_32, op2_32, diff_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op2_32 = BX_READ_32BIT_REG(i->nnn());
   diff_32 = op1_32 - (op2_32 + temp_CF);
   write_RMW_virtual_dword(diff_32);
@@ -204,36 +147,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_EdGdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_32(op1_32, op2_32, diff_32, BX_LF_INSTR_SUB_SBB32(temp_CF));
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_EdGdR(bxInstruction_c *i)
-{
-  bx_bool temp_CF = getB_CF();
-
-  Bit32u op1_32, op2_32, diff_32;
-
-  op1_32 = BX_READ_32BIT_REG(i->rm());
-  op2_32 = BX_READ_32BIT_REG(i->nnn());
-  diff_32 = op1_32 - (op2_32 + temp_CF);
-  BX_WRITE_32BIT_REGZ(i->rm(), diff_32);
-
-  SET_FLAGS_OSZAPC_32(op1_32, op2_32, diff_32, BX_LF_INSTR_SUB_SBB32(temp_CF));
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_GdEdM(bxInstruction_c *i)
-{
-  bx_bool temp_CF = getB_CF();
-
-  Bit32u op1_32, op2_32, diff_32;
-
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  op1_32 = BX_READ_32BIT_REG(i->nnn());
-  op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-  diff_32 = op1_32 - (op2_32 + temp_CF);
-  BX_WRITE_32BIT_REGZ(i->nnn(), diff_32);
-
-  SET_FLAGS_OSZAPC_32(op1_32, op2_32, diff_32, BX_LF_INSTR_SUB_SBB32(temp_CF));
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_GdEdR(bxInstruction_c *i)
 {
   bx_bool temp_CF = getB_CF();
@@ -268,9 +181,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_EdIdM(bxInstruction_c *i)
 
   Bit32u op1_32, op2_32 = i->Id(), diff_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   diff_32 = op1_32 - (op2_32 + temp_CF);
   write_RMW_virtual_dword(diff_32);
 
@@ -294,9 +207,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_EdGdM(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32, diff_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op2_32 = BX_READ_32BIT_REG(i->nnn());
   diff_32 = op1_32 - op2_32;
   write_RMW_virtual_dword(diff_32);
@@ -304,32 +217,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_EdGdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_32(op1_32, op2_32, diff_32);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_EdGdR(bxInstruction_c *i)
-{
-  Bit32u op1_32, op2_32, diff_32;
-
-  op1_32 = BX_READ_32BIT_REG(i->rm());
-  op2_32 = BX_READ_32BIT_REG(i->nnn());
-  diff_32 = op1_32 - op2_32;
-  BX_WRITE_32BIT_REGZ(i->rm(), diff_32);
-
-  SET_FLAGS_OSZAPC_SUB_32(op1_32, op2_32, diff_32);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_GdEdM(bxInstruction_c *i)
-{
-  Bit32u op1_32, op2_32, diff_32;
-
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  op1_32 = BX_READ_32BIT_REG(i->nnn());
-  op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-  diff_32 = op1_32 - op2_32;
-  BX_WRITE_32BIT_REGZ(i->nnn(), diff_32);
-
-  SET_FLAGS_OSZAPC_SUB_32(op1_32, op2_32, diff_32);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_GdEdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32, diff_32;
@@ -358,39 +245,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_EdGdM(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32, diff_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_virtual_dword(i->seg(), eaddr);
   op2_32 = BX_READ_32BIT_REG(i->nnn());
   diff_32 = op1_32 - op2_32;
 
   SET_FLAGS_OSZAPC_SUB_32(op1_32, op2_32, diff_32);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_EdGdR(bxInstruction_c *i)
-{
-  Bit32u op1_32, op2_32, diff_32;
-
-  op1_32 = BX_READ_32BIT_REG(i->rm());
-  op2_32 = BX_READ_32BIT_REG(i->nnn());
-  diff_32 = op1_32 - op2_32;
-
-  SET_FLAGS_OSZAPC_SUB_32(op1_32, op2_32, diff_32);
-}
-
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_GdEdM(bxInstruction_c *i)
-{
-  Bit32u op1_32, op2_32, diff_32;
-
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  op1_32 = BX_READ_32BIT_REG(i->nnn());
-  op2_32 = read_virtual_dword(i->seg(), RMAddr(i));
-  diff_32 = op1_32 - op2_32;
-
-  SET_FLAGS_OSZAPC_SUB_32(op1_32, op2_32, diff_32);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_GdEdR(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32, diff_32;
@@ -447,13 +310,13 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CDQ(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPXCHG_XBTS(bxInstruction_c *i)
 {
   BX_INFO(("CMPXCHG_XBTS: Generate #UD exception"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPXCHG_IBTS(bxInstruction_c *i)
 {
   BX_INFO(("CMPXCHG_IBTS: Generate #UD exception"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::XADD_EdGdM(bxInstruction_c *i)
@@ -467,9 +330,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XADD_EdGdM(bxInstruction_c *i)
    * dst  <-- tmp               | op1 = sum
    */
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op2_32 = BX_READ_32BIT_REG(i->nnn());
   sum_32 = op1_32 + op2_32;
   write_RMW_virtual_dword(sum_32);
@@ -480,7 +343,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XADD_EdGdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_32(op1_32, op2_32, sum_32);
 #else
   BX_INFO (("XADD_EdGd not supported for cpulevel <= 3"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
@@ -509,7 +372,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XADD_EdGdR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_32(op1_32, op2_32, sum_32);
 #else
   BX_INFO (("XADD_EdGd not supported for cpulevel <= 3"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
@@ -517,9 +380,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_EdIdM(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32, sum_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op2_32 = i->Id();
   sum_32 = op1_32 + op2_32;
   write_RMW_virtual_dword(sum_32);
@@ -546,9 +409,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_EdIdM(bxInstruction_c *i)
 
   Bit32u op1_32, op2_32 = i->Id(), sum_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   sum_32 = op1_32 + op2_32 + temp_CF;
   write_RMW_virtual_dword(sum_32);
 
@@ -572,9 +435,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_EdIdM(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32 = i->Id(), diff_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   diff_32 = op1_32 - op2_32;
   write_RMW_virtual_dword(diff_32);
 
@@ -596,9 +459,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_EdIdM(bxInstruction_c *i)
 {
   Bit32u op1_32, op2_32, diff_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_virtual_dword(i->seg(), eaddr);
   op2_32 = i->Id();
   diff_32 = op1_32 - op2_32;
 
@@ -620,9 +483,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::NEG_EdM(bxInstruction_c *i)
 {
   Bit32u op1_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op1_32 = - (Bit32s)(op1_32);
   write_RMW_virtual_dword(op1_32);
 
@@ -642,9 +505,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::INC_EdM(bxInstruction_c *i)
 {
   Bit32u op1_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op1_32++;
   write_RMW_virtual_dword(op1_32);
 
@@ -655,9 +518,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::DEC_EdM(bxInstruction_c *i)
 {
   Bit32u op1_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   op1_32--;
   write_RMW_virtual_dword(op1_32);
 
@@ -669,9 +532,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPXCHG_EdGdM(bxInstruction_c *i)
 #if BX_CPU_LEVEL >= 4
   Bit32u op1_32, op2_32, diff_32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_32 = read_RMW_virtual_dword(i->seg(), RMAddr(i));
+  op1_32 = read_RMW_virtual_dword(i->seg(), eaddr);
   diff_32 = EAX - op1_32;
   SET_FLAGS_OSZAPC_SUB_32(EAX, op1_32, diff_32);
 
@@ -686,7 +549,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPXCHG_EdGdM(bxInstruction_c *i)
   }
 #else
   BX_INFO(("CMPXCHG_EdGd: not supported for cpulevel <= 3"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
@@ -710,7 +573,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPXCHG_EdGdR(bxInstruction_c *i)
   }
 #else
   BX_INFO(("CMPXCHG_EdGd: not supported for cpulevel <= 3"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
@@ -719,10 +582,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPXCHG8B(bxInstruction_c *i)
 #if BX_CPU_LEVEL >= 5
   Bit64u op1_64, op2_64;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
   // check write permission for following write
-  op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword(i->seg(), eaddr);
   op2_64 = ((Bit64u) EDX << 32) | EAX;
 
   if (op1_64 == op2_64) {  // if accumulator == dest
@@ -740,6 +603,6 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPXCHG8B(bxInstruction_c *i)
 
 #else
   BX_INFO(("CMPXCHG8B: not supported for cpulevel <= 4"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
