@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: arith64.cc,v 1.46 2007/12/23 17:21:27 sshwarts Exp $
+// $Id: arith64.cc,v 1.58 2009/01/16 18:18:58 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,24 +22,24 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA B 02110-1301 USA
 /////////////////////////////////////////////////////////////////////////
 
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-
 #if BX_SUPPORT_X86_64
 
-void BX_CPU_C::ADD_EqGqM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_EqGqM(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, sum_64;
 
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   /* pointer, segment address pair */
-  op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   op2_64 = BX_READ_64BIT_REG(i->nnn());
   sum_64 = op1_64 + op2_64;
   write_RMW_virtual_qword(sum_64);
@@ -47,31 +47,7 @@ void BX_CPU_C::ADD_EqGqM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_64(op1_64, op2_64, sum_64);
 }
 
-void BX_CPU_C::ADD_EqGqR(bxInstruction_c *i)
-{
-  Bit64u op1_64, op2_64, sum_64;
-
-  op1_64 = BX_READ_64BIT_REG(i->rm());
-  op2_64 = BX_READ_64BIT_REG(i->nnn());
-  sum_64 = op1_64 + op2_64;
-  BX_WRITE_64BIT_REG(i->rm(), sum_64);
-
-  SET_FLAGS_OSZAPC_ADD_64(op1_64, op2_64, sum_64);
-}
-
-void BX_CPU_C::ADD_GqEqM(bxInstruction_c *i)
-{
-  Bit64u op1_64, op2_64, sum_64;
-
-  op1_64 = BX_READ_64BIT_REG(i->nnn());
-  op2_64 = read_virtual_qword(i->seg(), RMAddr(i));
-  sum_64 = op1_64 + op2_64;
-  BX_WRITE_64BIT_REG(i->nnn(), sum_64);
-
-  SET_FLAGS_OSZAPC_ADD_64(op1_64, op2_64, sum_64);
-}
-
-void BX_CPU_C::ADD_GqEqR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_GqEqR(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, sum_64;
 
@@ -83,7 +59,7 @@ void BX_CPU_C::ADD_GqEqR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_64(op1_64, op2_64, sum_64);
 }
 
-void BX_CPU_C::ADD_RAXId(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_RAXId(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, sum_64;
 
@@ -97,52 +73,24 @@ void BX_CPU_C::ADD_RAXId(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_64(op1_64, op2_64, sum_64);
 }
 
-void BX_CPU_C::ADC_EqGqM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_EqGqM(bxInstruction_c *i)
 {
   bx_bool temp_CF = getB_CF();
 
   Bit64u op1_64, op2_64, sum_64;
 
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   /* pointer, segment address pair */
-  op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   op2_64 = BX_READ_64BIT_REG(i->nnn());
   sum_64 = op1_64 + op2_64 + temp_CF;
   write_RMW_virtual_qword(sum_64);
 
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64, BX_INSTR_ADD_ADC64(temp_CF));
+  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64, BX_LF_INSTR_ADD_ADC64(temp_CF));
 }
 
-void BX_CPU_C::ADC_EqGqR(bxInstruction_c *i)
-{
-  bx_bool temp_CF = getB_CF();
-
-  Bit64u op1_64, op2_64, sum_64;
-
-  op1_64 = BX_READ_64BIT_REG(i->rm());
-  op2_64 = BX_READ_64BIT_REG(i->nnn());
-  sum_64 = op1_64 + op2_64 + temp_CF;
-  BX_WRITE_64BIT_REG(i->rm(), sum_64);
-
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64, BX_INSTR_ADD_ADC64(temp_CF));
-}
-
-void BX_CPU_C::ADC_GqEqM(bxInstruction_c *i)
-{
-  bx_bool temp_CF = getB_CF();
-
-  Bit64u op1_64, op2_64, sum_64;
-
-  op1_64 = BX_READ_64BIT_REG(i->nnn());
-  op2_64 = read_virtual_qword(i->seg(), RMAddr(i));
-  sum_64 = op1_64 + op2_64 + temp_CF;
-
-  /* now write sum back to destination */
-  BX_WRITE_64BIT_REG(i->nnn(), sum_64);
-
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64, BX_INSTR_ADD_ADC64(temp_CF));
-}
-
-void BX_CPU_C::ADC_GqEqR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_GqEqR(bxInstruction_c *i)
 {
   bx_bool temp_CF = getB_CF();
 
@@ -155,10 +103,10 @@ void BX_CPU_C::ADC_GqEqR(bxInstruction_c *i)
   /* now write sum back to destination */
   BX_WRITE_64BIT_REG(i->nnn(), sum_64);
 
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64, BX_INSTR_ADD_ADC64(temp_CF));
+  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64, BX_LF_INSTR_ADD_ADC64(temp_CF));
 }
 
-void BX_CPU_C::ADC_RAXId(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_RAXId(bxInstruction_c *i)
 {
   bx_bool temp_CF = getB_CF();
 
@@ -171,55 +119,27 @@ void BX_CPU_C::ADC_RAXId(bxInstruction_c *i)
   /* now write sum back to destination */
   RAX = sum_64;
 
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64, BX_INSTR_ADD_ADC64(temp_CF));
+  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64, BX_LF_INSTR_ADD_ADC64(temp_CF));
 }
 
-void BX_CPU_C::SBB_EqGqM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_EqGqM(bxInstruction_c *i)
 {
   bx_bool temp_CF = getB_CF();
 
   Bit64u op1_64, op2_64, diff_64;
 
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   /* pointer, segment address pair */
-  op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   op2_64 = BX_READ_64BIT_REG(i->nnn());
   diff_64 = op1_64 - (op2_64 + temp_CF);
   write_RMW_virtual_qword(diff_64);
 
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_INSTR_SUB_SBB64(temp_CF));
+  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_LF_INSTR_SUB_SBB64(temp_CF));
 }
 
-void BX_CPU_C::SBB_EqGqR(bxInstruction_c *i)
-{
-  bx_bool temp_CF = getB_CF();
-
-  Bit64u op1_64, op2_64, diff_64;
-
-  op1_64 = BX_READ_64BIT_REG(i->rm());
-  op2_64 = BX_READ_64BIT_REG(i->nnn());
-  diff_64 = op1_64 - (op2_64 + temp_CF);
-  BX_WRITE_64BIT_REG(i->rm(), diff_64);
-
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_INSTR_SUB_SBB64(temp_CF));
-}
-
-void BX_CPU_C::SBB_GqEqM(bxInstruction_c *i)
-{
-  bx_bool temp_CF = getB_CF();
-
-  Bit64u op1_64, op2_64, diff_64;
-
-  op1_64 = BX_READ_64BIT_REG(i->nnn());
-  op2_64 = read_virtual_qword(i->seg(), RMAddr(i));
-  diff_64 = op1_64 - (op2_64 + temp_CF);
-
-  /* now write diff back to destination */
-  BX_WRITE_64BIT_REG(i->nnn(), diff_64);
-
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_INSTR_SUB_SBB64(temp_CF));
-}
-
-void BX_CPU_C::SBB_GqEqR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_GqEqR(bxInstruction_c *i)
 {
   bx_bool temp_CF = getB_CF();
 
@@ -232,10 +152,10 @@ void BX_CPU_C::SBB_GqEqR(bxInstruction_c *i)
   /* now write diff back to destination */
   BX_WRITE_64BIT_REG(i->nnn(), diff_64);
 
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_INSTR_SUB_SBB64(temp_CF));
+  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_LF_INSTR_SUB_SBB64(temp_CF));
 }
 
-void BX_CPU_C::SBB_RAXId(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_RAXId(bxInstruction_c *i)
 {
   bx_bool temp_CF = getB_CF();
 
@@ -248,25 +168,27 @@ void BX_CPU_C::SBB_RAXId(bxInstruction_c *i)
   /* now write diff back to destination */
   RAX = diff_64;
 
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_INSTR_SUB_SBB64(temp_CF));
+  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_LF_INSTR_SUB_SBB64(temp_CF));
 }
 
-void BX_CPU_C::SBB_EqIdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_EqIdM(bxInstruction_c *i)
 {
   bx_bool temp_CF = getB_CF();
 
   Bit64u op1_64, op2_64, diff_64;
 
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   /* pointer, segment address pair */
-  op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   op2_64 = (Bit32s) i->Id();
   diff_64 = op1_64 - (op2_64 + temp_CF);
   write_RMW_virtual_qword(diff_64);
 
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_INSTR_SUB_SBB64(temp_CF));
+  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_LF_INSTR_SUB_SBB64(temp_CF));
 }
 
-void BX_CPU_C::SBB_EqIdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_EqIdR(bxInstruction_c *i)
 {
   bx_bool temp_CF = getB_CF();
 
@@ -277,15 +199,17 @@ void BX_CPU_C::SBB_EqIdR(bxInstruction_c *i)
   diff_64 = op1_64 - (op2_64 + temp_CF);
   BX_WRITE_64BIT_REG(i->rm(), diff_64);
 
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_INSTR_SUB_SBB64(temp_CF));
+  SET_FLAGS_OSZAPC_64(op1_64, op2_64, diff_64, BX_LF_INSTR_SUB_SBB64(temp_CF));
 }
 
-void BX_CPU_C::SUB_EqGqM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_EqGqM(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, diff_64;
 
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   /* pointer, segment address pair */
-  op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   op2_64 = BX_READ_64BIT_REG(i->nnn());
   diff_64 = op1_64 - op2_64;
   write_RMW_virtual_qword(diff_64);
@@ -293,33 +217,7 @@ void BX_CPU_C::SUB_EqGqM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_64(op1_64, op2_64, diff_64);
 }
 
-void BX_CPU_C::SUB_EqGqR(bxInstruction_c *i)
-{
-  Bit64u op1_64, op2_64, diff_64;
-
-  op1_64 = BX_READ_64BIT_REG(i->rm());
-  op2_64 = BX_READ_64BIT_REG(i->nnn());
-  diff_64 = op1_64 - op2_64;
-  BX_WRITE_64BIT_REG(i->rm(), diff_64);
-
-  SET_FLAGS_OSZAPC_SUB_64(op1_64, op2_64, diff_64);
-}
-
-void BX_CPU_C::SUB_GqEqM(bxInstruction_c *i)
-{
-  Bit64u op1_64, op2_64, diff_64;
-
-  op1_64 = BX_READ_64BIT_REG(i->nnn());
-  op2_64 = read_virtual_qword(i->seg(), RMAddr(i));
-  diff_64 = op1_64 - op2_64;
-
-  /* now write diff back to destination */
-  BX_WRITE_64BIT_REG(i->nnn(), diff_64);
-
-  SET_FLAGS_OSZAPC_SUB_64(op1_64, op2_64, diff_64);
-}
-
-void BX_CPU_C::SUB_GqEqR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_GqEqR(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, diff_64;
 
@@ -333,7 +231,7 @@ void BX_CPU_C::SUB_GqEqR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_64(op1_64, op2_64, diff_64);
 }
 
-void BX_CPU_C::SUB_RAXId(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_RAXId(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, diff_64;
 
@@ -347,40 +245,20 @@ void BX_CPU_C::SUB_RAXId(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_64(op1_64, op2_64, diff_64);
 }
 
-void BX_CPU_C::CMP_EqGqM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_EqGqM(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, diff_64;
 
-  op1_64 = read_virtual_qword(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_64 = read_virtual_qword_64(i->seg(), eaddr);
   op2_64 = BX_READ_64BIT_REG(i->nnn());
   diff_64 = op1_64 - op2_64;
 
   SET_FLAGS_OSZAPC_SUB_64(op1_64, op2_64, diff_64);
 }
 
-void BX_CPU_C::CMP_EqGqR(bxInstruction_c *i)
-{
-  Bit64u op1_64, op2_64, diff_64;
-
-  op1_64 = BX_READ_64BIT_REG(i->rm());
-  op2_64 = BX_READ_64BIT_REG(i->nnn());
-  diff_64 = op1_64 - op2_64;
-
-  SET_FLAGS_OSZAPC_SUB_64(op1_64, op2_64, diff_64);
-}
-
-void BX_CPU_C::CMP_GqEqM(bxInstruction_c *i)
-{
-  Bit64u op1_64, op2_64, diff_64;
-
-  op1_64 = BX_READ_64BIT_REG(i->nnn());
-  op2_64 = read_virtual_qword(i->seg(), RMAddr(i));
-  diff_64 = op1_64 - op2_64;
-
-  SET_FLAGS_OSZAPC_SUB_64(op1_64, op2_64, diff_64);
-}
-
-void BX_CPU_C::CMP_GqEqR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_GqEqR(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, diff_64;
 
@@ -391,7 +269,7 @@ void BX_CPU_C::CMP_GqEqR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_64(op1_64, op2_64, diff_64);
 }
 
-void BX_CPU_C::CMP_RAXId(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_RAXId(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, diff_64;
 
@@ -402,13 +280,13 @@ void BX_CPU_C::CMP_RAXId(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_64(op1_64, op2_64, diff_64);
 }
 
-void BX_CPU_C::CDQE(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CDQE(bxInstruction_c *i)
 {
   /* CWDE: no flags are affected */
   RAX = (Bit32s) EAX;
 }
 
-void BX_CPU_C::CQO(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CQO(bxInstruction_c *i)
 {
   /* CQO: no flags are affected */
 
@@ -418,7 +296,7 @@ void BX_CPU_C::CQO(bxInstruction_c *i)
       RDX = 0;
 }
 
-void BX_CPU_C::XADD_EqGqM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XADD_EqGqM(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, sum_64;
 
@@ -428,8 +306,10 @@ void BX_CPU_C::XADD_EqGqM(bxInstruction_c *i)
    * dst  <-- tmp               | op1 = sum
    */
 
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   /* pointer, segment address pair */
-  op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   op2_64 = BX_READ_64BIT_REG(i->nnn());
   sum_64 = op1_64 + op2_64;
   write_RMW_virtual_qword(sum_64);
@@ -440,7 +320,7 @@ void BX_CPU_C::XADD_EqGqM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_64(op1_64, op2_64, sum_64);
 }
 
-void BX_CPU_C::XADD_EqGqR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XADD_EqGqR(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, sum_64;
 
@@ -464,12 +344,14 @@ void BX_CPU_C::XADD_EqGqR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_64(op1_64, op2_64, sum_64);
 }
 
-void BX_CPU_C::ADD_EqIdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_EqIdM(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, sum_64;
 
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   /* pointer, segment address pair */
-  op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   op2_64 = (Bit32s) i->Id();
   sum_64 = op1_64 + op2_64;
   write_RMW_virtual_qword(sum_64);
@@ -477,7 +359,7 @@ void BX_CPU_C::ADD_EqIdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_64(op1_64, op2_64, sum_64);
 }
 
-void BX_CPU_C::ADD_EqIdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_EqIdR(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, sum_64;
 
@@ -489,22 +371,24 @@ void BX_CPU_C::ADD_EqIdR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_64(op1_64, op2_64, sum_64);
 }
 
-void BX_CPU_C::ADC_EqIdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_EqIdM(bxInstruction_c *i)
 {
   bx_bool temp_CF = getB_CF();
 
   Bit64u op1_64, op2_64, sum_64;
 
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   /* pointer, segment address pair */
-  op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   op2_64 = (Bit32s) i->Id();
   sum_64 = op1_64 + op2_64 + temp_CF;
   write_RMW_virtual_qword(sum_64);
 
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64, BX_INSTR_ADD_ADC64(temp_CF));
+  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64, BX_LF_INSTR_ADD_ADC64(temp_CF));
 }
 
-void BX_CPU_C::ADC_EqIdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_EqIdR(bxInstruction_c *i)
 {
   bx_bool temp_CF = getB_CF();
 
@@ -515,15 +399,17 @@ void BX_CPU_C::ADC_EqIdR(bxInstruction_c *i)
   sum_64 = op1_64 + op2_64 + temp_CF;
   BX_WRITE_64BIT_REG(i->rm(), sum_64);
 
-  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64, BX_INSTR_ADD_ADC64(temp_CF));
+  SET_FLAGS_OSZAPC_64(op1_64, op2_64, sum_64, BX_LF_INSTR_ADD_ADC64(temp_CF));
 }
 
-void BX_CPU_C::SUB_EqIdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_EqIdM(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, diff_64;
 
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   /* pointer, segment address pair */
-  op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   op2_64 = (Bit32s) i->Id();
   diff_64 = op1_64 - op2_64;
   write_RMW_virtual_qword(diff_64);
@@ -531,7 +417,7 @@ void BX_CPU_C::SUB_EqIdM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_64(op1_64, op2_64, diff_64);
 }
 
-void BX_CPU_C::SUB_EqIdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_EqIdR(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, diff_64;
 
@@ -543,18 +429,20 @@ void BX_CPU_C::SUB_EqIdR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_64(op1_64, op2_64, diff_64);
 }
 
-void BX_CPU_C::CMP_EqIdM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_EqIdM(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, diff_64;
 
-  op1_64 = read_virtual_qword(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_64 = read_virtual_qword_64(i->seg(), eaddr);
   op2_64 = (Bit32s) i->Id();
   diff_64 = op1_64 - op2_64;
 
   SET_FLAGS_OSZAPC_SUB_64(op1_64, op2_64, diff_64);
 }
 
-void BX_CPU_C::CMP_EqIdR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_EqIdR(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, diff_64;
 
@@ -565,75 +453,77 @@ void BX_CPU_C::CMP_EqIdR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_64(op1_64, op2_64, diff_64);
 }
 
-void BX_CPU_C::NEG_EqM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::NEG_EqM(bxInstruction_c *i)
 {
   Bit64u op1_64;
 
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   /* pointer, segment address pair */
-  op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
-  op1_64 = (Bit64u) -op1_64;
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  op1_64 = - (Bit64s)(op1_64);
   write_RMW_virtual_qword(op1_64);
 
-  SET_FLAGS_OSZAPC_RESULT_64(op1_64, BX_INSTR_NEG64);
+  SET_FLAGS_OSZAPC_RESULT_64(op1_64, BX_LF_INSTR_NEG64);
 }
 
-void BX_CPU_C::NEG_EqR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::NEG_EqR(bxInstruction_c *i)
 {
   Bit64u op1_64 = BX_READ_64BIT_REG(i->rm());
-  op1_64 = (Bit64u) -op1_64;
+  op1_64 = - (Bit64s)(op1_64);
   BX_WRITE_64BIT_REG(i->rm(), op1_64);
 
-  SET_FLAGS_OSZAPC_RESULT_64(op1_64, BX_INSTR_NEG64);
+  SET_FLAGS_OSZAPC_RESULT_64(op1_64, BX_LF_INSTR_NEG64);
 }
 
-void BX_CPU_C::INC_EqM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::INC_EqM(bxInstruction_c *i)
 {
   Bit64u op1_64;
 
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   /* pointer, segment address pair */
-  op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   op1_64++;
   write_RMW_virtual_qword(op1_64);
 
   SET_FLAGS_OSZAPC_INC_64(op1_64);
 }
 
-void BX_CPU_C::INC_EqR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::INC_EqR(bxInstruction_c *i)
 {
-  Bit64u op1_64 = BX_READ_64BIT_REG(i->rm());
-  op1_64++;
-  BX_WRITE_64BIT_REG(i->rm(), op1_64);
-
-  SET_FLAGS_OSZAPC_INC_64(op1_64);
+  Bit64u rrx = ++BX_READ_64BIT_REG(i->rm());
+  SET_FLAGS_OSZAPC_INC_64(rrx);
 }
 
-void BX_CPU_C::DEC_EqM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::DEC_EqM(bxInstruction_c *i)
 {
   Bit64u op1_64;
 
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   /* pointer, segment address pair */
-  op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   op1_64--;
   write_RMW_virtual_qword(op1_64);
 
   SET_FLAGS_OSZAPC_DEC_64(op1_64);
 }
 
-void BX_CPU_C::DEC_EqR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::DEC_EqR(bxInstruction_c *i)
 {
-  Bit64u op1_64 = BX_READ_64BIT_REG(i->rm());
-  op1_64--;
-  BX_WRITE_64BIT_REG(i->rm(), op1_64);
-
-  SET_FLAGS_OSZAPC_DEC_64(op1_64);
+  Bit64u rrx = --BX_READ_64BIT_REG(i->rm());
+  SET_FLAGS_OSZAPC_INC_64(rrx);
 }
 
-void BX_CPU_C::CMPXCHG_EqGqM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPXCHG_EqGqM(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, diff_64;
 
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
   /* pointer, segment address pair */
-  op1_64 = read_RMW_virtual_qword(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   diff_64 = RAX - op1_64;
   SET_FLAGS_OSZAPC_SUB_64(RAX, op1_64, diff_64);
 
@@ -648,7 +538,7 @@ void BX_CPU_C::CMPXCHG_EqGqM(bxInstruction_c *i)
   }
 }
 
-void BX_CPU_C::CMPXCHG_EqGqR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPXCHG_EqGqR(bxInstruction_c *i)
 {
   Bit64u op1_64, op2_64, diff_64;
 
@@ -667,27 +557,31 @@ void BX_CPU_C::CMPXCHG_EqGqR(bxInstruction_c *i)
   }
 }
 
-void BX_CPU_C::CMPXCHG16B(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPXCHG16B(bxInstruction_c *i)
 {
   Bit64u op1_64_lo, op1_64_hi, diff;
 
-  if (RMAddr(i) & 0xf) {
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  bx_address laddr = get_laddr64(i->seg(), eaddr);
+
+  if (laddr & 0xf) {
     BX_ERROR(("CMPXCHG16B: not aligned memory location (#GP)"));
     exception(BX_GP_EXCEPTION, 0, 0);
   }
 
   // check write permission for following write
-  op1_64_lo = read_RMW_virtual_qword(i->seg(), RMAddr(i));
-  op1_64_hi = read_RMW_virtual_qword(i->seg(), RMAddr(i) + 8);
+  op1_64_lo = read_RMW_virtual_qword_64(i->seg(), eaddr);
+  op1_64_hi = read_RMW_virtual_qword_64(i->seg(), eaddr + 8);
 
   diff  = RAX - op1_64_lo;
   diff |= RDX - op1_64_hi;
 
   if (diff == 0) {  // if accumulator == dest
-    // dest <-- src
+    // dest <-- src (RCX:RBX)
     write_RMW_virtual_qword(RCX);
-    // write permissions already checked by read_RMW_virtual_qword
-    write_virtual_qword(i->seg(), RMAddr(i), RBX);
+    // write permissions already checked by read_RMW_virtual_qword_64
+    write_virtual_qword_64(i->seg(), eaddr, RBX);
     assert_ZF();
   }
   else {
