@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: arith8.cc,v 1.51 2007/12/20 20:58:37 sshwarts Exp $
+// $Id: arith8.cc,v 1.61 2009/01/16 18:18:58 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,21 +22,21 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA B 02110-1301 USA
 /////////////////////////////////////////////////////////////////////////
 
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-
-void BX_CPU_C::ADD_EbGbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_EbGbM(bxInstruction_c *i)
 {
   Bit8u op1, op2, sum;
 
-  op1 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1 = read_RMW_virtual_byte(i->seg(), eaddr);
   op2 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
   sum = op1 + op2;
   write_RMW_virtual_byte(sum);
@@ -44,31 +44,7 @@ void BX_CPU_C::ADD_EbGbM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_8(op1, op2, sum);
 }
 
-void BX_CPU_C::ADD_EbGbR(bxInstruction_c *i)
-{
-  Bit8u op1, op2, sum;
-
-  op1 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
-  op2 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
-  sum = op1 + op2;
-  BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), sum);
-
-  SET_FLAGS_OSZAPC_ADD_8(op1, op2, sum);
-}
-
-void BX_CPU_C::ADD_GbEbM(bxInstruction_c *i)
-{
-  Bit8u op1, op2, sum;
-
-  op1 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
-  op2 = read_virtual_byte(i->seg(), RMAddr(i));
-  sum = op1 + op2;
-  BX_WRITE_8BIT_REGx(i->nnn(), i->extend8bitL(), sum);
-
-  SET_FLAGS_OSZAPC_ADD_8(op1, op2, sum);
-}
-
-void BX_CPU_C::ADD_GbEbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_GbEbR(bxInstruction_c *i)
 {
   Bit8u op1, op2, sum;
 
@@ -80,7 +56,7 @@ void BX_CPU_C::ADD_GbEbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_8(op1, op2, sum);
 }
 
-void BX_CPU_C::ADD_ALIb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_ALIb(bxInstruction_c *i)
 {
   Bit8u op1, op2, sum;
 
@@ -92,46 +68,22 @@ void BX_CPU_C::ADD_ALIb(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_8(op1, op2, sum);
 }
 
-void BX_CPU_C::ADC_EbGbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_EbGbM(bxInstruction_c *i)
 {
   Bit8u op1, op2, sum;
   bx_bool temp_CF = getB_CF();
 
-  op1 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1 = read_RMW_virtual_byte(i->seg(), eaddr);
   op2 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
   sum = op1 + op2 + temp_CF;
   write_RMW_virtual_byte(sum);
 
-  SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_INSTR_ADD_ADC8(temp_CF));
+  SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_LF_INSTR_ADD_ADC8(temp_CF));
 }
 
-void BX_CPU_C::ADC_EbGbR(bxInstruction_c *i)
-{
-  Bit8u op1, op2, sum;
-  bx_bool temp_CF = getB_CF();
-
-  op1 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
-  op2 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
-  sum = op1 + op2 + temp_CF;
-  BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), sum);
-
-  SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_INSTR_ADD_ADC8(temp_CF));
-}
-
-void BX_CPU_C::ADC_GbEbM(bxInstruction_c *i)
-{
-  Bit8u op1, op2, sum;
-  bx_bool temp_CF = getB_CF();
-
-  op1 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
-  op2 = read_virtual_byte(i->seg(), RMAddr(i));
-  sum = op1 + op2 + temp_CF;
-  BX_WRITE_8BIT_REGx(i->nnn(), i->extend8bitL(), sum);
-
-  SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_INSTR_ADD_ADC8(temp_CF));
-}
-
-void BX_CPU_C::ADC_GbEbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_GbEbR(bxInstruction_c *i)
 {
   Bit8u op1, op2, sum;
   bx_bool temp_CF = getB_CF();
@@ -141,10 +93,10 @@ void BX_CPU_C::ADC_GbEbR(bxInstruction_c *i)
   sum = op1 + op2 + temp_CF;
   BX_WRITE_8BIT_REGx(i->nnn(), i->extend8bitL(), sum);
 
-  SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_INSTR_ADD_ADC8(temp_CF));
+  SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_LF_INSTR_ADD_ADC8(temp_CF));
 }
 
-void BX_CPU_C::ADC_ALIb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_ALIb(bxInstruction_c *i)
 {
   Bit8u op1, op2, sum;
   bx_bool temp_CF = getB_CF();
@@ -154,49 +106,25 @@ void BX_CPU_C::ADC_ALIb(bxInstruction_c *i)
   sum = op1 + op2 + temp_CF;
   AL = sum;
 
-  SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_INSTR_ADD_ADC8(temp_CF));
+  SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_LF_INSTR_ADD_ADC8(temp_CF));
 }
 
-void BX_CPU_C::SBB_EbGbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_EbGbM(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8, diff_8;
   bx_bool temp_CF = getB_CF();
 
-  op1_8 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
   op2_8 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
   diff_8 = op1_8 - (op2_8 + temp_CF);
   write_RMW_virtual_byte(diff_8);
 
-  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_SUB_SBB8(temp_CF));
+  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_LF_INSTR_SUB_SBB8(temp_CF));
 }
 
-void BX_CPU_C::SBB_EbGbR(bxInstruction_c *i)
-{
-  Bit8u op1_8, op2_8, diff_8;
-  bx_bool temp_CF = getB_CF();
-
-  op1_8 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
-  op2_8 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
-  diff_8 = op1_8 - (op2_8 + temp_CF);
-  BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), diff_8);
-
-  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_SUB_SBB8(temp_CF));
-}
-
-void BX_CPU_C::SBB_GbEbM(bxInstruction_c *i)
-{
-  Bit8u op1_8, op2_8, diff_8;
-  bx_bool temp_CF = getB_CF();
-
-  op1_8 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
-  op2_8 = read_virtual_byte(i->seg(), RMAddr(i));
-  diff_8 = op1_8 - (op2_8 + temp_CF);
-  BX_WRITE_8BIT_REGx(i->nnn(), i->extend8bitL(), diff_8);
-
-  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_SUB_SBB8(temp_CF));
-}
-
-void BX_CPU_C::SBB_GbEbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_GbEbR(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8, diff_8;
   bx_bool temp_CF = getB_CF();
@@ -206,10 +134,10 @@ void BX_CPU_C::SBB_GbEbR(bxInstruction_c *i)
   diff_8 = op1_8 - (op2_8 + temp_CF);
   BX_WRITE_8BIT_REGx(i->nnn(), i->extend8bitL(), diff_8);
 
-  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_SUB_SBB8(temp_CF));
+  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_LF_INSTR_SUB_SBB8(temp_CF));
 }
 
-void BX_CPU_C::SBB_ALIb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_ALIb(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8, diff_8;
   bx_bool temp_CF = getB_CF();
@@ -219,22 +147,24 @@ void BX_CPU_C::SBB_ALIb(bxInstruction_c *i)
   diff_8 = op1_8 - (op2_8 + temp_CF);
   AL = diff_8;
 
-  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_SUB_SBB8(temp_CF));
+  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_LF_INSTR_SUB_SBB8(temp_CF));
 }
 
-void BX_CPU_C::SBB_EbIbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_EbIbM(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8 = i->Ib(), diff_8;
   bx_bool temp_CF = getB_CF();
 
-  op1_8 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
   diff_8 = op1_8 - (op2_8 + temp_CF);
   write_RMW_virtual_byte(diff_8);
 
-  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_SUB_SBB8(temp_CF));
+  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_LF_INSTR_SUB_SBB8(temp_CF));
 }
 
-void BX_CPU_C::SBB_EbIbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SBB_EbIbR(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8 = i->Ib(), diff_8;
   bx_bool temp_CF = getB_CF();
@@ -243,14 +173,16 @@ void BX_CPU_C::SBB_EbIbR(bxInstruction_c *i)
   diff_8 = op1_8 - (op2_8 + temp_CF);
   BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), diff_8);
 
-  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_INSTR_SUB_SBB8(temp_CF));
+  SET_FLAGS_OSZAPC_8(op1_8, op2_8, diff_8, BX_LF_INSTR_SUB_SBB8(temp_CF));
 }
 
-void BX_CPU_C::SUB_EbGbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_EbGbM(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8, diff_8;
 
-  op1_8 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
   op2_8 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
   diff_8 = op1_8 - op2_8;
   write_RMW_virtual_byte(diff_8);
@@ -258,31 +190,7 @@ void BX_CPU_C::SUB_EbGbM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_8(op1_8, op2_8, diff_8);
 }
 
-void BX_CPU_C::SUB_EbGbR(bxInstruction_c *i)
-{
-  Bit8u op1_8, op2_8, diff_8;
-
-  op1_8 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
-  op2_8 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
-  diff_8 = op1_8 - op2_8;
-  BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), diff_8);
-
-  SET_FLAGS_OSZAPC_SUB_8(op1_8, op2_8, diff_8);
-}
-
-void BX_CPU_C::SUB_GbEbM(bxInstruction_c *i)
-{
-  Bit8u op1_8, op2_8, diff_8;
-
-  op1_8 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
-  op2_8 = read_virtual_byte(i->seg(), RMAddr(i));
-  diff_8 = op1_8 - op2_8;
-  BX_WRITE_8BIT_REGx(i->nnn(), i->extend8bitL(), diff_8);
-
-  SET_FLAGS_OSZAPC_SUB_8(op1_8, op2_8, diff_8);
-}
-
-void BX_CPU_C::SUB_GbEbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_GbEbR(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8, diff_8;
 
@@ -294,7 +202,7 @@ void BX_CPU_C::SUB_GbEbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_8(op1_8, op2_8, diff_8);
 }
 
-void BX_CPU_C::SUB_ALIb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_ALIb(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8, diff_8;
 
@@ -306,40 +214,20 @@ void BX_CPU_C::SUB_ALIb(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_8(op1_8, op2_8, diff_8);
 }
 
-void BX_CPU_C::CMP_EbGbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_EbGbM(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8, diff_8;
 
-  op1_8 = read_virtual_byte(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_8 = read_virtual_byte(i->seg(), eaddr);
   op2_8 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
   diff_8 = op1_8 - op2_8;
 
   SET_FLAGS_OSZAPC_SUB_8(op1_8, op2_8, diff_8);
 }
 
-void BX_CPU_C::CMP_EbGbR(bxInstruction_c *i)
-{
-  Bit8u op1_8, op2_8, diff_8;
-
-  op1_8 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
-  op2_8 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
-  diff_8 = op1_8 - op2_8;
-
-  SET_FLAGS_OSZAPC_SUB_8(op1_8, op2_8, diff_8);
-}
-
-void BX_CPU_C::CMP_GbEbM(bxInstruction_c *i)
-{
-  Bit8u op1_8, op2_8, diff_8;
-
-  op1_8 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
-  op2_8 = read_virtual_byte(i->seg(), RMAddr(i));
-  diff_8 = op1_8 - op2_8;
-
-  SET_FLAGS_OSZAPC_SUB_8(op1_8, op2_8, diff_8);
-}
-
-void BX_CPU_C::CMP_GbEbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_GbEbR(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8, diff_8;
 
@@ -350,7 +238,7 @@ void BX_CPU_C::CMP_GbEbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_8(op1_8, op2_8, diff_8);
 }
 
-void BX_CPU_C::CMP_ALIb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_ALIb(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8, diff_8;
 
@@ -361,7 +249,7 @@ void BX_CPU_C::CMP_ALIb(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_8(op1_8, op2_8, diff_8);
 }
 
-void BX_CPU_C::XADD_EbGbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XADD_EbGbM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 4
   Bit8u op1, op2, sum;
@@ -372,7 +260,9 @@ void BX_CPU_C::XADD_EbGbM(bxInstruction_c *i)
    * dst  <-- tmp               | op1 = sum
    */
 
-  op1 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1 = read_RMW_virtual_byte(i->seg(), eaddr);
   op2 = BX_READ_8BIT_REGx(i->nnn(), i->extend8bitL());
   sum = op1 + op2;
   write_RMW_virtual_byte(sum);
@@ -383,11 +273,11 @@ void BX_CPU_C::XADD_EbGbM(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_8(op1, op2, sum);
 #else
   BX_INFO(("XADD_EbGb: not supported on < 80486"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
-void BX_CPU_C::XADD_EbGbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::XADD_EbGbR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 4
   Bit8u op1, op2, sum;
@@ -412,22 +302,24 @@ void BX_CPU_C::XADD_EbGbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_8(op1, op2, sum);
 #else
   BX_INFO(("XADD_EbGb: not supported on < 80486"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
-void BX_CPU_C::ADD_EbIbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_EbIbM(bxInstruction_c *i)
 {
   Bit8u op1, op2 = i->Ib(), sum;
 
-  op1 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1 = read_RMW_virtual_byte(i->seg(), eaddr);
   sum = op1 + op2;
   write_RMW_virtual_byte(sum);
 
   SET_FLAGS_OSZAPC_ADD_8(op1, op2, sum);
 }
 
-void BX_CPU_C::ADD_EbIbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADD_EbIbR(bxInstruction_c *i)
 {
   Bit8u op1, op2 = i->Ib(), sum;
 
@@ -438,19 +330,21 @@ void BX_CPU_C::ADD_EbIbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_ADD_8(op1, op2, sum);
 }
 
-void BX_CPU_C::ADC_EbIbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_EbIbM(bxInstruction_c *i)
 {
   Bit8u op1, op2 = i->Ib(), sum;
   bx_bool temp_CF = getB_CF();
 
-  op1 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1 = read_RMW_virtual_byte(i->seg(), eaddr);
   sum = op1 + op2 + temp_CF;
   write_RMW_virtual_byte(sum);
 
-  SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_INSTR_ADD_ADC8(temp_CF));
+  SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_LF_INSTR_ADD_ADC8(temp_CF));
 }
 
-void BX_CPU_C::ADC_EbIbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::ADC_EbIbR(bxInstruction_c *i)
 {
   Bit8u op1, op2 = i->Ib(), sum;
   bx_bool temp_CF = getB_CF();
@@ -459,21 +353,23 @@ void BX_CPU_C::ADC_EbIbR(bxInstruction_c *i)
   sum = op1 + op2 + temp_CF;
   BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), sum);
 
-  SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_INSTR_ADD_ADC8(temp_CF));
+  SET_FLAGS_OSZAPC_8(op1, op2, sum, BX_LF_INSTR_ADD_ADC8(temp_CF));
 }
 
-void BX_CPU_C::SUB_EbIbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_EbIbM(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8 = i->Ib(), diff_8;
 
-  op1_8 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
   diff_8 = op1_8 - op2_8;
   write_RMW_virtual_byte(diff_8);
 
   SET_FLAGS_OSZAPC_SUB_8(op1_8, op2_8, diff_8);
 }
 
-void BX_CPU_C::SUB_EbIbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::SUB_EbIbR(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8 = i->Ib(), diff_8;
 
@@ -484,18 +380,20 @@ void BX_CPU_C::SUB_EbIbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_8(op1_8, op2_8, diff_8);
 }
 
-void BX_CPU_C::CMP_EbIbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_EbIbM(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8 = i->Ib(), diff_8;
 
-  op1_8 = read_virtual_byte(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_8 = read_virtual_byte(i->seg(), eaddr);
 
   diff_8 = op1_8 - op2_8;
 
   SET_FLAGS_OSZAPC_SUB_8(op1_8, op2_8, diff_8);
 }
 
-void BX_CPU_C::CMP_EbIbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMP_EbIbR(bxInstruction_c *i)
 {
   Bit8u op1_8, op2_8 = i->Ib(), diff_8;
 
@@ -504,38 +402,42 @@ void BX_CPU_C::CMP_EbIbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_SUB_8(op1_8, op2_8, diff_8);
 }
 
-void BX_CPU_C::NEG_EbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::NEG_EbM(bxInstruction_c *i)
 {
   Bit8u op1_8;
 
-  op1_8 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
-  op1_8 = -op1_8;
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
+  op1_8 = - (Bit8s)(op1_8);
   write_RMW_virtual_byte(op1_8);
 
-  SET_FLAGS_OSZAPC_RESULT_8(op1_8, BX_INSTR_NEG8);
+  SET_FLAGS_OSZAPC_RESULT_8(op1_8, BX_LF_INSTR_NEG8);
 }
 
-void BX_CPU_C::NEG_EbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::NEG_EbR(bxInstruction_c *i)
 {
   Bit8u op1_8 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
-  op1_8 = -op1_8;
+  op1_8 = - (Bit8s)(op1_8);
   BX_WRITE_8BIT_REGx(i->rm(), i->extend8bitL(), op1_8);
 
-  SET_FLAGS_OSZAPC_RESULT_8(op1_8, BX_INSTR_NEG8);
+  SET_FLAGS_OSZAPC_RESULT_8(op1_8, BX_LF_INSTR_NEG8);
 }
 
-void BX_CPU_C::INC_EbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::INC_EbM(bxInstruction_c *i)
 {
   Bit8u op1_8;
 
-  op1_8 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
   op1_8++;
   write_RMW_virtual_byte(op1_8);
 
   SET_FLAGS_OSZAPC_INC_8(op1_8);
 }
 
-void BX_CPU_C::INC_EbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::INC_EbR(bxInstruction_c *i)
 {
   Bit8u op1_8 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
   op1_8++;
@@ -544,18 +446,20 @@ void BX_CPU_C::INC_EbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_INC_8(op1_8);
 }
 
-void BX_CPU_C::DEC_EbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::DEC_EbM(bxInstruction_c *i)
 {
   Bit8u op1_8;
 
-  op1_8 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
   op1_8--;
   write_RMW_virtual_byte(op1_8);
 
   SET_FLAGS_OSZAPC_DEC_8(op1_8);
 }
 
-void BX_CPU_C::DEC_EbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::DEC_EbR(bxInstruction_c *i)
 {
   Bit8u op1_8 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
   op1_8--;
@@ -564,12 +468,14 @@ void BX_CPU_C::DEC_EbR(bxInstruction_c *i)
   SET_FLAGS_OSZAPC_DEC_8(op1_8);
 }
 
-void BX_CPU_C::CMPXCHG_EbGbM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPXCHG_EbGbM(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 4
   Bit8u op1_8, op2_8, diff_8;
 
-  op1_8 = read_RMW_virtual_byte(i->seg(), RMAddr(i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  op1_8 = read_RMW_virtual_byte(i->seg(), eaddr);
   diff_8 = AL - op1_8;
   SET_FLAGS_OSZAPC_SUB_8(AL, op1_8, diff_8);
 
@@ -585,11 +491,11 @@ void BX_CPU_C::CMPXCHG_EbGbM(bxInstruction_c *i)
 
 #else
   BX_INFO(("CMPXCHG_EbGb: not supported for cpulevel <= 3"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
-void BX_CPU_C::CMPXCHG_EbGbR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMPXCHG_EbGbR(bxInstruction_c *i)
 {
 #if BX_CPU_LEVEL >= 4
   Bit8u op1_8, op2_8, diff_8;
@@ -610,6 +516,6 @@ void BX_CPU_C::CMPXCHG_EbGbR(bxInstruction_c *i)
 
 #else
   BX_INFO(("CMPXCHG_EbGb: not supported for cpulevel <= 3"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
