@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: bit16.cc,v 1.9 2008/04/25 07:40:50 sshwarts Exp $
+// $Id: bit16.cc,v 1.16 2009/01/16 18:18:58 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,7 +22,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA B 02110-1301 USA
 /////////////////////////////////////////////////////////////////////////
 
 #define NEED_CPU_REG_SHORTCUTS 1
@@ -32,70 +32,48 @@
 
 #if BX_CPU_LEVEL >= 3
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BSF_GwEw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BSF_GwEwR(bxInstruction_c *i)
 {
-  Bit16u op1_16, op2_16;
-
-  /* op2_16 is a register or memory reference */
-  if (i->modC0()) {
-    op2_16 = BX_READ_16BIT_REG(i->rm());
-  }
-  else {
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-    /* pointer, segment address pair */
-    op2_16 = read_virtual_word(i->seg(), RMAddr(i));
-  }
+  Bit16u op2_16 = BX_READ_16BIT_REG(i->rm());
 
   if (op2_16 == 0) {
     assert_ZF(); /* op1_16 undefined */
-    return;
   }
+  else {
+    Bit16u op1_16 = 0;
+    while ((op2_16 & 0x01) == 0) {
+      op1_16++;
+      op2_16 >>= 1;
+    }
 
-  op1_16 = 0;
-  while ((op2_16 & 0x01) == 0) {
-    op1_16++;
-    op2_16 >>= 1;
+    SET_FLAGS_OSZAPC_LOGIC_16(op1_16);
+    clear_ZF();
+
+    /* now write result back to destination */
+    BX_WRITE_16BIT_REG(i->nnn(), op1_16);
   }
-
-  SET_FLAGS_OSZAPC_LOGIC_16(op1_16);
-  clear_ZF();
-
-  /* now write result back to destination */
-  BX_WRITE_16BIT_REG(i->nnn(), op1_16);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BSR_GwEw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BSR_GwEwR(bxInstruction_c *i)
 {
-  Bit16u op1_16, op2_16;
-
-  /* op2_16 is a register or memory reference */
-  if (i->modC0()) {
-    op2_16 = BX_READ_16BIT_REG(i->rm());
-  }
-  else {
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-    /* pointer, segment address pair */
-    op2_16 = read_virtual_word(i->seg(), RMAddr(i));
-  }
+  Bit16u op2_16 = BX_READ_16BIT_REG(i->rm());
 
   if (op2_16 == 0) {
     assert_ZF(); /* op1_16 undefined */
-    return;
   }
+  else {
+    Bit16u op1_16 = 15;
+    while ((op2_16 & 0x8000) == 0) {
+      op1_16--;
+      op2_16 <<= 1;
+    }
 
-  op1_16 = 15;
-  while ((op2_16 & 0x8000) == 0) {
-    op1_16--;
-    op2_16 <<= 1;
+    SET_FLAGS_OSZAPC_LOGIC_16(op1_16);
+    clear_ZF();
+
+    /* now write result back to destination */
+    BX_WRITE_16BIT_REG(i->nnn(), op1_16);
   }
-
-  SET_FLAGS_OSZAPC_LOGIC_16(op1_16);
-  clear_ZF();
-
-  /* now write result back to destination */
-  BX_WRITE_16BIT_REG(i->nnn(), op1_16);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EwGwM(bxInstruction_c *i)
@@ -104,12 +82,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EwGwM(bxInstruction_c *i)
   Bit16u op1_16, op2_16, index;
   Bit32s displacement32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
   op2_16 = BX_READ_16BIT_REG(i->nnn());
   index = op2_16 & 0x0f;
   displacement32 = ((Bit16s) (op2_16&0xfff0)) / 16;
-  op1_addr = RMAddr(i) + 2 * displacement32;
+  op1_addr = eaddr + 2 * displacement32;
   if (! i->as32L())
     op1_addr = (Bit16u) op1_addr;
 #if BX_SUPPORT_X86_64
@@ -140,12 +118,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EwGwM(bxInstruction_c *i)
   Bit32s displacement32;
   bx_bool bit_i;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
   op2_16 = BX_READ_16BIT_REG(i->nnn());
   index = op2_16 & 0x0f;
   displacement32 = ((Bit16s) (op2_16 & 0xfff0)) / 16;
-  op1_addr = RMAddr(i) + 2 * displacement32;
+  op1_addr = eaddr + 2 * displacement32;
   if (! i->as32L())
     op1_addr = (Bit16u) op1_addr;
 #if BX_SUPPORT_X86_64
@@ -182,12 +160,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EwGwM(bxInstruction_c *i)
   Bit16u op1_16, op2_16, index;
   Bit32s displacement32;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
   op2_16 = BX_READ_16BIT_REG(i->nnn());
   index = op2_16 & 0x0f;
   displacement32 = ((Bit16s) (op2_16&0xfff0)) / 16;
-  op1_addr = RMAddr(i) + 2 * displacement32;
+  op1_addr = eaddr + 2 * displacement32;
   if (! i->as32L())
     op1_addr = (Bit16u) op1_addr;
 #if BX_SUPPORT_X86_64
@@ -226,12 +204,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EwGwM(bxInstruction_c *i)
   Bit16u op1_16, op2_16, index_16;
   Bit16s displacement16;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
   op2_16 = BX_READ_16BIT_REG(i->nnn());
   index_16 = op2_16 & 0x0f;
   displacement16 = ((Bit16s) (op2_16 & 0xfff0)) / 16;
-  op1_addr = RMAddr(i) + 2 * displacement16;
+  op1_addr = eaddr + 2 * displacement16;
   if (! i->as32L())
     op1_addr = (Bit16u) op1_addr;
 #if BX_SUPPORT_X86_64
@@ -264,9 +242,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EwGwR(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EwIbM(bxInstruction_c *i)
 {
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit16u op1_16 = read_virtual_word(i->seg(), RMAddr(i));
+  Bit16u op1_16 = read_virtual_word(i->seg(), eaddr);
   Bit8u  op2_8  = i->Ib() & 0xf;
 
   set_CF((op1_16 >> op2_8) & 0x01);
@@ -284,9 +262,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EwIbM(bxInstruction_c *i)
 {
   Bit8u op2_8 = i->Ib() & 0xf;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit16u op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
+  Bit16u op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   bx_bool temp_CF = (op1_16 >> op2_8) & 0x01;
   op1_16 |= (((Bit16u) 1) << op2_8);
   write_RMW_virtual_word(op1_16);
@@ -310,9 +288,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EwIbM(bxInstruction_c *i)
 {
   Bit8u op2_8 = i->Ib() & 0xf;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit16u op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
+  Bit16u op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   bx_bool temp_CF = (op1_16 >> op2_8) & 0x01;
   op1_16 ^= (((Bit16u) 1) << op2_8);  /* toggle bit */
   write_RMW_virtual_word(op1_16);
@@ -336,9 +314,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EwIbM(bxInstruction_c *i)
 {
   Bit8u op2_8 = i->Ib() & 0xf;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit16u op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
+  Bit16u op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   bx_bool temp_CF = (op1_16 >> op2_8) & 0x01;
   op1_16 &= ~(((Bit16u) 1) << op2_8);
   write_RMW_virtual_word(op1_16);
@@ -359,22 +337,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EwIbR(bxInstruction_c *i)
 }
 
 /* 0F B8 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::POPCNT_GwEw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::POPCNT_GwEwR(bxInstruction_c *i)
 {
-#if BX_SUPPORT_POPCNT || (BX_SUPPORT_SSE >= 5) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
-  Bit16u op1_16, op2_16;
+#if BX_SUPPORT_POPCNT || (BX_SUPPORT_SSE > 4) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
+  Bit16u op2_16 = BX_READ_16BIT_REG(i->rm());
 
-  /* op2_16 is a register or memory reference */
-  if (i->modC0()) {
-    op2_16 = BX_READ_16BIT_REG(i->rm());
-  }
-  else {
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-    /* pointer, segment address pair */
-    op2_16 = read_virtual_word(i->seg(), RMAddr(i));
-  }
-
-  op1_16 = 0;
+  Bit16u op1_16 = 0;
   while (op2_16 != 0) {
     if (op2_16 & 1) op1_16++;
     op2_16 >>= 1;
@@ -387,7 +355,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::POPCNT_GwEw(bxInstruction_c *i)
   BX_WRITE_16BIT_REG(i->nnn(), op1_16);
 #else
   BX_INFO(("POPCNT_GwEw: required POPCNT support, use --enable-popcnt option"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
