@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: bit64.cc,v 1.12 2008/05/10 18:10:52 sshwarts Exp $
+// $Id: bit64.cc,v 1.20 2009/01/16 18:18:58 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,7 +22,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA B 02110-1301 USA
 /////////////////////////////////////////////////////////////////////////
 
 #define NEED_CPU_REG_SHORTCUTS 1
@@ -32,70 +32,48 @@
 
 #if BX_SUPPORT_X86_64
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BSF_GqEq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BSF_GqEqR(bxInstruction_c *i)
 {
-  /* for 64 bit operand size mode */
-  Bit64u op1_64, op2_64;
-
-  /* op2_64 is a register or memory reference */
-  if (i->modC0()) {
-    op2_64 = BX_READ_64BIT_REG(i->rm());
-  }
-  else {
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-    /* pointer, segment address pair */
-    op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-  }
+  Bit64u op2_64 = BX_READ_64BIT_REG(i->rm());
 
   if (op2_64 == 0) {
     assert_ZF(); /* op1_64 undefined */
-    return;
   }
+  else {
+    Bit64u op1_64 = 0;
+    while ((op2_64 & 0x01) == 0) {
+      op1_64++;
+      op2_64 >>= 1;
+    }
 
-  op1_64 = 0;
-  while ((op2_64 & 0x01) == 0) {
-    op1_64++;
-    op2_64 >>= 1;
+    SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
+    clear_ZF();
+
+    /* now write result back to destination */
+    BX_WRITE_64BIT_REG(i->nnn(), op1_64);
   }
-
-  SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
-  clear_ZF();
-
-  /* now write result back to destination */
-  BX_WRITE_64BIT_REG(i->nnn(), op1_64);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::BSR_GqEq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::BSR_GqEqR(bxInstruction_c *i)
 {
-  /* for 64 bit operand size mode */
-  Bit64u op1_64, op2_64;
-
-  /* op2_64 is a register or memory reference */
-  if (i->modC0()) {
-    op2_64 = BX_READ_64BIT_REG(i->rm());
-  }
-  else {
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-    /* pointer, segment address pair */
-    op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-  }
+  Bit64u op2_64 = BX_READ_64BIT_REG(i->rm());
 
   if (op2_64 == 0) {
     assert_ZF(); /* op1_64 undefined */
-    return;
   }
+  else {
+    Bit64u op1_64 = 63;
+    while ((op2_64 & BX_CONST64(0x8000000000000000)) == 0) {
+      op1_64--;
+      op2_64 <<= 1;
+    }
 
-  op1_64 = 63;
-  while ((op2_64 & BX_CONST64(0x8000000000000000)) == 0) {
-    op1_64--;
-    op2_64 <<= 1;
+    SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
+    clear_ZF();
+
+    /* now write result back to destination */
+    BX_WRITE_64BIT_REG(i->nnn(), op1_64);
   }
-
-  SET_FLAGS_OSZAPC_LOGIC_64(op1_64);
-  clear_ZF();
-
-  /* now write result back to destination */
-  BX_WRITE_64BIT_REG(i->nnn(), op1_64);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EqGqM(bxInstruction_c *i)
@@ -105,12 +83,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EqGqM(bxInstruction_c *i)
   Bit64s displacement64;
   Bit64u index;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
   op2_64 = BX_READ_64BIT_REG(i->nnn());
   index = op2_64 & 0x3f;
   displacement64 = ((Bit64s) (op2_64 & BX_CONST64(0xffffffffffffffc0))) / 64;
-  op1_addr = RMAddr(i) + 8 * displacement64;
+  op1_addr = eaddr + 8 * displacement64;
   if (! i->as64L())
     op1_addr = (Bit32u) op1_addr;
 
@@ -137,12 +115,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EqGqM(bxInstruction_c *i)
   Bit64s displacement64;
   bx_bool bit_i;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
   op2_64 = BX_READ_64BIT_REG(i->nnn());
   index = op2_64 & 0x3f;
   displacement64 = ((Bit64s) (op2_64 & BX_CONST64(0xffffffffffffffc0))) / 64;
-  op1_addr = RMAddr(i) + 8 * displacement64;
+  op1_addr = eaddr + 8 * displacement64;
   if (! i->as64L())
     op1_addr = (Bit32u) op1_addr;
 
@@ -175,12 +153,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EqGqM(bxInstruction_c *i)
   Bit64u op1_64, op2_64, index;
   Bit64s displacement64;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
   op2_64 = BX_READ_64BIT_REG(i->nnn());
   index = op2_64 & 0x3f;
   displacement64 = ((Bit64s) (op2_64 & BX_CONST64(0xffffffffffffffc0))) / 64;
-  op1_addr = RMAddr(i) + 8 * displacement64;
+  op1_addr = eaddr + 8 * displacement64;
   if (! i->as64L())
     op1_addr = (Bit32u) op1_addr;
 
@@ -215,12 +193,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EqGqM(bxInstruction_c *i)
   Bit64s displacement64;
   Bit64u index;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
   op2_64 = BX_READ_64BIT_REG(i->nnn());
   index = op2_64 & 0x3f;
   displacement64 = ((Bit64s) (op2_64 & BX_CONST64(0xffffffffffffffc0))) / 64;
-  op1_addr = RMAddr(i) + 8 * displacement64;
+  op1_addr = eaddr + 8 * displacement64;
   if (! i->as64L())
     op1_addr = (Bit32u) op1_addr;
 
@@ -249,9 +227,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EqGqR(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::BT_EqIbM(bxInstruction_c *i)
 {
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit64u op1_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
+  Bit64u op1_64 = read_virtual_qword_64(i->seg(), eaddr);
   Bit8u  op2_8  = i->Ib() & 0x3f;
 
   set_CF((op1_64 >> op2_8) & 0x01);
@@ -269,9 +247,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTS_EqIbM(bxInstruction_c *i)
 {
   Bit8u op2_8 = i->Ib() & 0x3f;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit64u op1_64 = read_RMW_virtual_qword_64(i->seg(), RMAddr(i));
+  Bit64u op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   bx_bool temp_CF = (op1_64 >> op2_8) & 0x01;
   op1_64 |= (((Bit64u) 1) << op2_8);
   write_RMW_virtual_qword(op1_64);
@@ -295,9 +273,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTC_EqIbM(bxInstruction_c *i)
 {
   Bit8u op2_8 = i->Ib() & 0x3f;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit64u op1_64 = read_RMW_virtual_qword_64(i->seg(), RMAddr(i));
+  Bit64u op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   bx_bool temp_CF = (op1_64 >> op2_8) & 0x01;
   op1_64 ^= (((Bit64u) 1) << op2_8);  /* toggle bit */
   write_RMW_virtual_qword(op1_64);
@@ -321,9 +299,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EqIbM(bxInstruction_c *i)
 {
   Bit8u op2_8 = i->Ib() & 0x3f;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit64u op1_64 = read_RMW_virtual_qword_64(i->seg(), RMAddr(i));
+  Bit64u op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   bx_bool temp_CF = (op1_64 >> op2_8) & 0x01;
   op1_64 &= ~(((Bit64u) 1) << op2_8);
   write_RMW_virtual_qword(op1_64);
@@ -344,22 +322,12 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::BTR_EqIbR(bxInstruction_c *i)
 }
 
 /* 0F B8 */
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::POPCNT_GqEq(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::POPCNT_GqEqR(bxInstruction_c *i)
 {
-#if BX_SUPPORT_POPCNT || (BX_SUPPORT_SSE >= 5) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
-  Bit64u op1_64, op2_64;
+#if BX_SUPPORT_POPCNT || (BX_SUPPORT_SSE > 4) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
+  Bit64u op2_64 = BX_READ_64BIT_REG(i->rm());
 
-  /* op2_16 is a register or memory reference */
-  if (i->modC0()) {
-    op2_64 = BX_READ_64BIT_REG(i->rm());
-  }
-  else {
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-    /* pointer, segment address pair */
-    op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-  }
-
-  op1_64 = 0;
+  Bit64u op1_64 = 0;
   while (op2_64 != 0) {
     if (op2_64 & 1) op1_64++;
     op2_64 >>= 1;
@@ -372,7 +340,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::POPCNT_GqEq(bxInstruction_c *i)
   BX_WRITE_64BIT_REG(i->nnn(), op1_64);
 #else
   BX_INFO(("POPCNT_GqEq: required POPCNT support, use --enable-popcnt option"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
