@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: cpu.cc,v 1.230 2008/05/10 20:35:03 sshwarts Exp $
+// $Id: cpu.cc,v 1.289.2.1 2009/06/07 07:49:10 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,7 +22,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA B 02110-1301 USA
 /////////////////////////////////////////////////////////////////////////
 
 #define NEED_CPU_REG_SHORTCUTS 1
@@ -32,19 +32,12 @@
 
 #include "iodev/iodev.h"
 
-#if BX_EXTERNAL_DEBUGGER
-#include "extdb.h"
-#endif
-
 // Make code more tidy with a few macros.
 #if BX_SUPPORT_X86_64==0
 #define RIP EIP
 #define RCX ECX
 #endif
 
-// ICACHE instrumentation code
-#if BX_SUPPORT_ICACHE
-
 #define InstrumentICACHE 0
 
 #if InstrumentICACHE
@@ -63,14 +56,11 @@ static unsigned iCacheMisses=0;
   } \
 }
 #define InstrICache_Increment(v) (v)++
-
 #else
 #define InstrICache_Stats()
 #define InstrICache_Increment(v)
 #endif
 
-#endif // BX_SUPPORT_ICACHE
-
 // The CHECK_MAX_INSTRUCTIONS macro allows cpu_loop to execute a few
 // instructions and then return so that the other processors have a chance to
 // run.  This is used by bochs internal debugger or when simulating
@@ -100,7 +90,7 @@ void BX_CPU_C::cpu_loop(Bit32u max_instr_count)
     // only from exception function we can get here ...
     BX_INSTR_NEW_INSTRUCTION(BX_CPU_ID);
     BX_TICK1_IF_SINGLE_PROCESSOR();
-#if BX_DEBUGGER || BX_EXTERNAL_DEBUGGER || BX_GDBSTUB
+#if BX_DEBUGGER || BX_GDBSTUB
     if (dbg_instruction_epilog()) return;
 #endif
     CHECK_MAX_INSTRUCTIONS(max_instr_count);
@@ -138,16 +128,15 @@ void BX_CPU_C::cpu_loop(Bit32u max_instr_count)
 
 no_async_event:
 
-    Bit32u eipBiased = RIP + BX_CPU_THIS_PTR eipPageBias;
+    bx_address eipBiased = RIP + BX_CPU_THIS_PTR eipPageBias;
 
     if (eipBiased >= BX_CPU_THIS_PTR eipPageWindowSize) {
       prefetch();
       eipBiased = RIP + BX_CPU_THIS_PTR eipPageBias;
     }
 
-#if BX_SUPPORT_ICACHE
-    bx_phy_address pAddr = BX_CPU_THIS_PTR pAddrA20Page + eipBiased;
-    bxICacheEntry_c *entry = BX_CPU_THIS_PTR iCache.get_entry(pAddr);
+    bx_phy_address pAddr = BX_CPU_THIS_PTR pAddrPage + eipBiased;
+    bxICacheEntry_c *entry = BX_CPU_THIS_PTR iCache.get_entry(pAddr, BX_CPU_THIS_PTR fetchModeMask);
     bxInstruction_c *i = entry->i;
 
     InstrICache_Increment(iCacheLookups);
@@ -156,37 +145,27 @@ no_async_event:
     if ((entry->pAddr == pAddr) &&
         (entry->writeStamp == *(BX_CPU_THIS_PTR currPageWriteStampPtr)))
     {
-      // iCache hit. An instruction was found in the iCache.
-#if BX_INSTRUMENTATION
-      BX_INSTR_OPCODE(BX_CPU_ID, BX_CPU_THIS_PTR eipFetchPtr + eipBiased,
-         i->ilen(), BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.d_b, Is64BitMode());
-#endif
+      // iCache hit. An instruction was found in the iCache
     }
     else {
       // iCache miss. No validated instruction with matching fetch parameters
       // is in the iCache.
       InstrICache_Increment(iCacheMisses);
-      serveICacheMiss(entry, eipBiased, pAddr);
+      serveICacheMiss(entry, (Bit32u) eipBiased, pAddr);
       i = entry->i;
     }
-#else
-    bxInstruction_c iStorage, *i = &iStorage;
-    unsigned remainingInPage = BX_CPU_THIS_PTR eipPageWindowSize - eipBiased;
-    const Bit8u *fetchPtr = BX_CPU_THIS_PTR eipFetchPtr + eipBiased;
-    fetchInstruction(i, fetchPtr, remainingInPage);
-#endif
+
+    BxExecutePtr_tR execute = i->execute;
 
 #if BX_SUPPORT_TRACE_CACHE
-    unsigned length = entry->ilen;
+    bxInstruction_c *last = i + (entry->ilen);
 
-    for(;;i++) {
+    for(;;) {
 #endif
-      // An instruction will have been fetched using either the normal case,
-      // or the boundary fetch (across pages), by this point.
-      BX_INSTR_FETCH_DECODE_COMPLETED(BX_CPU_ID, i);
 
-#if BX_DEBUGGER || BX_EXTERNAL_DEBUGGER || BX_GDBSTUB
-      if (dbg_instruction_prolog()) return;
+#if BX_INSTRUMENTATION
+      BX_INSTR_OPCODE(BX_CPU_ID, BX_CPU_THIS_PTR eipFetchPtr + (RIP + BX_CPU_THIS_PTR eipPageBias),
+         i->ilen(), BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.d_b, Is64BitMode());
 #endif
 
 #if BX_DISASM
@@ -199,7 +178,7 @@ no_async_event:
       // decoding instruction compeleted -> continue with execution
       BX_INSTR_BEFORE_EXECUTION(BX_CPU_ID, i);
       RIP += i->ilen();
-      BX_CPU_CALL_METHOD(i->execute, (i)); // might iterate repeat instruction
+      BX_CPU_CALL_METHOD(execute, (i)); // might iterate repeat instruction
       BX_CPU_THIS_PTR prev_rip = RIP; // commit new RIP
       BX_INSTR_AFTER_EXECUTION(BX_CPU_ID, i);
       BX_TICK1_IF_SINGLE_PROCESSOR();
@@ -207,8 +186,12 @@ no_async_event:
       // inform instrumentation about new instruction
       BX_INSTR_NEW_INSTRUCTION(BX_CPU_ID);
 
+#if BX_SUPPORT_TRACE_CACHE
+      execute = (++i)->execute;
+#endif
+
       // note instructions generating exceptions never reach this point
-#if BX_DEBUGGER || BX_EXTERNAL_DEBUGGER || BX_GDBSTUB
+#if BX_DEBUGGER || BX_GDBSTUB
       if (dbg_instruction_epilog()) return;
 #endif
 
@@ -221,7 +204,7 @@ no_async_event:
         break;
       }
 
-      if (--length == 0) goto no_async_event;
+      if (i == last) goto no_async_event;
     }
 #endif
   }  // while (1)
@@ -299,75 +282,128 @@ void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat(bxInstruction_c *i, BxExecutePtr_tR
 #endif
 }
 
-void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat_ZFL(bxInstruction_c *i, BxExecutePtr_tR execute)
+void BX_CPP_AttrRegparmN(2) BX_CPU_C::repeat_ZF(bxInstruction_c *i, BxExecutePtr_tR execute)
 {
+  unsigned rep = i->repUsedValue();
+
   // non repeated instruction
-  if (! i->repUsedL()) {
+  if (! rep) {
     BX_CPU_CALL_METHOD(execute, (i));
     return;
   }
 
-  unsigned rep = i->repUsedValue();
-
+  if (rep == 3) { /* repeat prefix 0xF3 */
 #if BX_SUPPORT_X86_64
-  if (i->as64L()) {
-    while(1) {
-      if (RCX != 0) {
-        BX_CPU_CALL_METHOD(execute, (i));
-        BX_INSTR_REPEAT_ITERATION(BX_CPU_ID, i);
-        RCX --;
-      }
-      if (rep==3 && get_ZF()==0) return;
-      if (rep==2 && get_ZF()!=0) return;
-      if (RCX == 0) return;
+    if (i->as64L()) {
+      while(1) {
+        if (RCX != 0) {
+          BX_CPU_CALL_METHOD(execute, (i));
+          BX_INSTR_REPEAT_ITERATION(BX_CPU_ID, i);
+          RCX --;
+        }
+        if (! get_ZF() || RCX == 0) return;
 
 #if BX_DEBUGGER == 0
-      if (BX_CPU_THIS_PTR async_event)
+        if (BX_CPU_THIS_PTR async_event)
 #endif
-        break; // exit always if debugger enabled
+          break; // exit always if debugger enabled
 
-      BX_TICK1_IF_SINGLE_PROCESSOR();
+        BX_TICK1_IF_SINGLE_PROCESSOR();
+      }
+    }
+    else
+#endif
+    if (i->as32L()) {
+      while(1) {
+        if (ECX != 0) {
+          BX_CPU_CALL_METHOD(execute, (i));
+          BX_INSTR_REPEAT_ITERATION(BX_CPU_ID, i);
+          RCX = ECX - 1;
+        }
+        if (! get_ZF() || ECX == 0) return;
+
+#if BX_DEBUGGER == 0
+        if (BX_CPU_THIS_PTR async_event)
+#endif
+          break; // exit always if debugger enabled
+
+        BX_TICK1_IF_SINGLE_PROCESSOR();
+      }
+    }
+    else  // 16bit addrsize
+    {
+      while(1) {
+        if (CX != 0) {
+          BX_CPU_CALL_METHOD(execute, (i));
+          BX_INSTR_REPEAT_ITERATION(BX_CPU_ID, i);
+          CX --;
+        }
+        if (! get_ZF() || CX == 0) return;
+
+#if BX_DEBUGGER == 0
+        if (BX_CPU_THIS_PTR async_event)
+#endif
+          break; // exit always if debugger enabled
+
+        BX_TICK1_IF_SINGLE_PROCESSOR();
+      }
     }
   }
-  else
-#endif
-  if (i->as32L()) {
-    while(1) {
-      if (ECX != 0) {
-        BX_CPU_CALL_METHOD(execute, (i));
-        BX_INSTR_REPEAT_ITERATION(BX_CPU_ID, i);
-        RCX = ECX - 1;
-      }
-      if (rep==3 && get_ZF()==0) return;
-      if (rep==2 && get_ZF()!=0) return;
-      if (ECX == 0) return;
+  else {          /* repeat prefix 0xF2 */
+#if BX_SUPPORT_X86_64
+    if (i->as64L()) {
+      while(1) {
+        if (RCX != 0) {
+          BX_CPU_CALL_METHOD(execute, (i));
+          BX_INSTR_REPEAT_ITERATION(BX_CPU_ID, i);
+          RCX --;
+        }
+        if (get_ZF() || RCX == 0) return;
 
 #if BX_DEBUGGER == 0
-      if (BX_CPU_THIS_PTR async_event)
+        if (BX_CPU_THIS_PTR async_event)
 #endif
-        break; // exit always if debugger enabled
+          break; // exit always if debugger enabled
 
-      BX_TICK1_IF_SINGLE_PROCESSOR();
+        BX_TICK1_IF_SINGLE_PROCESSOR();
+      }
     }
-  }
-  else  // 16bit addrsize
-  {
-    while(1) {
-      if (CX != 0) {
-        BX_CPU_CALL_METHOD(execute, (i));
-        BX_INSTR_REPEAT_ITERATION(BX_CPU_ID, i);
-        CX --;
-      }
-      if (rep==3 && get_ZF()==0) return;
-      if (rep==2 && get_ZF()!=0) return;
-      if (CX == 0) return;
+    else
+#endif
+    if (i->as32L()) {
+      while(1) {
+        if (ECX != 0) {
+          BX_CPU_CALL_METHOD(execute, (i));
+          BX_INSTR_REPEAT_ITERATION(BX_CPU_ID, i);
+          RCX = ECX - 1;
+        }
+        if (get_ZF() || ECX == 0) return;
 
 #if BX_DEBUGGER == 0
-      if (BX_CPU_THIS_PTR async_event)
+        if (BX_CPU_THIS_PTR async_event)
 #endif
-        break; // exit always if debugger enabled
+          break; // exit always if debugger enabled
 
-      BX_TICK1_IF_SINGLE_PROCESSOR();
+        BX_TICK1_IF_SINGLE_PROCESSOR();
+      }
+    }
+    else  // 16bit addrsize
+    {
+      while(1) {
+        if (CX != 0) {
+          BX_CPU_CALL_METHOD(execute, (i));
+          BX_INSTR_REPEAT_ITERATION(BX_CPU_ID, i);
+          CX --;
+        }
+        if (get_ZF() || CX == 0) return;
+
+#if BX_DEBUGGER == 0
+        if (BX_CPU_THIS_PTR async_event)
+#endif
+          break; // exit always if debugger enabled
+
+        BX_TICK1_IF_SINGLE_PROCESSOR();
+      }
     }
   }
 
@@ -384,25 +420,25 @@ unsigned BX_CPU_C::handleAsyncEvent(void)
   //
   // This area is where we process special conditions and events.
   //
-  if (BX_CPU_THIS_PTR debug_trap & BX_DEBUG_TRAP_SPECIAL) {
-    // I made up the bitmask above to mean HALT state.
-    // for one processor, pass the time as quickly as possible until
+  if (BX_CPU_THIS_PTR activity_state) {
+    // For one processor, pass the time as quickly as possible until
     // an interrupt wakes up the CPU.
     while (1)
     {
-      if ((BX_CPU_INTR && (BX_CPU_THIS_PTR get_IF() || (BX_CPU_THIS_PTR debug_trap & BX_DEBUG_TRAP_MWAIT_IF))) ||
-           BX_CPU_THIS_PTR nmi_pending || BX_CPU_THIS_PTR smi_pending)
+      if ((BX_CPU_INTR && (BX_CPU_THIS_PTR get_IF() || 
+          (BX_CPU_THIS_PTR activity_state == BX_ACTIVITY_STATE_MWAIT_IF))) ||
+           BX_CPU_THIS_PTR pending_NMI || BX_CPU_THIS_PTR pending_SMI || BX_CPU_THIS_PTR pending_INIT)
       {
         // interrupt ends the HALT condition
 #if BX_SUPPORT_MONITOR_MWAIT
-        if (BX_CPU_THIS_PTR debug_trap & BX_DEBUG_TRAP_MWAIT)
+        if (BX_CPU_THIS_PTR activity_state >= BX_ACTIVITY_STATE_MWAIT)
           BX_MEM(0)->clear_monitor(BX_CPU_THIS_PTR bx_cpuid);
 #endif
-        BX_CPU_THIS_PTR debug_trap = 0; // clear traps for after resume
+        BX_CPU_THIS_PTR activity_state = 0;
         BX_CPU_THIS_PTR inhibit_mask = 0; // clear inhibits for after resume
         break;
       }
-      if ((BX_CPU_THIS_PTR debug_trap & BX_DEBUG_TRAP_SPECIAL) == 0) {
+      if (BX_CPU_THIS_PTR activity_state == BX_ACTIVITY_STATE_ACTIVE) {
         BX_INFO(("handleAsyncEvent: reset detected in HLT state"));
         break;
       }
@@ -433,6 +469,10 @@ unsigned BX_CPU_C::handleAsyncEvent(void)
     return 1; // Return to caller of cpu_loop.
   }
 
+  // VMLAUNCH/VMRESUME cannot be executed with interrupts inhibited.
+  // Save inhibit interrupts state into shadow bits after clearing
+  BX_CPU_THIS_PTR inhibit_mask = (BX_CPU_THIS_PTR inhibit_mask << 2) & 0xF;
+
   // Priority 1: Hardware Reset and Machine Checks
   //   RESET
   //   Machine Check
@@ -440,77 +480,106 @@ unsigned BX_CPU_C::handleAsyncEvent(void)
 
   // Priority 2: Trap on Task Switch
   //   T flag in TSS is set
-  if (BX_CPU_THIS_PTR debug_trap & 0x00008000) {
-    BX_CPU_THIS_PTR dr6 |= BX_CPU_THIS_PTR debug_trap;
+  if (BX_CPU_THIS_PTR debug_trap & BX_DEBUG_TRAP_TASK_SWITCH_BIT)
     exception(BX_DB_EXCEPTION, 0, 0); // no error, not interrupt
-  }
 
   // Priority 3: External Hardware Interventions
   //   FLUSH
   //   STOPCLK
   //   SMI
   //   INIT
-  // (bochs doesn't support these)
-  if (BX_CPU_THIS_PTR smi_pending && ! BX_CPU_THIS_PTR smm_mode())
+  if (BX_CPU_THIS_PTR pending_SMI && ! BX_CPU_THIS_PTR smm_mode())
   {
     // clear SMI pending flag and disable NMI when SMM was accepted
-    BX_CPU_THIS_PTR smi_pending = 0;
-    BX_CPU_THIS_PTR nmi_disable = 1;
+    BX_CPU_THIS_PTR pending_SMI = 0;
     enter_system_management_mode();
   }
 
+  if (BX_CPU_THIS_PTR pending_INIT && ! BX_CPU_THIS_PTR disable_INIT) {
+#if BX_SUPPORT_VMX
+    if (BX_CPU_THIS_PTR in_vmx_guest) {
+      BX_ERROR(("VMEXIT: INIT pin asserted"));
+      VMexit(0, VMX_VMEXIT_INIT, 0);
+    }
+#endif
+    // reset will clear pending INIT
+    BX_CPU_THIS_PTR reset(BX_RESET_SOFTWARE);
+  }
+
   // Priority 4: Traps on Previous Instruction
   //   Breakpoints
   //   Debug Trap Exceptions (TF flag set or data/IO breakpoint)
   if (BX_CPU_THIS_PTR debug_trap &&
-       !(BX_CPU_THIS_PTR inhibit_mask & BX_INHIBIT_DEBUG))
+       !(BX_CPU_THIS_PTR inhibit_mask & BX_INHIBIT_DEBUG_SHADOW))
   {
     // A trap may be inhibited on this boundary due to an instruction
     // which loaded SS.  If so we clear the inhibit_mask below
     // and don't execute this code until the next boundary.
-    // Commit debug events to DR6
-    BX_CPU_THIS_PTR dr6 |= BX_CPU_THIS_PTR debug_trap;
     exception(BX_DB_EXCEPTION, 0, 0); // no error, not interrupt
   }
 
   // Priority 5: External Interrupts
   //   NMI Interrupts
   //   Maskable Hardware Interrupts
-  if (BX_CPU_THIS_PTR inhibit_mask & BX_INHIBIT_INTERRUPTS) {
+  if (BX_CPU_THIS_PTR inhibit_mask & BX_INHIBIT_INTERRUPTS_SHADOW) {
     // Processing external interrupts is inhibited on this
     // boundary because of certain instructions like STI.
     // inhibit_mask is cleared below, in which case we will have
     // an opportunity to check interrupts on the next instruction
     // boundary.
   }
-  else if (BX_CPU_THIS_PTR nmi_pending) {
-    BX_CPU_THIS_PTR nmi_pending = 0;
-    BX_CPU_THIS_PTR nmi_disable = 1;
+#if BX_SUPPORT_VMX
+  else if (! BX_CPU_THIS_PTR disable_NMI && BX_CPU_THIS_PTR in_vmx_guest && 
+       VMEXIT(VMX_VM_EXEC_CTRL2_NMI_WINDOW_VMEXIT))
+  {
+    // NMI-window exiting
+    BX_ERROR(("VMEXIT: NMI window exiting"));
+    VMexit(0, VMX_VMEXIT_NMI_WINDOW, 0);
+  }
+#endif
+  else if (BX_CPU_THIS_PTR pending_NMI && ! BX_CPU_THIS_PTR disable_NMI) {
+    BX_CPU_THIS_PTR pending_NMI = 0;
+    BX_CPU_THIS_PTR disable_NMI = 1;
     BX_CPU_THIS_PTR errorno = 0;
     BX_CPU_THIS_PTR EXT = 1; /* external event */
+#if BX_SUPPORT_VMX
+    VMexit_Event(0, BX_NMI, 2, 0, 0);
+#endif
     BX_INSTR_HWINTERRUPT(BX_CPU_ID, 2, BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value, RIP);
-    interrupt(2, 0, 0, 0);
+    interrupt(2, BX_NMI, 0, 0);
   }
+#if BX_SUPPORT_VMX
+  else if (BX_CPU_THIS_PTR vmx_interrupt_window && BX_CPU_THIS_PTR get_IF()) {
+    // interrupt-window exiting
+    BX_ERROR(("VMEXIT: interrupt window exiting"));
+    VMexit(0, VMX_VMEXIT_INTERRUPT_WINDOW, 0);
+  }
+#endif
   else if (BX_CPU_INTR && BX_CPU_THIS_PTR get_IF() && BX_DBG_ASYNC_INTR)
   {
     Bit8u vector;
-
+#if BX_SUPPORT_VMX
+    VMexit_ExtInterrupt();
+#endif
     // NOTE: similar code in ::take_irq()
 #if BX_SUPPORT_APIC
-    if (BX_CPU_THIS_PTR local_apic.INTR)
-      vector = BX_CPU_THIS_PTR local_apic.acknowledge_int();
+    if (BX_CPU_THIS_PTR lapic.INTR)
+      vector = BX_CPU_THIS_PTR lapic.acknowledge_int();
     else
 #endif
       // if no local APIC, always acknowledge the PIC.
       vector = DEV_pic_iac(); // may set INTR with next interrupt
     BX_CPU_THIS_PTR errorno = 0;
     BX_CPU_THIS_PTR EXT = 1; /* external event */
+#if BX_SUPPORT_VMX
+    VMexit_Event(0, BX_EXTERNAL_INTERRUPT, vector, 0, 0);
+#endif
     BX_INSTR_HWINTERRUPT(BX_CPU_ID, vector,
         BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value, RIP);
-    interrupt(vector, 0, 0, 0);
+    interrupt(vector, BX_EXTERNAL_INTERRUPT, 0, 0);
     // Set up environment, as would be when this main cpu loop gets
     // invoked.  At the end of normal instructions, we always commmit
-    // the new EIP/ESP values.  But here, we call interrupt() much like
+    // the new EIP.  But here, we call interrupt() much like
     // it was a sofware interrupt instruction, and need to effect the
     // commit here.  This code mirrors similar code above.
     BX_CPU_THIS_PTR prev_rip = RIP; // commit new RIP
@@ -553,7 +622,7 @@ unsigned BX_CPU_C::handleAsyncEvent(void)
     // TF is set before execution of next instruction.  Schedule
     // a debug trap (#DB) after execution.  After completion of
     // next instruction, the code above will invoke the trap.
-    BX_CPU_THIS_PTR debug_trap |= 0x00004000; // BS flag in DR6
+    BX_CPU_THIS_PTR debug_trap |= BX_DEBUG_SINGLE_STEP_BIT;
   }
 
   // Now we can handle things which are synchronous to instruction
@@ -567,17 +636,14 @@ unsigned BX_CPU_C::handleAsyncEvent(void)
     if (BX_CPU_THIS_PTR dr7 & 0x000000ff) {
       bx_address iaddr = get_laddr(BX_SEG_REG_CS, BX_CPU_THIS_PTR prev_rip);
       Bit32u dr6_bits = hwdebug_compare(iaddr, 1, BX_HWDebugInstruction, BX_HWDebugInstruction);
-      if (dr6_bits)
-      {
+      if (dr6_bits) {
         // Add to the list of debug events thus far.
-        BX_CPU_THIS_PTR async_event = 1;
         BX_CPU_THIS_PTR debug_trap |= dr6_bits;
         // If debug events are not inhibited on this boundary,
         // fire off a debug fault.  Otherwise handle it on the next
         // boundary. (becomes a trap)
-        if (! (BX_CPU_THIS_PTR inhibit_mask & BX_INHIBIT_DEBUG)) {
-          // Commit debug events to DR6
-          BX_CPU_THIS_PTR dr6 = BX_CPU_THIS_PTR debug_trap;
+        if (! (BX_CPU_THIS_PTR inhibit_mask & BX_INHIBIT_DEBUG_SHADOW)) {
+          BX_ERROR(("#DB: x86 code breakpoint catched"));
           exception(BX_DB_EXCEPTION, 0, 0); // no error, not interrupt
         }
       }
@@ -585,17 +651,20 @@ unsigned BX_CPU_C::handleAsyncEvent(void)
   }
 #endif
 
-  // We have ignored processing of external interrupts and
-  // debug events on this boundary.  Reset the mask so they
-  // will be processed on the next boundary.
-  BX_CPU_THIS_PTR inhibit_mask = 0;
-
-  if (!(BX_CPU_INTR ||
+  if (!((BX_CPU_INTR && BX_CPU_THIS_PTR get_IF()) ||
         BX_CPU_THIS_PTR debug_trap ||
         BX_HRQ ||
         BX_CPU_THIS_PTR get_TF()
+#if BX_SUPPORT_VMX
+     || BX_CPU_THIS_PTR vmx_interrupt_window || BX_CPU_THIS_PTR inhibit_mask
+#endif
 #if BX_X86_DEBUGGER
-        || (BX_CPU_THIS_PTR dr7 & 0xff)
+       // any debug code breakpoint is set
+     || ((BX_CPU_THIS_PTR dr7 & 0xff) &&
+           (((BX_CPU_THIS_PTR dr7 >> 16) & 3) == 0 ||
+            ((BX_CPU_THIS_PTR dr7 >> 20) & 3) == 0 ||
+            ((BX_CPU_THIS_PTR dr7 >> 24) & 3) == 0 ||
+            ((BX_CPU_THIS_PTR dr7 >> 28) & 3) == 0))
 #endif
         ))
     BX_CPU_THIS_PTR async_event = 0;
@@ -614,14 +683,8 @@ unsigned BX_CPU_C::handleAsyncEvent(void)
 
 void BX_CPU_C::prefetch(void)
 {
-  bx_address temp_rip = RIP;
-  bx_address laddr = BX_CPU_THIS_PTR get_laddr(BX_SEG_REG_CS, temp_rip);
-  bx_phy_address pAddr;
-  unsigned pageOffset = PAGE_OFFSET(laddr);
-
-  // Calculate RIP at the beginning of the page.
-  BX_CPU_THIS_PTR eipPageBias = pageOffset - RIP;
-  BX_CPU_THIS_PTR eipPageWindowSize = 4096;
+  bx_address laddr;
+  unsigned pageOffset;
 
 #if BX_SUPPORT_X86_64
   if (Is64BitMode()) {
@@ -629,17 +692,34 @@ void BX_CPU_C::prefetch(void)
       BX_ERROR(("prefetch: #GP(0): RIP crossed canonical boundary"));
       exception(BX_GP_EXCEPTION, 0, 0);
     }
+
+    // linear address is equal to RIP in 64-bit long mode
+    pageOffset = PAGE_OFFSET(EIP);
+    laddr = RIP;
+
+    // Calculate RIP at the beginning of the page.
+    BX_CPU_THIS_PTR eipPageBias = pageOffset - RIP;
+    BX_CPU_THIS_PTR eipPageWindowSize = 4096;
   }
   else
 #endif
   {
-    Bit32u temp_limit = BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.limit_scaled;
-    if (((Bit32u) temp_rip) > temp_limit) {
-      BX_ERROR(("prefetch: EIP [%08x] > CS.limit [%08x]", (Bit32u) temp_rip, temp_limit));
+    BX_CLEAR_64BIT_HIGH(BX_64BIT_REG_RIP); /* avoid 32-bit EIP wrap */
+    laddr = BX_CPU_THIS_PTR get_laddr32(BX_SEG_REG_CS, EIP);
+    pageOffset = PAGE_OFFSET(laddr);
+
+    // Calculate RIP at the beginning of the page.
+    BX_CPU_THIS_PTR eipPageBias = (bx_address) pageOffset - EIP;
+
+    Bit32u limit = BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.limit_scaled;
+    if (EIP > limit) {
+      BX_ERROR(("prefetch: EIP [%08x] > CS.limit [%08x]", EIP, limit));
       exception(BX_GP_EXCEPTION, 0, 0);
     }
-    if (temp_limit + BX_CPU_THIS_PTR eipPageBias < 4096) {
-      BX_CPU_THIS_PTR eipPageWindowSize = temp_limit + BX_CPU_THIS_PTR eipPageBias + 1;
+
+    BX_CPU_THIS_PTR eipPageWindowSize = 4096;
+    if (limit + BX_CPU_THIS_PTR eipPageBias < 4096) {
+      BX_CPU_THIS_PTR eipPageWindowSize = (Bit32u)(limit + BX_CPU_THIS_PTR eipPageBias + 1);
     }
   }
 
@@ -648,34 +728,34 @@ void BX_CPU_C::prefetch(void)
   bx_TLB_entry *tlbEntry = &BX_CPU_THIS_PTR TLB.entry[TLB_index];
   Bit8u *fetchPtr = 0;
 
-  if (tlbEntry->lpf == lpf && (tlbEntry->accessBits & (0x01 << CPL))) {
-    pAddr = A20ADDR(tlbEntry->ppf | pageOffset);
-#if BX_SupportGuest2HostTLB
-    fetchPtr = (Bit8u*) (tlbEntry->hostPageAddr);
-#endif
+  if ((tlbEntry->lpf == lpf) && !(tlbEntry->accessBits & USER_PL)) {
+    BX_CPU_THIS_PTR pAddrPage = tlbEntry->ppf;
+    fetchPtr = (Bit8u*) tlbEntry->hostPageAddr;
   }  
   else {
+    bx_phy_address pAddr;
+
     if (BX_CPU_THIS_PTR cr0.get_PG()) {
-      pAddr = translate_linear(laddr, CPL, BX_READ, CODE_ACCESS);
-      pAddr = A20ADDR(pAddr);
+      pAddr = translate_linear(laddr, CPL, BX_EXECUTE);
     } 
     else {
-      pAddr = A20ADDR(laddr);
+      pAddr = (bx_phy_address) laddr;
     }
-  }
 
-  BX_CPU_THIS_PTR pAddrA20Page = LPFOf(pAddr);
+    BX_CPU_THIS_PTR pAddrPage = LPFOf(pAddr);
+  }
 
   if (fetchPtr) {
     BX_CPU_THIS_PTR eipFetchPtr = fetchPtr;
   }
   else {
     BX_CPU_THIS_PTR eipFetchPtr = BX_MEM(0)->getHostMemAddr(BX_CPU_THIS,
-        BX_CPU_THIS_PTR pAddrA20Page, BX_READ, CODE_ACCESS);
+        BX_CPU_THIS_PTR pAddrPage, BX_EXECUTE);
   }
 
   // Sanity checks
   if (! BX_CPU_THIS_PTR eipFetchPtr) {
+    bx_phy_address pAddr = BX_CPU_THIS_PTR pAddrPage + pageOffset;
     if (pAddr >= BX_MEM(0)->get_memory_len()) {
       BX_PANIC(("prefetch: running in bogus memory, pAddr=0x" FMT_PHY_ADDRX, pAddr));
     }
@@ -684,13 +764,7 @@ void BX_CPU_C::prefetch(void)
     }
   }
 
-#if BX_SUPPORT_ICACHE
-  BX_CPU_THIS_PTR currPageWriteStampPtr = pageWriteStampTable.getPageWriteStampPtr(pAddr);
-  Bit32u pageWriteStamp = *(BX_CPU_THIS_PTR currPageWriteStampPtr);
-  pageWriteStamp &= ~ICacheWriteStampFetchModeMask; // Clear out old fetch mode bits
-  pageWriteStamp |=  BX_CPU_THIS_PTR fetchModeMask; // And add new ones
-  pageWriteStampTable.setPageWriteStamp(pAddr, pageWriteStamp);
-#endif
+  BX_CPU_THIS_PTR currPageWriteStampPtr = pageWriteStampTable.getPageWriteStampPtr(BX_CPU_THIS_PTR pAddrPage);
 }
 
 void BX_CPU_C::boundaryFetch(const Bit8u *fetchPtr, unsigned remainingInPage, bxInstruction_c *i)
@@ -731,10 +805,10 @@ void BX_CPU_C::boundaryFetch(const Bit8u *fetchPtr, unsigned remainingInPage, bx
   }
 #if BX_SUPPORT_X86_64
   if (BX_CPU_THIS_PTR cpu_mode == BX_MODE_LONG_64)
-    ret = fetchDecode64(fetchBuffer, i, fetchBufferLimit);
+    ret = fetchDecode64(fetchBuffer, i, remainingInPage+fetchBufferLimit);
   else
 #endif
-    ret = fetchDecode32(fetchBuffer, i, fetchBufferLimit);
+    ret = fetchDecode32(fetchBuffer, i, remainingInPage+fetchBufferLimit);
 
   if (ret==0) {
     BX_INFO(("boundaryFetch #GP(0): failed to complete instruction decoding"));
@@ -748,178 +822,83 @@ void BX_CPU_C::boundaryFetch(const Bit8u *fetchPtr, unsigned remainingInPage, bx
   // again on the next instruction.  Perhaps we can optimize this to
   // eliminate the extra prefetch() since we do it above, but have to
   // think about repeated instructions, etc.
-  invalidate_prefetch_q();
+  // invalidate_prefetch_q();
 
   BX_INSTR_OPCODE(BX_CPU_ID, fetchBuffer, i->ilen(),
       BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.d_b, Is64BitMode());
 }
 
+void BX_CPU_C::deliver_SIPI(unsigned vector)
+{
+  if (BX_CPU_THIS_PTR activity_state == BX_ACTIVITY_STATE_WAIT_FOR_SIPI) {
+    BX_CPU_THIS_PTR activity_state = BX_ACTIVITY_STATE_ACTIVE;
+    RIP = 0;
+    load_seg_reg(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS], vector*0x100);
+    BX_CPU_THIS_PTR disable_INIT = 0; // enable INIT pin back
+    BX_INFO(("CPU %d started up at %04X:%08X by APIC",
+                   BX_CPU_THIS_PTR bx_cpuid, vector*0x100, EIP));
+  } else {
+    BX_INFO(("CPU %d started up by APIC, but was not halted at the time", BX_CPU_THIS_PTR bx_cpuid));
+  }
+}
+
+void BX_CPU_C::deliver_INIT(void)
+{
+  if (! BX_CPU_THIS_PTR disable_INIT) {
+    BX_CPU_THIS_PTR pending_INIT = 1;
+    BX_CPU_THIS_PTR async_event = 1;
+  }
+}
+
 void BX_CPU_C::deliver_NMI(void)
 {
-  BX_CPU_THIS_PTR nmi_pending = 1;
+  BX_CPU_THIS_PTR pending_NMI = 1;
   BX_CPU_THIS_PTR async_event = 1;
 }
 
 void BX_CPU_C::deliver_SMI(void)
 {
-  BX_CPU_THIS_PTR smi_pending = 1;
+  BX_CPU_THIS_PTR pending_SMI = 1;
   BX_CPU_THIS_PTR async_event = 1;
 }
 
-#if BX_EXTERNAL_DEBUGGER
-void BX_CPU_C::ask(int level, const char *prefix, const char *fmt, va_list ap)
+void BX_CPU_C::set_INTR(bx_bool value)
 {
-  char buf1[1024];
-  vsprintf (buf1, fmt, ap);
-  printf ("%s %s\n", prefix, buf1);
-  trap_debugger(1, BX_CPU_THIS);
-}
-#endif
-
-#if BX_DEBUGGER || BX_EXTERNAL_DEBUGGER || BX_GDBSTUB
-bx_bool BX_CPU_C::dbg_instruction_prolog(void)
-{
-#if BX_DEBUGGER
-  if(dbg_check_begin_instr_bpoint()) return 1;
-#endif
-
-#if BX_EXTERNAL_DEBUGGER
-  bx_external_debugger(BX_CPU_THIS);
-#endif
-
-  return 0;
+  BX_CPU_THIS_PTR INTR = value;
+  BX_CPU_THIS_PTR async_event = 1;
 }
 
+#if BX_DEBUGGER || BX_GDBSTUB
 bx_bool BX_CPU_C::dbg_instruction_epilog(void)
 {
 #if BX_DEBUGGER
-  if (dbg_check_end_instr_bpoint()) return 1;
-#endif
-
-#if BX_GDBSTUB
-  if (bx_dbg.gdbstub_enabled) {
-    unsigned reason = bx_gdbstub_check(EIP);
-    if (reason != GDBSTUB_STOP_NO_REASON) return 1;
-  }
-#endif
-
-  return 0;
-}
-#endif // BX_DEBUGGER || BX_EXTERNAL_DEBUGGER || BX_GDBSTUB
-
-#if BX_DEBUGGER
-extern unsigned dbg_show_mask;
-
-bx_bool BX_CPU_C::dbg_check_begin_instr_bpoint(void)
-{
   Bit64u tt = bx_pc_system.time_ticks();
   bx_address debug_eip = RIP;
   Bit16u cs = BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value;
 
+  BX_CPU_THIS_PTR guard_found.icount++;
   BX_CPU_THIS_PTR guard_found.cs  = cs;
   BX_CPU_THIS_PTR guard_found.eip = debug_eip;
   BX_CPU_THIS_PTR guard_found.laddr = BX_CPU_THIS_PTR get_laddr(BX_SEG_REG_CS, debug_eip);
-  BX_CPU_THIS_PTR guard_found.is_32bit_code =
-    BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.d_b;
-  BX_CPU_THIS_PTR guard_found.is_64bit_code = Is64BitMode();
+  BX_CPU_THIS_PTR guard_found.code_32_64 = BX_CPU_THIS_PTR fetchModeMask;
 
-  // support for 'show' command in debugger
-  if(dbg_show_mask) {
-    int rv = bx_dbg_show_symbolic();
-    if (rv) return(rv);
-  }
-
-  // see if debugger is looking for iaddr breakpoint of any type
-  if (bx_guard.guard_for & BX_DBG_GUARD_IADDR_ALL) {
-#if (BX_DBG_MAX_VIR_BPOINTS > 0)
-    if (bx_guard.guard_for & BX_DBG_GUARD_IADDR_VIR) {
-      if ((BX_CPU_THIS_PTR guard_found.icount!=0) ||
-          (tt != BX_CPU_THIS_PTR guard_found.time_tick))
-      {
-        for (unsigned i=0; i<bx_guard.iaddr.num_virtual; i++) {
-          if (bx_guard.iaddr.vir[i].enabled &&
-             (bx_guard.iaddr.vir[i].cs  == cs) &&
-             (bx_guard.iaddr.vir[i].eip == debug_eip))
-          {
-            BX_CPU_THIS_PTR guard_found.guard_found = BX_DBG_GUARD_IADDR_VIR;
-            BX_CPU_THIS_PTR guard_found.iaddr_index = i;
-	    BX_CPU_THIS_PTR guard_found.time_tick = tt;
-            return(1); // on a breakpoint
-          }
-        }
-      }
-    }
-#endif
-#if (BX_DBG_MAX_LIN_BPOINTS > 0)
-    if (bx_guard.guard_for & BX_DBG_GUARD_IADDR_LIN) {
-      if ((BX_CPU_THIS_PTR guard_found.icount!=0) ||
-          (tt != BX_CPU_THIS_PTR guard_found.time_tick))
-      {
-        for (unsigned i=0; i<bx_guard.iaddr.num_linear; i++) {
-          if (bx_guard.iaddr.lin[i].enabled &&
-             (bx_guard.iaddr.lin[i].addr == BX_CPU_THIS_PTR guard_found.laddr))
-          {
-            BX_CPU_THIS_PTR guard_found.guard_found = BX_DBG_GUARD_IADDR_LIN;
-            BX_CPU_THIS_PTR guard_found.iaddr_index = i;
-	    BX_CPU_THIS_PTR guard_found.time_tick = tt;
-            return(1); // on a breakpoint
-          }
-        }
-      }
-    }
-#endif
-#if (BX_DBG_MAX_PHY_BPOINTS > 0)
-    if (bx_guard.guard_for & BX_DBG_GUARD_IADDR_PHY) {
-      bx_phy_address phy;
-      bx_bool valid = dbg_xlate_linear2phy(BX_CPU_THIS_PTR guard_found.laddr, &phy);
-      // The "guard_found.icount!=0" condition allows you to step or
-      // continue beyond a breakpoint.  Bryce tried removing it once,
-      // and once you get to a breakpoint you are stuck there forever.
-      // Not pretty.
-      if (valid && ((BX_CPU_THIS_PTR guard_found.icount!=0) ||
-          (tt != BX_CPU_THIS_PTR guard_found.time_tick)))
-      {
-        for (unsigned i=0; i<bx_guard.iaddr.num_physical; i++) {
-          if (bx_guard.iaddr.phy[i].enabled && (bx_guard.iaddr.phy[i].addr == phy))
-          {
-            BX_CPU_THIS_PTR guard_found.guard_found = BX_DBG_GUARD_IADDR_PHY;
-            BX_CPU_THIS_PTR guard_found.iaddr_index = i;
-	    BX_CPU_THIS_PTR guard_found.time_tick = tt;
-            return(1); // on a breakpoint
-          }
-        }
-      }
-    }
-#endif
-  }
-
-  return(0); // not on a breakpoint
-}
-
-bx_bool BX_CPU_C::dbg_check_end_instr_bpoint(void)
-{
-  BX_CPU_THIS_PTR guard_found.icount++;
-  BX_CPU_THIS_PTR guard_found.cs  =
-    BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value;
-  BX_CPU_THIS_PTR guard_found.eip = RIP;
-  BX_CPU_THIS_PTR guard_found.laddr = BX_CPU_THIS_PTR get_laddr(BX_SEG_REG_CS, RIP);
-  BX_CPU_THIS_PTR guard_found.is_32bit_code =
-    BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.d_b;
-  BX_CPU_THIS_PTR guard_found.is_64bit_code = Is64BitMode();
+  //
+  // Take care of break point conditions generated during instruction execution
+  //
 
   // Check if we hit read/write or time breakpoint
   if (BX_CPU_THIS_PTR break_point) {
     switch (BX_CPU_THIS_PTR break_point) {
     case BREAK_POINT_TIME:
-      BX_INFO(("[" FMT_LL "d] Caught time breakpoint", bx_pc_system.time_ticks()));
+      BX_INFO(("[" FMT_LL "d] Caught time breakpoint", tt));
       BX_CPU_THIS_PTR stop_reason = STOP_TIME_BREAK_POINT;
       return(1); // on a breakpoint
     case BREAK_POINT_READ:
-      BX_INFO(("[" FMT_LL "d] Caught read watch point", bx_pc_system.time_ticks()));
+      BX_INFO(("[" FMT_LL "d] Caught read watch point", tt));
       BX_CPU_THIS_PTR stop_reason = STOP_READ_WATCH_POINT;
       return(1); // on a breakpoint
     case BREAK_POINT_WRITE:
-      BX_INFO(("[" FMT_LL "d] Caught write watch point", bx_pc_system.time_ticks()));
+      BX_INFO(("[" FMT_LL "d] Caught write watch point", tt));
       BX_CPU_THIS_PTR stop_reason = STOP_WRITE_WATCH_POINT;
       return(1); // on a breakpoint
     default:
@@ -933,16 +912,82 @@ bx_bool BX_CPU_C::dbg_check_end_instr_bpoint(void)
     return(1); // on a breakpoint
   }
 
-  // convenient point to see if user typed Ctrl-C
-  if (bx_guard.interrupt_requested &&
-     (bx_guard.guard_for & BX_DBG_GUARD_CTRL_C))
-  {
-    BX_CPU_THIS_PTR guard_found.guard_found = BX_DBG_GUARD_CTRL_C;
-    return(1); // Ctrl-C pressed
+  // convenient point to see if user requested debug break or typed Ctrl-C
+  if (bx_guard.interrupt_requested) {
+    return(1);
   }
 
-  return(0); // no breakpoint
+  // support for 'show' command in debugger
+  extern unsigned dbg_show_mask;
+  if(dbg_show_mask) {
+    int rv = bx_dbg_show_symbolic();
+    if (rv) return(rv);
+  }
+
+  // Just committed an instruction, before fetching a new one
+  // see if debugger is looking for iaddr breakpoint of any type
+  if (bx_guard.guard_for & BX_DBG_GUARD_IADDR_ALL) {
+#if (BX_DBG_MAX_VIR_BPOINTS > 0)
+    if (bx_guard.guard_for & BX_DBG_GUARD_IADDR_VIR) {
+      for (unsigned n=0; n<bx_guard.iaddr.num_virtual; n++) {
+        if (bx_guard.iaddr.vir[n].enabled &&
+           (bx_guard.iaddr.vir[n].cs  == cs) &&
+           (bx_guard.iaddr.vir[n].eip == debug_eip))
+        {
+          BX_CPU_THIS_PTR guard_found.guard_found = BX_DBG_GUARD_IADDR_VIR;
+          BX_CPU_THIS_PTR guard_found.iaddr_index = n;
+          BX_CPU_THIS_PTR guard_found.time_tick = tt;
+          return(1); // on a breakpoint
+        }
+      }
+    }
+#endif
+#if (BX_DBG_MAX_LIN_BPOINTS > 0)
+    if (bx_guard.guard_for & BX_DBG_GUARD_IADDR_LIN) {
+      for (unsigned n=0; n<bx_guard.iaddr.num_linear; n++) {
+        if (bx_guard.iaddr.lin[n].enabled &&
+           (bx_guard.iaddr.lin[n].addr == BX_CPU_THIS_PTR guard_found.laddr))
+        {
+          BX_CPU_THIS_PTR guard_found.guard_found = BX_DBG_GUARD_IADDR_LIN;
+          BX_CPU_THIS_PTR guard_found.iaddr_index = n;
+          BX_CPU_THIS_PTR guard_found.time_tick = tt;
+          return(1); // on a breakpoint
+        }
+      }
+    }
+#endif
+#if (BX_DBG_MAX_PHY_BPOINTS > 0)
+    if (bx_guard.guard_for & BX_DBG_GUARD_IADDR_PHY) {
+      bx_phy_address phy;
+      bx_bool valid = dbg_xlate_linear2phy(BX_CPU_THIS_PTR guard_found.laddr, &phy);
+      if (valid) {
+        for (unsigned n=0; n<bx_guard.iaddr.num_physical; n++) {
+          if (bx_guard.iaddr.phy[n].enabled && (bx_guard.iaddr.phy[n].addr == phy))
+          {
+            BX_CPU_THIS_PTR guard_found.guard_found = BX_DBG_GUARD_IADDR_PHY;
+            BX_CPU_THIS_PTR guard_found.iaddr_index = n;
+            BX_CPU_THIS_PTR guard_found.time_tick = tt;
+            return(1); // on a breakpoint
+          }
+        }
+      }
+    }
+#endif
+  }
+#endif
+
+#if BX_GDBSTUB
+  if (bx_dbg.gdbstub_enabled) {
+    unsigned reason = bx_gdbstub_check(EIP);
+    if (reason != GDBSTUB_STOP_NO_REASON) return(1);
+  }
+#endif
+
+  return(0);
 }
+#endif // BX_DEBUGGER || BX_GDBSTUB
+
+#if BX_DEBUGGER
 
 void BX_CPU_C::dbg_take_irq(void)
 {
@@ -955,7 +1000,7 @@ void BX_CPU_C::dbg_take_irq(void)
       BX_CPU_THIS_PTR errorno = 0;
       BX_CPU_THIS_PTR EXT = 1; // external event
       BX_CPU_THIS_PTR async_event = 1; // set in case INTR is triggered
-      interrupt(vector, 0, 0, 0);
+      interrupt(vector, BX_EXTERNAL_INTERRUPT, 0, 0);
     }
   }
 }
@@ -970,7 +1015,7 @@ void BX_CPU_C::dbg_force_interrupt(unsigned vector)
     BX_CPU_THIS_PTR errorno = 0;
     BX_CPU_THIS_PTR EXT = 1; // external event
     BX_CPU_THIS_PTR async_event = 1; // probably don't need this
-    interrupt(vector, 0, 0, 0);
+    interrupt(vector, BX_EXTERNAL_INTERRUPT, 0, 0);
   }
 }
 
