@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: cpuid.cc,v 1.59 2007/12/15 17:42:20 sshwarts Exp $
+// $Id: cpuid.cc,v 1.80.2.1 2009/06/07 07:49:10 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2007 Stanislav Shwartsman
@@ -17,17 +17,15 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA B 02110-1301 USA
 //
 /////////////////////////////////////////////////////////////////////////
 
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-
 #if BX_SUPPORT_X86_64==0
 // Make life easier for merging code.
 #define RAX EAX
@@ -36,14 +34,8 @@
 #define RDX EDX
 #endif
 
-#if BX_SUPPORT_3DNOW
-  #define BX_CPU_VENDOR_INTEL 0
-#else
-  #define BX_CPU_VENDOR_INTEL 1
-#endif
-
 /* Get CPU version information. */
-Bit32u BX_CPU_C::get_cpu_version_information()
+Bit32u BX_CPU_C::get_cpu_version_information(void)
 {
   Bit32u family = 0, model = 0, stepping = 0;
   Bit32u extended_model = 0;
@@ -114,69 +106,126 @@ Bit32u BX_CPU_C::get_cpu_version_information()
 
 #endif  // BX_CPU_LEVEL > 3
 
-  return (extended_family << 20) | 
-         (extended_model << 16) | 
-         (family << 8) | 
+  return (extended_family << 20) |
+         (extended_model << 16) |
+         (family << 8) |
          (model<<4) | stepping;
-}       
+}
 
 /* Get CPU extended feature flags. */
-Bit32u BX_CPU_C::get_extended_cpuid_features()
+Bit32u BX_CPU_C::get_extended_cpuid_features(void)
 {
   // [0:0]   SSE3: SSE3 Instructions
-  // [2:1]   reserved
+  // [1:1]   PCLMULQDQ Instruction support
+  // [2:2]   reserved
   // [3:3]   MONITOR/MWAIT support
   // [4:4]   DS-CPL: CPL qualified debug store
   // [5:5]   VMX: Virtual Machine Technology
-  // [6:6]   reserved
+  // [6:6]   SMX: Secure Virtual Machine Technology
   // [7:7]   EST: Enhanced Intel SpeedStep Technology
   // [8:8]   TM2: Thermal Monitor 2
   // [9:9]   SSE3E: SSE3E Instructions (Intel Core Duo 2 new instructions)
   // [10:10] CNXT-ID: L1 context ID
-  // [12:11] reserved
+  // [11:11] reserved
+  // [12:12] FMA Instructions support
   // [13:13] CMPXCHG16B: CMPXCHG16B instruction support
   // [14:14] xTPR update control
-  // [18:15] reserved
-  // [19:19] SSE4.1: SSE4.1 Instructions
-  // [20:20] SSE4.2: SSE4.2 (SSE4E) Instructions
-  // [21:22] Reserved
-  // [23:23] POPCNT instruction support
-  // [31:21] reserved
+  // [17:15] reserved
+  // [18:18] DCA - Direct Cache Access
+  // [19:19] SSE4.1 Instructions
+  // [20:20] SSE4.2 Instructions
+  // [21:21] X2APIC
+  // [22:22] MOVBE instruction
+  // [23:23] POPCNT instruction
+  // [24:24] reserved
+  // [25:25] AES Instructions
+  // [26:26] XSAVE extensions support
+  // [27:27] OSXSAVE support
+  // [28:28] AVX extensions support
+  // [31:29] reserved
 
   Bit32u features = 0;
 
 #if BX_SUPPORT_SSE >= 3
-  features |= 0x1;      // support SSE3
+  features |= 0x1;               // support SSE3
 #endif
 #if BX_SUPPORT_MONITOR_MWAIT
-  features |= (1<<3);   // support MONITOR/MWAIT
+  features |= (1<<3);            // support MONITOR/MWAIT
+#endif
+#if BX_SUPPORT_VMX
+  features |= (1<<5);            // support VMX
 #endif
 #if (BX_SUPPORT_SSE >= 4) || (BX_SUPPORT_SSE >= 3 && BX_SUPPORT_SSE_EXTENSION > 0)
-  features |= (1<<9);   // support SSE3E
+  features |= (1<<9);            // support SSE3E
 #endif
 
 #if BX_SUPPORT_X86_64
-  features |= (1<<13);  // support CMPXCHG16B
+  features |= (1<<13);           // support CMPXCHG16B
 #endif
 
 #if BX_SUPPORT_SSE >= 4
-  features |= (1<<19);  // support SSE4.1
+  features |= (1<<19);           // support SSE4.1
 #endif
 
-#if (BX_SUPPORT_SSE >= 5) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
-  features |= (1<<20);  // support SSE4.2 (SSE4E)
+#if (BX_SUPPORT_SSE > 4) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
+  features |= (1<<20);           // support SSE4.2
+#endif
+
+#if BX_SUPPORT_MOVBE
+  features |= (1<<22);           // support MOVBE instruction
+#endif
+
+#if BX_SUPPORT_POPCNT || (BX_SUPPORT_SSE > 4) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
+  features |= (1<<23);           // support POPCNT instruction
+#endif
+
+#if BX_SUPPORT_AES
+  features |= (1<<25);           // support AES instructions
+#endif
+
+#if BX_SUPPORT_XSAVE
+  features |= (1<<26) | (1<<27); // support XSAVE extensions
 #endif
 
-#if BX_SUPPORT_POPCNT || (BX_SUPPORT_SSE >= 5) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
-  features |= (1<<23);  // support POPCNT instruction
-#endif  
-  
   return features;
 }
 
 /* Get CPU feature flags. Returned by CPUID functions 1 and 80000001.  */
-Bit32u BX_CPU_C::get_std_cpuid_features()
+Bit32u BX_CPU_C::get_std_cpuid_features(void)
 {
+  //   [0:0]   FPU on chip
+  //   [1:1]   VME: Virtual-8086 Mode enhancements
+  //   [2:2]   DE: Debug Extensions (I/O breakpoints)
+  //   [3:3]   PSE: Page Size Extensions
+  //   [4:4]   TSC: Time Stamp Counter
+  //   [5:5]   MSR: RDMSR and WRMSR support
+  //   [6:6]   PAE: Physical Address Extensions
+  //   [7:7]   MCE: Machine Check Exception
+  //   [8:8]   CXS: CMPXCHG8B instruction
+  //   [9:9]   APIC: APIC on Chip
+  //   [10:10] Reserved
+  //   [11:11] SYSENTER/SYSEXIT support
+  //   [12:12] MTRR: Memory Type Range Reg
+  //   [13:13] PGE/PTE Global Bit
+  //   [14:14] MCA: Machine Check Architecture
+  //   [15:15] CMOV: Cond Mov/Cmp Instructions
+  //   [16:16] PAT: Page Attribute Table
+  //   [17:17] PSE-36: Physical Address Extensions
+  //   [18:18] Processor Serial Number
+  //   [19:19] CLFLUSH: CLFLUSH Instruction support
+  //   [20:20] Reserved
+  //   [21:21] DS: Debug Store
+  //   [22:22] ACPI: Thermal Monitor and Software Controlled Clock Facilities
+  //   [23:23] MMX Technology
+  //   [24:24] FXSR: FXSAVE/FXRSTOR (also indicates CR4.OSFXSR is available)
+  //   [25:25] SSE: SSE Extensions
+  //   [26:26] SSE2: SSE2 Extensions
+  //   [27:27] Reserved
+  //   [28:28] Hyper Threading Technology
+  //   [29:29] TM: Thermal Monitor
+  //   [30:30] Reserved
+  //   [31:31] PBE: Pending Break Enable
+
   Bit32u features = 0;
 
 #if BX_SUPPORT_FPU
@@ -186,72 +235,86 @@ Bit32u BX_CPU_C::get_std_cpuid_features()
   features |= (1<<1);
 #endif
 
+  features |= (1<<2);   // support Debug Extensions
+
 #if (BX_CPU_LEVEL >= 5)
-  features |= (1<<4);   // implement TSC
-  features |= (1<<5);   // support RDMSR/WRMSR
-  features |= (1<<8);   // Support CMPXCHG8B instruction
-
-#if BX_SUPPORT_MMX
-  features |= (1<<23);  // support MMX
+  features |= (1<<4);             // support Time Stamp Counter
+  features |= (1<<5);             // support RDMSR/WRMSR
+  features |= (1<<7) | (1<<14);   // support Machine Check
+  features |= (1<<8);             // support CMPXCHG8B instruction
 #endif
 
-#endif
-
-#if BX_CPU_LEVEL >= 6
-  features |= (1<<24);  // Implement FSAVE/FXRSTOR instructions.
-#endif
-
-#if BX_SUPPORT_CLFLUSH
-  features |= (1<<19);  // Implement CLFLUSH instruction
-#endif
-
-#if BX_CPU_LEVEL >= 6
-  features |= (1<<15);  // Implement CMOV instructions.
 #if BX_SUPPORT_APIC
   // if MSR_APICBASE APIC Global Enable bit has been cleared,
   // the CPUID feature flag for the APIC is set to 0.
   if (BX_CPU_THIS_PTR msr.apicbase & 0x800)
     features |= (1<<9); // APIC on chip
 #endif
+
+#if BX_SUPPORT_SEP
+  features |= (1<<11);  // SYSENTER/SYSEXIT
+#endif
+
+#if BX_SUPPORT_CLFLUSH
+  features |= (1<<19);  // Implement CLFLUSH instruction
+#endif
+
+#if BX_SUPPORT_MMX
+  features |= (1<<23);  // support MMX
+#endif
+
+#if BX_CPU_LEVEL >= 6
+  features |= (1<<24);  // Implement FSAVE/FXRSTOR instructions.
+#endif
+
 #if BX_SUPPORT_SSE >= 1
   features |= (1<<25);  // support SSE
 #endif
 #if BX_SUPPORT_SSE >= 2
   features |= (1<<26);  // support SSE2
 #endif
+
+#if BX_CPU_LEVEL >= 6
+  features |= (1<<15);  // Implement CMOV instructions.
 #endif
 
-#if BX_SUPPORT_SEP
-  features |= (1<<11);  // SYSENTER/SYSEXIT
-#endif
-
-#if BX_SUPPORT_MTRR
-  features |= (1<<12);  // Implement MTRRs
-#endif
 #if BX_SUPPORT_LARGE_PAGES
-  features |= (1<< 3);  // Support Page-Size Extension (2M/4M pages)
-#endif
-#if BX_SUPPORT_GLOBAL_PAGES
-  features |= (1<<13);  // Support Global pages
+  features |= (1<<3) | (1<<17);  // support Page-Size Extension (4M pages)
 #endif
 #if BX_SUPPORT_PAE
-  features |= (1<<6);   // Support PAE
+  features |= (1<<6);   // support PAE
+#endif
+#if BX_SUPPORT_MTRR
+  features |= (1<<12) | (1<<16);  // Implement MTRRs and PAT
+#endif
+#if BX_SUPPORT_GLOBAL_PAGES
+  features |= (1<<13);  // support Global pages
 #endif
 
 #if BX_SUPPORT_SMP
   // Intel(R) HyperThreading Technology
-  if (SIM->get_param_num(BXPN_CPU_NTHREADS)->get() > 1) 
+  if (SIM->get_param_num(BXPN_CPU_NTHREADS)->get() > 1)
     features |= (1<<28);
 #endif
 
   return features;
 }
 
-void BX_CPU_C::CPUID(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::CPUID(bxInstruction_c *i)
 {
-  Bit32u function = EAX;
-
 #if BX_CPU_LEVEL >= 4
+  Bit32u function    = EAX;
+#if BX_SUPPORT_XSAVE
+  Bit32u subfunction = ECX;
+#endif
+
+#if BX_SUPPORT_VMX
+  if (BX_CPU_THIS_PTR in_vmx_guest) {
+    BX_ERROR(("VMEXIT: CPUID in VMX non-root operation"));
+    VMexit(i, VMX_VMEXIT_CPUID, 0);
+  }
+#endif
+
   if(function < 0x80000000) {
     if(function < MAX_STD_CPUID_FUNCTION) {
       RAX = BX_CPU_THIS_PTR cpuid_std_function[function].eax;
@@ -265,6 +328,14 @@ void BX_CPU_C::CPUID(bxInstruction_c *i)
         if ((BX_CPU_THIS_PTR msr.apicbase & 0x800) == 0)
           RDX &= ~(1<<9); // APIC on chip
       }
+#endif
+#if BX_SUPPORT_XSAVE
+      if (function == 0xD && subfunction > 0) {
+        RAX = 0;
+        RBX = 0;
+        RCX = 0;
+        RDX = 0;
+      }
 #endif
       return;
     }
@@ -277,7 +348,7 @@ void BX_CPU_C::CPUID(bxInstruction_c *i)
       RCX = BX_CPU_THIS_PTR cpuid_ext_function[function].ecx;
       RDX = BX_CPU_THIS_PTR cpuid_ext_function[function].edx;
 #if BX_SUPPORT_APIC
-      if (EAX == 1) {
+      if (function == 1) {
         // if MSR_APICBASE APIC Global Enable bit has been cleared,
         // the CPUID feature flag for the APIC is set to 0.
         if ((BX_CPU_THIS_PTR msr.apicbase & 0x800) == 0)
@@ -295,12 +366,16 @@ void BX_CPU_C::CPUID(bxInstruction_c *i)
   RDX = 0;
 #else
   BX_INFO(("CPUID: not available on < 486"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
 void BX_CPU_C::set_cpuid_defaults(void)
 {
+  Bit8u *vendor_string = (Bit8u *)SIM->get_param_string(BXPN_VENDOR_STRING)->getptr();
+  Bit8u *brand_string = (Bit8u *)SIM->get_param_string(BXPN_BRAND_STRING)->getptr();
+  bool cpuid_limit_winnt = SIM->get_param_bool(BXPN_CPUID_LIMIT_WINNT)->get();
+
   cpuid_function_t *cpuid;
   int i;
 
@@ -330,20 +405,23 @@ void BX_CPU_C::set_cpuid_defaults(void)
 #if BX_CPU_LEVEL <= 5
   // 486 and Pentium processors
   cpuid->eax = 1;
-#else 
+#else
   // for Pentium Pro, Pentium II, Pentium 4 processors
-  cpuid->eax = BX_SUPPORT_MONITOR_MWAIT ? 5 : 2;
+  cpuid->eax = 2;
+  // do not report CPUID functions above 0x3 if cpuid_limit_winnt is set
+  // to workaround WinNT issue.
+  if (! cpuid_limit_winnt) {
+    if (BX_SUPPORT_MONITOR_MWAIT)
+      cpuid->eax = 0x5;
+    if (BX_SUPPORT_XSAVE)
+      cpuid->eax = 0xD;
+  }
 #endif
 
-#if BX_CPU_VENDOR_INTEL
-  cpuid->ebx = 0x756e6547; // "Genu"
-  cpuid->edx = 0x49656e69; // "ineI"
-  cpuid->ecx = 0x6c65746e; // "ntel"
-#else
-  cpuid->ebx = 0x68747541; // "Auth"
-  cpuid->edx = 0x69746e65; // "enti"
-  cpuid->ecx = 0x444d4163; // "cAMD"
-#endif
+  // CPUID vendor string (e.g. GenuineIntel, AuthenticAMD, CentaurHauls, ...)
+  memcpy(&(cpuid->ebx), vendor_string    , 4);
+  memcpy(&(cpuid->edx), vendor_string + 4, 4);
+  memcpy(&(cpuid->ecx), vendor_string + 8, 4);
 
   // ------------------------------------------------------
   // CPUID function 0x00000001
@@ -355,10 +433,9 @@ void BX_CPU_C::set_cpuid_defaults(void)
   //   [11:8]  Family: 4=486, 5=Pentium, 6=PPro, ...
   //   [13:12] Type: 0=OEM, 1=overdrive, 2=dual cpu, 3=reserved
   //   [31:14] Reserved
-
   cpuid->eax = get_cpu_version_information();
 
-  // EBX:      
+  // EBX:
   //   [7:0]   Brand ID
   //   [15:8]  CLFLUSH cache line size (value*8 = cache line size in bytes)
   //   [23:16] Number of logical processors in one physical processor
@@ -366,7 +443,7 @@ void BX_CPU_C::set_cpuid_defaults(void)
 
   cpuid->ebx = 0;
 #if BX_SUPPORT_APIC
-  cpuid->ebx |= (BX_CPU_THIS_PTR local_apic.get_id() << 24);
+  cpuid->ebx |= ((BX_CPU_THIS_PTR lapic.get_id() & 0xff) << 24);
 #endif
 #if BX_SUPPORT_CLFLUSH
   cpuid->ebx |= (CACHE_LINE_SIZE / 8) << 8;
@@ -379,22 +456,33 @@ void BX_CPU_C::set_cpuid_defaults(void)
 
   // ECX:       Extended Feature Flags
   //   [0:0]   SSE3: SSE3 Instructions
-  //   [2:1]   reserved
+  //   [1:1]   PCLMULQDQ Instruction support
+  //   [2:2]   reserved
   //   [3:3]   MONITOR/MWAIT support
   //   [4:4]   DS-CPL: CPL qualified debug store
   //   [5:5]   VMX: Virtual Machine Technology
-  //   [6:6]   reserved
+  //   [6:6]   SMX: Secure Virtual Machine Technology
   //   [7:7]   EST: Enhanced Intel SpeedStep Technology
   //   [8:8]   TM2: Thermal Monitor 2
   //   [9:9]   SSE3E: SSE3E Instructions (Intel Core Duo 2 new instructions)
   //   [10:10] CNXT-ID: L1 context ID
-  //   [12:11] reserved
+  //   [11:11] reserved
+  //   [12:12] FMA Instructions support
   //   [13:13] CMPXCHG16B: CMPXCHG16B instruction support
   //   [14:14] xTPR update control
-  //   [18:15] reserved
-  //   [19:19] SSE4.1: SSE4.1 Instructions
-  //   [20:20] SSE4.2: SSE4.2 (SSE4E) Instructions
-  //   [31:21] reserved
+  //   [17:15] reserved
+  //   [18:18] DCA - Direct Cache Access
+  //   [19:19] SSE4.1 Instructions
+  //   [20:20] SSE4.2 Instructions
+  //   [21:21] X2APIC
+  //   [22:22] MOVBE instruction
+  //   [23:23] POPCNT instruction
+  //   [24:24] reserved
+  //   [25:25] AES Instructions
+  //   [26:26] XSAVE extensions support
+  //   [27:27] OSXSAVE support
+  //   [28:28] AVX extensions support
+  //   [31:29] reserved
   cpuid->ecx = get_extended_cpuid_features();
 
   // EDX:       Feature Flags
@@ -458,6 +546,10 @@ void BX_CPU_C::set_cpuid_defaults(void)
   cpuid->ecx = 0;
   cpuid->edx = 0;
 
+  // do not report CPUID functions above 0x3 if cpuid_limit_winnt is set
+  // to workaround WinNT issue.
+  if (cpuid_limit_winnt) return;
+
   // ------------------------------------------------------
   // CPUID function 0x00000004 - Deterministic Cache Parameters
   cpuid = &(BX_CPU_THIS_PTR cpuid_std_function[4]);
@@ -485,6 +577,21 @@ void BX_CPU_C::set_cpuid_defaults(void)
   cpuid->edx = 0;
 #endif
 
+#if BX_SUPPORT_XSAVE
+  // ------------------------------------------------------
+  // CPUID function 0x0000000D
+  cpuid = &(BX_CPU_THIS_PTR cpuid_std_function[0xD]);
+
+  // EAX - XCR0 lower 32 bits
+  // EBX - Maximum size (in bytes) required by enabled features
+  // ECX - Maximum size (in bytes) required by CPU supported features
+  // EDX - XCR0 upper 32 bits
+  cpuid->eax = BX_CPU_THIS_PTR xcr0.get32();
+  cpuid->ebx = 512+64;
+  cpuid->ecx = 512+64;
+  cpuid->edx = 0;
+#endif
+
 #if BX_SUPPORT_SSE >= 2   // report Pentium 4 extended functions
 
   // ------------------------------------------------------
@@ -501,9 +608,9 @@ void BX_CPU_C::set_cpuid_defaults(void)
   cpuid->edx = 0;          // Reserved for Intel
   cpuid->ecx = 0;
 #else
-  cpuid->ebx = 0x68747541; // "Auth"
-  cpuid->edx = 0x69746e65; // "enti"
-  cpuid->ecx = 0x444d4163; // "cAMD"
+  memcpy(&(cpuid->ebx), vendor_string    , 4);
+  memcpy(&(cpuid->edx), vendor_string + 4, 4);
+  memcpy(&(cpuid->ecx), vendor_string + 8, 4);
 #endif
 
   // ------------------------------------------------------
@@ -521,7 +628,7 @@ void BX_CPU_C::set_cpuid_defaults(void)
   //     [1:1]   AMD CmpLegacy
   //     [2:2]   AMD Secure Virtual Machine Technology
   //     [3:3]   Extended APIC Space
-  //     [4:4]   Alternative CR8 (treat lock mov cr0 as mov cr8) 
+  //     [4:4]   Alternative CR8 (treat lock mov cr0 as mov cr8)
   //     [5:5]   LZCNT support
   //     [6:6]   SSE4A support
   //     [7:7]   Misaligned SSE support
@@ -537,7 +644,7 @@ void BX_CPU_C::set_cpuid_defaults(void)
   cpuid->ecx |= (1<<7);
 #endif
 
-  // EDX: 
+  // EDX:
   // Many of the bits in EDX are the same as EAX [*] for AMD
   // [*] [0:0]   FPU on chip
   // [*] [1:1]   VME: Virtual-8086 Mode enhancements
@@ -577,63 +684,38 @@ void BX_CPU_C::set_cpuid_defaults(void)
 #endif
 #if BX_SUPPORT_X86_64
   features |= (1 << 29) | (1 << 27) | (1 << 25) | (1 << 20) | (1 << 11);
+#if BX_SUPPORT_1G_PAGES
+  features |= (1 << 26);
+#endif
 #endif
   cpuid->edx = features;
 
-#if BX_CPU_VENDOR_INTEL
   // Processor Brand String, use the value that is returned
-  // by the first processor in the Pentium 4 family 
-  // (according to Intel manual)
+  // by the first processor in the Pentium 4 family
+  // (according to Intel manual or the AMD manual)
 
   // ------------------------------------------------------
   // CPUID function 0x80000002
+
   cpuid = &(BX_CPU_THIS_PTR cpuid_ext_function[2]);
-  cpuid->eax = 0x20202020; // "    "
-  cpuid->ebx = 0x20202020; // "    "
-  cpuid->ecx = 0x20202020; // "    "
-  cpuid->edx = 0x6E492020; // "  In"
+  memcpy(&(cpuid->eax), brand_string     , 4);
+  memcpy(&(cpuid->ebx), brand_string +  4, 4);
+  memcpy(&(cpuid->ecx), brand_string +  8, 4);
+  memcpy(&(cpuid->edx), brand_string + 12, 4);
 
   // CPUID function 0x80000003
   cpuid = &(BX_CPU_THIS_PTR cpuid_ext_function[3]);
-  cpuid->eax = 0x286C6574; // "tel("
-  cpuid->ebx = 0x50202952; // "R) P"
-  cpuid->ecx = 0x69746E65; // "enti"
-  cpuid->edx = 0x52286D75; // "um(R"
+  memcpy(&(cpuid->eax), brand_string + 16, 4);
+  memcpy(&(cpuid->ebx), brand_string + 20, 4);
+  memcpy(&(cpuid->ecx), brand_string + 24, 4);
+  memcpy(&(cpuid->edx), brand_string + 28, 4);
 
   // CPUID function 0x80000004
   cpuid = &(BX_CPU_THIS_PTR cpuid_ext_function[4]);
-  cpuid->eax = 0x20342029; // ") 4 "
-  cpuid->ebx = 0x20555043; // "CPU "
-  cpuid->ecx = 0x20202020; // "    "
-  cpuid->edx = 0x00202020; // "    "
-#else
-  // Processor Brand String, use the value given 
-  // in AMD manuals.
-
-  // ------------------------------------------------------
-  // CPUID function 0x80000002
-  cpuid = &(BX_CPU_THIS_PTR cpuid_ext_function[2]);
-  cpuid->eax = 0x20444D41; // "AMD "
-  cpuid->ebx = 0x6C687441; // "Athl"
-  cpuid->ecx = 0x74286E6F; // "on(t"
-  cpuid->edx = 0x7020296D; // "m) p"
-
-  // ------------------------------------------------------
-  // CPUID function 0x80000003
-  cpuid = &(BX_CPU_THIS_PTR cpuid_ext_function[3]);
-  cpuid->eax = 0x65636F72; // "roce"
-  cpuid->ebx = 0x726F7373; // "ssor"
-  cpuid->ecx = 0x00000000;
-  cpuid->edx = 0x00000000;
-
-  // ------------------------------------------------------
-  // CPUID function 0x80000004
-  cpuid = &(BX_CPU_THIS_PTR cpuid_ext_function[4]);
-  cpuid->eax = 0x00000000;
-  cpuid->ebx = 0x00000000;
-  cpuid->ecx = 0x00000000;
-  cpuid->edx = 0x00000000;
-#endif
+  memcpy(&(cpuid->eax), brand_string + 32, 4);
+  memcpy(&(cpuid->ebx), brand_string + 36, 4);
+  memcpy(&(cpuid->ecx), brand_string + 40, 4);
+  memcpy(&(cpuid->edx), brand_string + 44, 4);
 
 #if BX_SUPPORT_X86_64
   // ------------------------------------------------------
@@ -658,7 +740,7 @@ void BX_CPU_C::set_cpuid_defaults(void)
 
   // ------------------------------------------------------
   // CPUID function 0x00000007
-  cpuid = &(BX_CPU_THIS_PTR cpuid_std_function[7]);
+  cpuid = &(BX_CPU_THIS_PTR cpuid_ext_function[7]);
 
   cpuid->eax = 0;
   cpuid->ebx = 0;
