@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: data_xfer64.cc,v 1.37 2008/05/10 18:10:52 sshwarts Exp $
+// $Id: data_xfer64.cc,v 1.43 2009/04/06 18:44:28 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,7 +22,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA B 02110-1301 USA
 /////////////////////////////////////////////////////////////////////////
 
 #define NEED_CPU_REG_SHORTCUTS 1
@@ -44,23 +44,33 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_RRXIq(bxInstruction_c *i)
   BX_WRITE_64BIT_REG(i->opcodeReg(), i->Iq());
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_EqGqM(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV64_GdEdM(bxInstruction_c *i)
 {
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  Bit64u eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  write_virtual_qword_64(i->seg(), RMAddr(i), BX_READ_64BIT_REG(i->nnn()));
+  Bit32u val32 = read_virtual_dword_64(i->seg(), eaddr);
+  BX_WRITE_32BIT_REGZ(i->nnn(), val32);
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_EqGqR(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV64_EdGdM(bxInstruction_c *i)
 {
-  BX_WRITE_64BIT_REG(i->rm(), BX_READ_64BIT_REG(i->nnn()));
+  Bit64u eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  write_virtual_dword_64(i->seg(), eaddr, BX_READ_32BIT_REG(i->nnn()));
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_EqGqM(bxInstruction_c *i)
+{
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+
+  write_virtual_qword_64(i->seg(), eaddr, BX_READ_64BIT_REG(i->nnn()));
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_GqEqM(bxInstruction_c *i)
 {
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit64u val64 = read_virtual_qword_64(i->seg(), RMAddr(i));
+  Bit64u val64 = read_virtual_qword_64(i->seg(), eaddr);
   BX_WRITE_64BIT_REG(i->nnn(), val64);
 }
 
@@ -71,9 +81,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_GqEqR(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::LEA_GqM(bxInstruction_c *i)
 {
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  BX_WRITE_64BIT_REG(i->nnn(), RMAddr(i));
+  BX_WRITE_64BIT_REG(i->nnn(), eaddr);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_ALOq(bxInstruction_c *i)
@@ -120,9 +130,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_EqIdM(bxInstruction_c *i)
 {
   Bit64u op_64 = (Bit32s) i->Id();
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  write_virtual_qword_64(i->seg(), RMAddr(i), op_64);
+  write_virtual_qword_64(i->seg(), eaddr, op_64);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_EqIdR(bxInstruction_c *i)
@@ -133,9 +143,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOV_EqIdR(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVZX_GqEbM(bxInstruction_c *i)
 {
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit8u op2_8 = read_virtual_byte_64(i->seg(), RMAddr(i));
+  Bit8u op2_8 = read_virtual_byte_64(i->seg(), eaddr);
 
   /* zero extend byte op2 into qword op1 */
   BX_WRITE_64BIT_REG(i->nnn(), (Bit64u) op2_8);
@@ -151,9 +161,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVZX_GqEbR(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVZX_GqEwM(bxInstruction_c *i)
 {
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit16u op2_16 = read_virtual_word_64(i->seg(), RMAddr(i));
+  Bit16u op2_16 = read_virtual_word_64(i->seg(), eaddr);
 
   /* zero extend word op2 into qword op1 */
   BX_WRITE_64BIT_REG(i->nnn(), (Bit64u) op2_16);
@@ -169,9 +179,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVZX_GqEwR(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSX_GqEbM(bxInstruction_c *i)
 {
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit8u op2_8 = read_virtual_byte_64(i->seg(), RMAddr(i));
+  Bit8u op2_8 = read_virtual_byte_64(i->seg(), eaddr);
 
   /* sign extend byte op2 into qword op1 */
   BX_WRITE_64BIT_REG(i->nnn(), (Bit8s) op2_8);
@@ -187,9 +197,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSX_GqEbR(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSX_GqEwM(bxInstruction_c *i)
 {
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit16u op2_16 = read_virtual_word_64(i->seg(), RMAddr(i));
+  Bit16u op2_16 = read_virtual_word_64(i->seg(), eaddr);
 
   /* sign extend word op2 into qword op1 */
   BX_WRITE_64BIT_REG(i->nnn(), (Bit16s) op2_16);
@@ -205,9 +215,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSX_GqEwR(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::MOVSX_GqEdM(bxInstruction_c *i)
 {
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit32u op2_32 = read_virtual_dword_64(i->seg(), RMAddr(i));
+  Bit32u op2_32 = read_virtual_dword_64(i->seg(), eaddr);
 
   /* sign extend word op2 into qword op1 */
   BX_WRITE_64BIT_REG(i->nnn(), (Bit32s) op2_32);
@@ -225,9 +235,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XCHG_EqGqM(bxInstruction_c *i)
 {
   Bit64u op2_64, op1_64;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  op1_64 = read_RMW_virtual_qword_64(i->seg(), RMAddr(i));
+  op1_64 = read_RMW_virtual_qword_64(i->seg(), eaddr);
   op2_64 = BX_READ_64BIT_REG(i->nnn());
 
   write_RMW_virtual_qword(op2_64);
@@ -247,256 +257,96 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::XCHG_EqGqR(bxInstruction_c *i)
 //       of whether condition is true or not.  Thus, exceptions may
 //       occur even if the MOV does not take place.
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVO_GqEqM(bxInstruction_c *i)
-{
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit64u op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-
-  if (get_OF())
-    BX_WRITE_64BIT_REG(i->nnn(), op2_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVO_GqEqR(bxInstruction_c *i)
 {
   if (get_OF())
     BX_WRITE_64BIT_REG(i->nnn(), BX_READ_64BIT_REG(i->rm()));
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNO_GqEqM(bxInstruction_c *i)
-{
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit64u op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-
-  if (!get_OF())
-    BX_WRITE_64BIT_REG(i->nnn(), op2_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNO_GqEqR(bxInstruction_c *i)
 {
   if (!get_OF())
     BX_WRITE_64BIT_REG(i->nnn(), BX_READ_64BIT_REG(i->rm()));
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVB_GqEqM(bxInstruction_c *i)
-{
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit64u op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-
-  if (get_CF())
-    BX_WRITE_64BIT_REG(i->nnn(), op2_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVB_GqEqR(bxInstruction_c *i)
 {
   if (get_CF())
     BX_WRITE_64BIT_REG(i->nnn(), BX_READ_64BIT_REG(i->rm()));
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNB_GqEqM(bxInstruction_c *i)
-{
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit64u op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-
-  if (!get_CF())
-    BX_WRITE_64BIT_REG(i->nnn(), op2_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNB_GqEqR(bxInstruction_c *i)
 {
   if (!get_CF())
     BX_WRITE_64BIT_REG(i->nnn(), BX_READ_64BIT_REG(i->rm()));
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVZ_GqEqM(bxInstruction_c *i)
-{
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit64u op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-
-  if (get_ZF())
-    BX_WRITE_64BIT_REG(i->nnn(), op2_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVZ_GqEqR(bxInstruction_c *i)
 {
   if (get_ZF())
     BX_WRITE_64BIT_REG(i->nnn(), BX_READ_64BIT_REG(i->rm()));
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNZ_GqEqM(bxInstruction_c *i)
-{
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit64u op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-
-  if (!get_ZF())
-    BX_WRITE_64BIT_REG(i->nnn(), op2_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNZ_GqEqR(bxInstruction_c *i)
 {
   if (!get_ZF())
     BX_WRITE_64BIT_REG(i->nnn(), BX_READ_64BIT_REG(i->rm()));
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVBE_GqEqM(bxInstruction_c *i)
-{
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit64u op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-
-  if (get_CF() || get_ZF())
-    BX_WRITE_64BIT_REG(i->nnn(), op2_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVBE_GqEqR(bxInstruction_c *i)
 {
   if (get_CF() || get_ZF())
     BX_WRITE_64BIT_REG(i->nnn(), BX_READ_64BIT_REG(i->rm()));
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNBE_GqEqM(bxInstruction_c *i)
-{
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit64u op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-
-  if (! (get_CF() || get_ZF()))
-    BX_WRITE_64BIT_REG(i->nnn(), op2_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNBE_GqEqR(bxInstruction_c *i)
 {
   if (! (get_CF() || get_ZF()))
     BX_WRITE_64BIT_REG(i->nnn(), BX_READ_64BIT_REG(i->rm()));
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVS_GqEqM(bxInstruction_c *i)
-{
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit64u op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-
-  if (get_SF())
-    BX_WRITE_64BIT_REG(i->nnn(), op2_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVS_GqEqR(bxInstruction_c *i)
 {
   if (get_SF())
     BX_WRITE_64BIT_REG(i->nnn(), BX_READ_64BIT_REG(i->rm()));
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNS_GqEqM(bxInstruction_c *i)
-{
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit64u op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-
-  if (!get_SF())
-    BX_WRITE_64BIT_REG(i->nnn(), op2_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNS_GqEqR(bxInstruction_c *i)
 {
   if (!get_SF())
     BX_WRITE_64BIT_REG(i->nnn(), BX_READ_64BIT_REG(i->rm()));
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVP_GqEqM(bxInstruction_c *i)
-{
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit64u op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-
-  if (get_PF())
-    BX_WRITE_64BIT_REG(i->nnn(), op2_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVP_GqEqR(bxInstruction_c *i)
 {
   if (get_PF())
     BX_WRITE_64BIT_REG(i->nnn(), BX_READ_64BIT_REG(i->rm()));
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNP_GqEqM(bxInstruction_c *i)
-{
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit64u op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-
-  if (!get_PF())
-    BX_WRITE_64BIT_REG(i->nnn(), op2_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNP_GqEqR(bxInstruction_c *i)
 {
   if (!get_PF())
     BX_WRITE_64BIT_REG(i->nnn(), BX_READ_64BIT_REG(i->rm()));
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVL_GqEqM(bxInstruction_c *i)
-{
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit64u op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-
-  if (getB_SF() != getB_OF())
-    BX_WRITE_64BIT_REG(i->nnn(), op2_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVL_GqEqR(bxInstruction_c *i)
 {
   if (getB_SF() != getB_OF())
     BX_WRITE_64BIT_REG(i->nnn(), BX_READ_64BIT_REG(i->rm()));
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNL_GqEqM(bxInstruction_c *i)
-{
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit64u op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-
-  if (getB_SF() == getB_OF())
-    BX_WRITE_64BIT_REG(i->nnn(), op2_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNL_GqEqR(bxInstruction_c *i)
 {
   if (getB_SF() == getB_OF())
     BX_WRITE_64BIT_REG(i->nnn(), BX_READ_64BIT_REG(i->rm()));
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVLE_GqEqM(bxInstruction_c *i)
-{
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit64u op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-
-  if (get_ZF() || (getB_SF() != getB_OF()))
-    BX_WRITE_64BIT_REG(i->nnn(), op2_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVLE_GqEqR(bxInstruction_c *i)
 {
   if (get_ZF() || (getB_SF() != getB_OF()))
     BX_WRITE_64BIT_REG(i->nnn(), BX_READ_64BIT_REG(i->rm()));
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNLE_GqEqM(bxInstruction_c *i)
-{
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-  Bit64u op2_64 = read_virtual_qword_64(i->seg(), RMAddr(i));
-
-  if (! get_ZF() && (getB_SF() == getB_OF()))
-    BX_WRITE_64BIT_REG(i->nnn(), op2_64);
-}
-
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::CMOVNLE_GqEqR(bxInstruction_c *i)
 {
   if (! get_ZF() && (getB_SF() == getB_OF()))
