@@ -1,5 +1,5 @@
 ////////////////////////////////////////////////////////////////////////
-// $Id: jmp_far.cc,v 1.16 2008/05/25 15:53:29 sshwarts Exp $
+// $Id: jmp_far.cc,v 1.20 2009/04/14 13:43:21 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2005 Stanislav Shwartsman
@@ -17,7 +17,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA B 02110-1301 USA
 //
 ////////////////////////////////////////////////////////////////////////
 
@@ -77,31 +77,6 @@ BX_CPU_C::jump_protected(bxInstruction_c *i, Bit16u cs_raw, bx_address disp)
         BX_ERROR(("jump_protected: gate type %u unsupported in long mode", (unsigned) descriptor.type));
         exception(BX_GP_EXCEPTION, cs_raw & 0xfffc, 0);
       }
-    }
-    else
-#endif
-    {
-      switch (descriptor.type) {
-        case BX_SYS_SEGMENT_AVAIL_286_TSS:
-        case BX_SYS_SEGMENT_AVAIL_386_TSS:
-        case BX_286_CALL_GATE:
-        case BX_386_CALL_GATE:
-        case BX_TASK_GATE:
-          break;
-        default:
-          BX_ERROR(("jump_protected: gate type %u unsupported", (unsigned) descriptor.type));
-         exception(BX_GP_EXCEPTION, cs_raw & 0xfffc, 0);
-      }
-    }
-
-    // task gate must be present else #NP(gate selector)
-    if (! IS_PRESENT(descriptor)) {
-      BX_ERROR(("jump_protected: call gate.p == 0"));
-      exception(BX_NP_EXCEPTION, cs_raw & 0xfffc, 0);
-    }
-
-#if BX_SUPPORT_X86_64
-    if (long_mode()) {
       jmp_call_gate64(&selector);
       return;
     }
@@ -117,7 +92,7 @@ BX_CPU_C::jump_protected(bxInstruction_c *i, Bit16u cs_raw, bx_address disp)
           BX_DEBUG(("jump_protected: jump to 386 TSS"));
 
         // SWITCH_TASKS _without_ nesting to TSS
-        task_switch(&selector, &descriptor, BX_TASK_FROM_JUMP, dword1, dword2);
+        task_switch(i, &selector, &descriptor, BX_TASK_FROM_JUMP, dword1, dword2);
 
         // EIP must be in code seg limit, else #GP(0)
         if (EIP > BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.limit_scaled) {
@@ -127,23 +102,22 @@ BX_CPU_C::jump_protected(bxInstruction_c *i, Bit16u cs_raw, bx_address disp)
         return;
 
       case BX_TASK_GATE:
-        jmp_task_gate(&descriptor);
+        task_gate(i, &selector, &descriptor, BX_TASK_FROM_JUMP);
         return;
 
       case BX_286_CALL_GATE:
       case BX_386_CALL_GATE:
-        jmp_call_gate(&descriptor);
+        jmp_call_gate(&selector, &descriptor);
         return;
 
-      default: // can't get here
-        BX_PANIC(("jump_protected: gate type %u unsupported", (unsigned) descriptor.type));
+      default:
+        BX_ERROR(("jump_protected: gate type %u unsupported", (unsigned) descriptor.type));
         exception(BX_GP_EXCEPTION, cs_raw & 0xfffc, 0);
     }
   }
 }
 
-  void BX_CPP_AttrRegparmN(1)
-BX_CPU_C::jmp_task_gate(bx_descriptor_t *gate_descriptor)
+void BX_CPU_C::task_gate(bxInstruction_c *i, bx_selector_t *selector, bx_descriptor_t *gate_descriptor, unsigned source)
 {
   Bit16u          raw_tss_selector;
   bx_selector_t   tss_selector;
@@ -151,13 +125,19 @@ BX_CPU_C::jmp_task_gate(bx_descriptor_t *gate_descriptor)
   Bit32u dword1, dword2;
   Bit32u temp_eIP;
 
+  // task gate must be present else #NP(gate selector)
+  if (! gate_descriptor->p) {
+    BX_ERROR(("task_gate: task gate not present"));
+    exception(BX_NP_EXCEPTION, selector->value & 0xfffc, 0);
+  }
+
   // examine selector to TSS, given in Task Gate descriptor
   // must specify global in the local/global bit else #GP(TSS selector)
   raw_tss_selector = gate_descriptor->u.taskgate.tss_selector;
   parse_selector(raw_tss_selector, &tss_selector);
 
   if (tss_selector.ti) {
-    BX_ERROR(("jmp_task_gate: tss_selector.ti=1"));
+    BX_ERROR(("task_gate: tss_selector.ti=1"));
     exception(BX_GP_EXCEPTION, raw_tss_selector & 0xfffc, 0);
   }
 
@@ -169,24 +149,24 @@ BX_CPU_C::jmp_task_gate(bx_descriptor_t *gate_descriptor)
   parse_descriptor(dword1, dword2, &tss_descriptor);
 
   if (tss_descriptor.valid==0 || tss_descriptor.segment) {
-    BX_ERROR(("jmp_task_gate: TSS selector points to bad TSS"));
+    BX_ERROR(("task_gate: TSS selector points to bad TSS"));
     exception(BX_GP_EXCEPTION, raw_tss_selector & 0xfffc, 0);
   }
   if (tss_descriptor.type!=BX_SYS_SEGMENT_AVAIL_286_TSS &&
       tss_descriptor.type!=BX_SYS_SEGMENT_AVAIL_386_TSS)
   {
-    BX_ERROR(("jmp_task_gate: TSS selector points to bad TSS"));
+    BX_ERROR(("task_gate: TSS selector points to bad TSS"));
     exception(BX_GP_EXCEPTION, raw_tss_selector & 0xfffc, 0);
   }
 
   // task state segment must be present, else #NP(tss selector)
   if (! IS_PRESENT(tss_descriptor)) {
-    BX_ERROR(("jmp_task_gate: TSS descriptor.p == 0"));
+    BX_ERROR(("task_gate: TSS descriptor.p == 0"));
     exception(BX_NP_EXCEPTION, raw_tss_selector & 0xfffc, 0);
   }
 
   // SWITCH_TASKS _without_ nesting to TSS
-  task_switch(&tss_selector, &tss_descriptor, BX_TASK_FROM_JUMP, dword1, dword2);
+  task_switch(i, &tss_selector, &tss_descriptor, source, dword1, dword2);
 
   // EIP must be within code segment limit, else #GP(0)
   if (BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.d_b)
@@ -195,13 +175,13 @@ BX_CPU_C::jmp_task_gate(bx_descriptor_t *gate_descriptor)
     temp_eIP =  IP;
 
   if (temp_eIP > BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].cache.u.segment.limit_scaled) {
-    BX_ERROR(("jmp_task_gate: EIP > CS.limit"));
+    BX_ERROR(("task_gate: EIP > CS.limit"));
     exception(BX_GP_EXCEPTION, 0, 0);
   }
 }
 
-  void BX_CPP_AttrRegparmN(1)
-BX_CPU_C::jmp_call_gate(bx_descriptor_t *gate_descriptor)
+  void BX_CPP_AttrRegparmN(2)
+BX_CPU_C::jmp_call_gate(bx_selector_t *selector, bx_descriptor_t *gate_descriptor)
 {
   bx_selector_t gate_cs_selector;
   bx_descriptor_t gate_cs_descriptor;
@@ -212,6 +192,12 @@ BX_CPU_C::jmp_call_gate(bx_descriptor_t *gate_descriptor)
   else
     BX_DEBUG(("jmp_call_gate: jump to 386 CALL GATE"));
 
+  // task gate must be present else #NP(gate selector)
+  if (! gate_descriptor->p) {
+    BX_ERROR(("jmp_call_gate: call gate not present!"));
+    exception(BX_NP_EXCEPTION, selector->value & 0xfffc, 0);
+  }
+
   // examine selector to code segment given in call gate descriptor
   // selector must not be null, else #GP(0)
   Bit16u gate_cs_raw = gate_descriptor->u.gate.dest_selector;
@@ -247,6 +233,12 @@ BX_CPU_C::jmp_call_gate64(bx_selector_t *gate_selector)
   fetch_raw_descriptor_64(gate_selector, &dword1, &dword2, &dword3, BX_GP_EXCEPTION);
   parse_descriptor(dword1, dword2, &gate_descriptor);
 
+  // gate must be present else #NP(gate selector)
+  if (! IS_PRESENT(gate_descriptor)) {
+    BX_ERROR(("jmp_call_gate64: call gate.p == 0"));
+    exception(BX_NP_EXCEPTION, gate_selector->value & 0xfffc, 0);
+  }
+
   Bit16u dest_selector = gate_descriptor.u.gate.dest_selector;
   // selector must not be null else #GP(0)
   if ((dest_selector & 0xfffc) == 0) {
