@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: mult16.cc,v 1.25 2007/12/20 20:58:37 sshwarts Exp $
+// $Id: mult16.cc,v 1.33 2009/01/16 18:18:58 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,34 +22,23 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA B 02110-1301 USA
 /////////////////////////////////////////////////////////////////////////
 
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-
-void BX_CPU_C::MUL_AXEw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MUL_AXEwR(bxInstruction_c *i)
 {
-  Bit16u op1_16, op2_16;
-
-  op1_16 = AX;
-
-  /* op2 is a register or memory reference */
-  if (i->modC0()) {
-    op2_16 = BX_READ_16BIT_REG(i->rm());
-  }
-  else {
-    /* pointer, segment address pair */
-    op2_16 = read_virtual_word(i->seg(), RMAddr(i));
-  }
+  Bit16u op1_16 = AX;
+  Bit16u op2_16 = BX_READ_16BIT_REG(i->rm());
 
   Bit32u product_32  = ((Bit32u) op1_16) * ((Bit32u) op2_16);
   Bit16u product_16l = (product_32 & 0xFFFF);
   Bit16u product_16h =  product_32 >> 16;
+
   /* now write product back to destination */
   AX = product_16l;
   DX = product_16h;
@@ -62,20 +51,10 @@ void BX_CPU_C::MUL_AXEw(bxInstruction_c *i)
   }
 }
 
-void BX_CPU_C::IMUL_AXEw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::IMUL_AXEwR(bxInstruction_c *i)
 {
-  Bit16s op1_16, op2_16;
-
-  op1_16 = AX;
-
-  /* op2 is a register or memory reference */
-  if (i->modC0()) {
-    op2_16 = BX_READ_16BIT_REG(i->rm());
-  }
-  else {
-    /* pointer, segment address pair */
-    op2_16 = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
-  }
+  Bit16s op1_16 = AX;
+  Bit16s op2_16 = BX_READ_16BIT_REG(i->rm());
 
   Bit32s product_32  = ((Bit32s) op1_16) * ((Bit32s) op2_16);
   Bit16u product_16l = (product_32 & 0xFFFF);
@@ -96,102 +75,55 @@ void BX_CPU_C::IMUL_AXEw(bxInstruction_c *i)
   }
 }
 
-void BX_CPU_C::DIV_AXEw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::DIV_AXEwR(bxInstruction_c *i)
 {
-  Bit16u op2_16, remainder_16, quotient_16l;
-  Bit32u op1_32, quotient_32;
-
-  op1_32 = (((Bit32u) DX) << 16) | ((Bit32u) AX);
-
-  /* op2 is a register or memory reference */
-  if (i->modC0()) {
-    op2_16 = BX_READ_16BIT_REG(i->rm());
-  }
-  else {
-    /* pointer, segment address pair */
-    op2_16 = read_virtual_word(i->seg(), RMAddr(i));
-  }
-
+  Bit16u op2_16 = BX_READ_16BIT_REG(i->rm());
   if (op2_16 == 0)
     exception(BX_DE_EXCEPTION, 0, 0);
 
-  quotient_32  = op1_32 / op2_16;
-  remainder_16 = op1_32 % op2_16;
-  quotient_16l = quotient_32 & 0xFFFF;
+  Bit32u op1_32 = (((Bit32u) DX) << 16) | ((Bit32u) AX);
+
+  Bit32u quotient_32  = op1_32 / op2_16;
+  Bit16u remainder_16 = op1_32 % op2_16;
+  Bit16u quotient_16l = quotient_32 & 0xFFFF;
 
   if (quotient_32 != quotient_16l)
     exception(BX_DE_EXCEPTION, 0, 0);
 
-  /* set EFLAGS:
-   * DIV affects the following flags: O,S,Z,A,P,C are undefined
-   */
-
-#if INTEL_DIV_FLAG_BUG == 1
-  assert_CF();
-#endif
-
   /* now write quotient back to destination */
   AX = quotient_16l;
   DX = remainder_16;
 }
 
-void BX_CPU_C::IDIV_AXEw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::IDIV_AXEwR(bxInstruction_c *i)
 {
-  Bit16s op2_16, remainder_16, quotient_16l;
-  Bit32s op1_32, quotient_32;
+  Bit32s op1_32 = ((((Bit32u) DX) << 16) | ((Bit32u) AX));
 
-  op1_32 = ((((Bit32u) DX) << 16) | ((Bit32u) AX));
+  /* check MIN_INT case */
+  if (op1_32 == ((Bit32s)0x80000000))
+    exception(BX_DE_EXCEPTION, 0, 0);
 
-  /* op2 is a register or memory reference */
-  if (i->modC0()) {
-    op2_16 = BX_READ_16BIT_REG(i->rm());
-  }
-  else {
-    /* pointer, segment address pair */
-    op2_16 = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
-  }
+  Bit16s op2_16 = BX_READ_16BIT_REG(i->rm());
 
   if (op2_16 == 0)
     exception(BX_DE_EXCEPTION, 0, 0);
 
-  /* check MIN_INT divided by -1 case */
-  if ((op1_32 == ((Bit32s)0x80000000)) && (op2_16 == -1))
-    exception(BX_DE_EXCEPTION, 0, 0);
-
-  quotient_32  = op1_32 / op2_16;
-  remainder_16 = op1_32 % op2_16;
-  quotient_16l = quotient_32 & 0xFFFF;
+  Bit32s quotient_32  = op1_32 / op2_16;
+  Bit16s remainder_16 = op1_32 % op2_16;
+  Bit16s quotient_16l = quotient_32 & 0xFFFF;
 
   if (quotient_32 != quotient_16l)
     exception(BX_DE_EXCEPTION, 0, 0);
 
-  /* set EFLAGS:
-   * IDIV affects the following flags: O,S,Z,A,P,C are undefined
-   */
-
-#if INTEL_DIV_FLAG_BUG == 1
-  assert_CF();
-#endif
-
   /* now write quotient back to destination */
   AX = quotient_16l;
   DX = remainder_16;
 }
 
-void BX_CPU_C::IMUL_GwEwIw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::IMUL_GwEwIwR(bxInstruction_c *i)
 {
-  Bit16s op2_16, op3_16;
-
-  op3_16 = i->Iw();
-
-  /* op2 is a register or memory reference */
-  if (i->modC0()) {
-    op2_16 = BX_READ_16BIT_REG(i->rm());
-  }
-  else {
-    /* pointer, segment address pair */
-    op2_16 = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
-  }
+  Bit16s op2_16 = BX_READ_16BIT_REG(i->rm());
+  Bit16s op3_16 = i->Iw();
 
   Bit32s product_32  = op2_16 * op3_16;
   Bit16u product_16 = (product_32 & 0xFFFF);
@@ -210,22 +142,12 @@ void BX_CPU_C::IMUL_GwEwIw(bxInstruction_c *i)
   }
 }
 
-void BX_CPU_C::IMUL_GwEw(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::IMUL_GwEwR(bxInstruction_c *i)
 {
-  Bit16s op1_16, op2_16;
+  Bit16s op1_16 = BX_READ_16BIT_REG(i->nnn());
+  Bit16s op2_16 = BX_READ_16BIT_REG(i->rm());
 
-  /* op2 is a register or memory reference */
-  if (i->modC0()) {
-    op2_16 = BX_READ_16BIT_REG(i->rm());
-  }
-  else {
-    /* pointer, segment address pair */
-    op2_16 = (Bit16s) read_virtual_word(i->seg(), RMAddr(i));
-  }
-
-  op1_16 = BX_READ_16BIT_REG(i->nnn());
-
-  Bit32s product_32  = op1_16 * op2_16;
+  Bit32s product_32 = op1_16 * op2_16;
   Bit16u product_16 = (product_32 & 0xFFFF);
 
   /* now write product back to destination */
