@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: mult8.cc,v 1.29 2008/05/24 10:26:03 sshwarts Exp $
+// $Id: mult8.cc,v 1.32 2009/01/16 18:18:58 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,7 +22,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA B 02110-1301 USA
 /////////////////////////////////////////////////////////////////////////
 
 #define NEED_CPU_REG_SHORTCUTS 1
@@ -30,21 +30,10 @@
 #include "cpu.h"
 #define LOG_THIS BX_CPU_THIS_PTR
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::MUL_ALEb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::MUL_ALEbR(bxInstruction_c *i)
 {
-  Bit8u op2, op1;
-
-  op1 = AL;
-
-  /* op2 is a register or memory reference */
-  if (i->modC0()) {
-    op2 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-  }
-  else {
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-    /* pointer, segment address pair */
-    op2 = read_virtual_byte(i->seg(), RMAddr(i));
-  }
+  Bit8u op1 = AL;
+  Bit8u op2 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
 
   Bit32u product_16 = ((Bit16u) op1) * ((Bit16u) op2);
 
@@ -62,21 +51,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::MUL_ALEb(bxInstruction_c *i)
   }
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::IMUL_ALEb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::IMUL_ALEbR(bxInstruction_c *i)
 {
-  Bit8s op2, op1;
-
-  op1 = AL;
-
-  /* op2 is a register or memory reference */
-  if (i->modC0()) {
-    op2 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-  }
-  else {
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-    /* pointer, segment address pair */
-    op2 = (Bit8s) read_virtual_byte(i->seg(), RMAddr(i));
-  }
+  Bit8s op1 = AL;
+  Bit8s op2 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
 
   Bit16s product_16 = op1 * op2;
   Bit8u  product_8 = (product_16 & 0xFF);
@@ -96,30 +74,18 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::IMUL_ALEb(bxInstruction_c *i)
   }
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::DIV_ALEb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::DIV_ALEbR(bxInstruction_c *i)
 {
-  Bit8u op2, quotient_8l, remainder_8;
-  Bit16u quotient_16, op1;
-
-  op1 = AX;
-
-  /* op2 is a register or memory reference */
-  if (i->modC0()) {
-    op2 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-  }
-  else {
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-    /* pointer, segment address pair */
-    op2 = read_virtual_byte(i->seg(), RMAddr(i));
-  }
-
+  Bit8u op2 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
   if (op2 == 0) {
     exception(BX_DE_EXCEPTION, 0, 0);
   }
 
-  quotient_16 = op1 / op2;
-  remainder_8 = op1 % op2;
-  quotient_8l = quotient_16 & 0xFF;
+  Bit16u op1 = AX;
+
+  Bit16u quotient_16 = op1 / op2;
+  Bit8u remainder_8 = op1 % op2;
+  Bit8u quotient_8l = quotient_16 & 0xFF;
 
   if (quotient_16 != quotient_8l)
   {
@@ -131,33 +97,22 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::DIV_ALEb(bxInstruction_c *i)
   AH = remainder_8;
 }
 
-void BX_CPP_AttrRegparmN(1) BX_CPU_C::IDIV_ALEb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::IDIV_ALEbR(bxInstruction_c *i)
 {
-  Bit8s op2, quotient_8l, remainder_8;
-  Bit16s quotient_16, op1;
-
-  op1 = AX;
-
-  /* op2 is a register or memory reference */
-  if (i->modC0()) {
-    op2 = BX_READ_8BIT_REGx(i->rm(),i->extend8bitL());
-  }
-  else {
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-    /* pointer, segment address pair */
-    op2 = (Bit8s) read_virtual_byte(i->seg(), RMAddr(i));
-  }
-
-  if (op2 == 0)
-    exception(BX_DE_EXCEPTION, 0, 0);
+  Bit16s op1 = AX;
 
   /* check MIN_INT case */
   if (op1 == ((Bit16s)0x8000))
     exception(BX_DE_EXCEPTION, 0, 0);
 
-  quotient_16 = op1 / op2;
-  remainder_8 = op1 % op2;
-  quotient_8l = quotient_16 & 0xFF;
+  Bit8s op2 = BX_READ_8BIT_REGx(i->rm(), i->extend8bitL());
+
+  if (op2 == 0)
+    exception(BX_DE_EXCEPTION, 0, 0);
+
+  Bit16s quotient_16 = op1 / op2;
+  Bit8s remainder_8 = op1 % op2;
+  Bit8s quotient_8l = quotient_16 & 0xFF;
 
   if (quotient_16 != quotient_8l)
     exception(BX_DE_EXCEPTION, 0, 0);
