@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: protect_ctrl.cc,v 1.87 2008/05/26 21:51:46 sshwarts Exp $
+// $Id: protect_ctrl.cc,v 1.93 2009/04/05 19:09:44 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,7 +22,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA B 02110-1301 USA
 /////////////////////////////////////////////////////////////////////////
 
 #define NEED_CPU_REG_SHORTCUTS 1
@@ -36,7 +36,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ARPL_EwGw(bxInstruction_c *i)
 
   if (real_mode() || v8086_mode()) {
     BX_DEBUG(("ARPL: not recognized in real or virtual-8086 mode"));
-    UndefinedOpcode(i);
+    exception(BX_UD_EXCEPTION, 0, 0);
   }
 
   /* op1_16 is a register or memory reference */
@@ -44,9 +44,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ARPL_EwGw(bxInstruction_c *i)
     op1_16 = BX_READ_16BIT_REG(i->rm());
   }
   else {
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+    bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
-    op1_16 = read_RMW_virtual_word(i->seg(), RMAddr(i));
+    op1_16 = read_RMW_virtual_word(i->seg(), eaddr);
   }
 
   op2_16 = BX_READ_16BIT_REG(i->nnn());
@@ -77,16 +77,16 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LAR_GvEw(bxInstruction_c *i)
 
   if (real_mode() || v8086_mode()) {
     BX_ERROR(("LAR: not recognized in real or virtual-8086 mode"));
-    UndefinedOpcode(i);
+    exception(BX_UD_EXCEPTION, 0, 0);
   }
 
   if (i->modC0()) {
     raw_selector = BX_READ_16BIT_REG(i->rm());
   }
   else {
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+    bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
-    raw_selector = read_virtual_word(i->seg(), RMAddr(i));
+    raw_selector = read_virtual_word(i->seg(), eaddr);
   }
 
   /* if selector null, clear ZF and done */
@@ -185,16 +185,16 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LSL_GvEw(bxInstruction_c *i)
 
   if (real_mode() || v8086_mode()) {
     BX_ERROR(("LSL: not recognized in real or virtual-8086 mode"));
-    UndefinedOpcode(i);
+    exception(BX_UD_EXCEPTION, 0, 0);
   }
 
   if (i->modC0()) {
     raw_selector = BX_READ_16BIT_REG(i->rm());
   }
   else {
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+    bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
-    raw_selector = read_virtual_word(i->seg(), RMAddr(i));
+    raw_selector = read_virtual_word(i->seg(), eaddr);
   }
 
   /* if selector null, clear ZF and done */
@@ -267,7 +267,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SLDT_Ew(bxInstruction_c *i)
 {
   if (real_mode() || v8086_mode()) {
     BX_ERROR(("SLDT: not recognized in real or virtual-8086 mode"));
-    UndefinedOpcode(i);
+    exception(BX_UD_EXCEPTION, 0, 0);
   }
 
   Bit16u val16 = BX_CPU_THIS_PTR ldtr.selector.value;
@@ -280,9 +280,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SLDT_Ew(bxInstruction_c *i)
     }
   }
   else {
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+    bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
-    write_virtual_word(i->seg(), RMAddr(i), val16);
+    write_virtual_word(i->seg(), eaddr, val16);
   }
 }
 
@@ -290,7 +290,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::STR_Ew(bxInstruction_c *i)
 {
   if (real_mode() || v8086_mode()) {
     BX_ERROR(("STR: not recognized in real or virtual-8086 mode"));
-    UndefinedOpcode(i);
+    exception(BX_UD_EXCEPTION, 0, 0);
   }
 
   Bit16u val16 = BX_CPU_THIS_PTR tr.selector.value;
@@ -303,9 +303,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::STR_Ew(bxInstruction_c *i)
     }
   }
   else {
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+    bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
-    write_virtual_word(i->seg(), RMAddr(i), val16);
+    write_virtual_word(i->seg(), eaddr, val16);
   }
 }
 
@@ -322,7 +322,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LLDT_Ew(bxInstruction_c *i)
 
   if (real_mode() || v8086_mode()) {
     BX_ERROR(("LLDT: not recognized in real or virtual-8086 mode"));
-    UndefinedOpcode(i);
+    exception(BX_UD_EXCEPTION, 0, 0);
   }
 
   if (CPL != 0) {
@@ -334,9 +334,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LLDT_Ew(bxInstruction_c *i)
     raw_selector = BX_READ_16BIT_REG(i->rm());
   }
   else {
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+    bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
-    raw_selector = read_virtual_word(i->seg(), RMAddr(i));
+    raw_selector = read_virtual_word(i->seg(), eaddr);
   }
 
   invalidate_prefetch_q();
@@ -385,10 +385,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LLDT_Ew(bxInstruction_c *i)
   }
 
 #if BX_SUPPORT_X86_64
-  descriptor.u.system.base |= ((Bit64u)(dword3) << 32);
+  descriptor.u.segment.base |= ((Bit64u)(dword3) << 32);
   BX_DEBUG(("64 bit LDT base = 0x%08x%08x",
-     GET32H(descriptor.u.system.base), GET32L(descriptor.u.system.base)));
-  if (!IsCanonical(descriptor.u.system.base)) {
+     GET32H(descriptor.u.segment.base), GET32L(descriptor.u.segment.base)));
+  if (!IsCanonical(descriptor.u.segment.base)) {
     BX_ERROR(("LLDT: non-canonical LDT descriptor base!"));
     exception(BX_GP_EXCEPTION, raw_selector & 0xfffc, 0);
   }
@@ -411,7 +411,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LTR_Ew(bxInstruction_c *i)
 
   if (real_mode() || v8086_mode()) {
     BX_ERROR(("LTR: not recognized in real or virtual-8086 mode"));
-    UndefinedOpcode(i);
+    exception(BX_UD_EXCEPTION, 0, 0);
   }
 
   if (CPL != 0) {
@@ -423,9 +423,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LTR_Ew(bxInstruction_c *i)
     raw_selector = BX_READ_16BIT_REG(i->rm());
   }
   else {
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+    bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
-    raw_selector = read_virtual_word(i->seg(), RMAddr(i));
+    raw_selector = read_virtual_word(i->seg(), eaddr);
   }
 
   invalidate_prefetch_q();
@@ -480,10 +480,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LTR_Ew(bxInstruction_c *i)
   }
 
 #if BX_SUPPORT_X86_64
-  descriptor.u.system.base |= ((Bit64u)(dword3) << 32);
+  descriptor.u.segment.base |= ((Bit64u)(dword3) << 32);
   BX_DEBUG(("64 bit TSS base = 0x%08x%08x",
-     GET32H(descriptor.u.system.base), GET32L(descriptor.u.system.base)));
-  if (!IsCanonical(descriptor.u.system.base)) {
+     GET32H(descriptor.u.segment.base), GET32L(descriptor.u.segment.base)));
+  if (!IsCanonical(descriptor.u.segment.base)) {
     BX_ERROR(("LTR: non-canonical TSS descriptor base!"));
     exception(BX_GP_EXCEPTION, raw_selector & 0xfffc, 0);
   }
@@ -514,16 +514,16 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VERR_Ew(bxInstruction_c *i)
 
   if (real_mode() || v8086_mode()) {
     BX_ERROR(("VERR: not recognized in real or virtual-8086 mode"));
-    UndefinedOpcode(i);
+    exception(BX_UD_EXCEPTION, 0, 0);
   }
 
   if (i->modC0()) {
     raw_selector = BX_READ_16BIT_REG(i->rm());
   }
   else {
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+    bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
-    raw_selector = read_virtual_word(i->seg(), RMAddr(i));
+    raw_selector = read_virtual_word(i->seg(), eaddr);
   }
 
   /* if selector null, clear ZF and done */
@@ -578,18 +578,19 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VERR_Ew(bxInstruction_c *i)
     if ((descriptor.dpl<CPL) || (descriptor.dpl<selector.rpl)) {
       BX_DEBUG(("VERR: non-conforming code not withing priv level"));
       clear_ZF();  /* inaccessible */
-      return;
     }
-
-    assert_ZF(); /* accessible */
+    else {
+      assert_ZF(); /* accessible */
+    }
   }
   else { /* data segment */
     if ((descriptor.dpl<CPL) || (descriptor.dpl<selector.rpl)) {
       BX_DEBUG(("VERR: data seg not withing priv level"));
       clear_ZF(); /* not accessible */
-      return;
     }
-    assert_ZF(); /* accessible */
+    else {
+      assert_ZF(); /* accessible */
+    }
   }
 }
 
@@ -603,16 +604,16 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VERW_Ew(bxInstruction_c *i)
 
   if (real_mode() || v8086_mode()) {
     BX_ERROR(("VERW: not recognized in real or virtual-8086 mode"));
-    UndefinedOpcode(i);
+    exception(BX_UD_EXCEPTION, 0, 0);
   }
 
   if (i->modC0()) {
     raw_selector = BX_READ_16BIT_REG(i->rm());
   }
   else {
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+    bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
-    raw_selector = read_virtual_word(i->seg(), RMAddr(i));
+    raw_selector = read_virtual_word(i->seg(), eaddr);
   }
 
   /* if selector null, clear ZF and done */
@@ -654,14 +655,15 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::VERW_Ew(bxInstruction_c *i)
     if ((descriptor.dpl<CPL) || (descriptor.dpl<selector.rpl)) {
       BX_DEBUG(("VERW: writable data seg not within priv level"));
       clear_ZF(); /* not accessible */
-      return;
     }
-    assert_ZF();  /* accessible */
-    return;
+    else {
+      assert_ZF();  /* accessible */
+    }
+  }
+  else {
+    BX_DEBUG(("VERW: data seg not writable"));
+    clear_ZF(); /* not accessible */
   }
-
-  BX_DEBUG(("VERW: data seg not writable"));
-  clear_ZF(); /* not accessible */
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::SGDT_Ms(bxInstruction_c *i)
@@ -669,10 +671,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SGDT_Ms(bxInstruction_c *i)
   Bit16u limit_16 = BX_CPU_THIS_PTR gdtr.limit;
   Bit32u base_32  = (Bit32u) BX_CPU_THIS_PTR gdtr.base;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  Bit32u eaddr = (Bit32u) BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  write_virtual_word(i->seg(), RMAddr(i), limit_16);
-  write_virtual_dword(i->seg(), RMAddr(i)+2, base_32);
+  write_virtual_word_32(i->seg(), eaddr, limit_16);
+  write_virtual_dword_32(i->seg(), eaddr+2, base_32);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::SIDT_Ms(bxInstruction_c *i)
@@ -680,10 +682,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SIDT_Ms(bxInstruction_c *i)
   Bit16u limit_16 = BX_CPU_THIS_PTR idtr.limit;
   Bit32u base_32  = (Bit32u) BX_CPU_THIS_PTR idtr.base;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  Bit32u eaddr = (Bit32u) BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  write_virtual_word(i->seg(), RMAddr(i), limit_16);
-  write_virtual_dword(i->seg(), RMAddr(i)+2, base_32);
+  write_virtual_word_32(i->seg(), eaddr, limit_16);
+  write_virtual_dword_32(i->seg(), eaddr+2, base_32);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::LGDT_Ms(bxInstruction_c *i)
@@ -700,10 +702,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LGDT_Ms(bxInstruction_c *i)
 
   invalidate_prefetch_q();
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  Bit32u eaddr = (Bit32u) BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit16u limit_16 = read_virtual_word(i->seg(), RMAddr(i));
-  Bit32u base_32 = read_virtual_dword(i->seg(), RMAddr(i) + 2);
+  Bit16u limit_16 = read_virtual_word_32(i->seg(), eaddr);
+  Bit32u base_32 = read_virtual_dword_32(i->seg(), eaddr + 2);
 
   if (i->os32L() == 0) base_32 &= 0x00ffffff; /* ignore upper 8 bits */
 
@@ -725,10 +727,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LIDT_Ms(bxInstruction_c *i)
 
   invalidate_prefetch_q();
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  Bit32u eaddr = (Bit32u) BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit32u base_32 = read_virtual_dword(i->seg(), RMAddr(i) + 2);
-  Bit16u limit_16 = read_virtual_word(i->seg(), RMAddr(i));
+  Bit32u base_32 = read_virtual_dword_32(i->seg(), eaddr + 2);
+  Bit16u limit_16 = read_virtual_word_32(i->seg(), eaddr);
 
   if (i->os32L() == 0) base_32 &= 0x00ffffff; /* ignore upper 8 bits */
 
@@ -743,10 +745,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SGDT64_Ms(bxInstruction_c *i)
   Bit16u limit_16 = BX_CPU_THIS_PTR gdtr.limit;
   Bit64u base_64  = BX_CPU_THIS_PTR gdtr.base;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  write_virtual_word_64(i->seg(), RMAddr(i), limit_16);
-  write_virtual_qword_64(i->seg(), RMAddr(i)+2, base_64);
+  write_virtual_word_64(i->seg(), eaddr, limit_16);
+  write_virtual_qword_64(i->seg(), eaddr+2, base_64);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::SIDT64_Ms(bxInstruction_c *i)
@@ -754,10 +756,10 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::SIDT64_Ms(bxInstruction_c *i)
   Bit16u limit_16 = BX_CPU_THIS_PTR idtr.limit;
   Bit64u base_64  = BX_CPU_THIS_PTR idtr.base;
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  write_virtual_word_64(i->seg(), RMAddr(i), limit_16);
-  write_virtual_qword_64(i->seg(), RMAddr(i)+2, base_64);
+  write_virtual_word_64(i->seg(), eaddr, limit_16);
+  write_virtual_qword_64(i->seg(), eaddr+2, base_64);
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::LGDT64_Ms(bxInstruction_c *i)
@@ -769,14 +771,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LGDT64_Ms(bxInstruction_c *i)
 
   invalidate_prefetch_q();
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit64u base_64 = read_virtual_qword_64(i->seg(), RMAddr(i) + 2);
+  Bit64u base_64 = read_virtual_qword_64(i->seg(), eaddr + 2);
   if (! IsCanonical(base_64)) {
     BX_ERROR(("LGDT64_Ms: loaded base64 address is not in canonical form!"));
     exception(BX_GP_EXCEPTION, 0, 0);
   }
-  Bit16u limit_16 = read_virtual_word_64(i->seg(), RMAddr(i));
+  Bit16u limit_16 = read_virtual_word_64(i->seg(), eaddr);
 
   BX_CPU_THIS_PTR gdtr.limit = limit_16;
   BX_CPU_THIS_PTR gdtr.base = base_64;
@@ -791,14 +793,14 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::LIDT64_Ms(bxInstruction_c *i)
 
   invalidate_prefetch_q();
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit64u base_64 = read_virtual_qword_64(i->seg(), RMAddr(i) + 2);
+  Bit64u base_64 = read_virtual_qword_64(i->seg(), eaddr + 2);
   if (! IsCanonical(base_64)) {
     BX_ERROR(("LIDT64_Ms: loaded base64 address is not in canonical form!"));
     exception(BX_GP_EXCEPTION, 0, 0);
   }
-  Bit16u limit_16 = read_virtual_word_64(i->seg(), RMAddr(i));
+  Bit16u limit_16 = read_virtual_word_64(i->seg(), eaddr);
 
   BX_CPU_THIS_PTR idtr.limit = limit_16;
   BX_CPU_THIS_PTR idtr.base = base_64;
