@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: sse_string.cc,v 1.2 2007/11/17 23:28:33 sshwarts Exp $
+// $Id: sse_string.cc,v 1.12 2009/01/16 18:18:59 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2007 Stanislav Shwartsman
@@ -17,11 +17,10 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA B 02110-1301 USA
 //
 /////////////////////////////////////////////////////////////////////////
 
-
 #define NEED_CPU_REG_SHORTCUTS 1
 #include "bochs.h"
 #include "cpu.h"
@@ -32,10 +31,10 @@
 #define RCX ECX
 #endif
 
-#if (BX_SUPPORT_SSE >= 5) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
+#if (BX_SUPPORT_SSE > 4) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
 
 // Compare all pairs of Ai, Bj according to imm8 control
-static void compare_strings(bx_bool BoolRes[16][16], BxPackedXmmRegister op1, BxPackedXmmRegister op2, Bit8u imm)
+static void compare_strings(Bit8u BoolRes[16][16], BxPackedXmmRegister op1, BxPackedXmmRegister op2, Bit8u imm)
 {
   unsigned i, j;
   unsigned aggregation_operation = (imm >> 2) & 3;
@@ -205,7 +204,7 @@ static bx_bool override_if_data_invalid(bx_bool val, bx_bool i_valid, bx_bool j_
   return val;
 }
 
-static Bit16u aggregate(bx_bool BoolRes[16][16], unsigned len1, unsigned len2, Bit8u imm)
+static Bit16u aggregate(Bit8u BoolRes[16][16], unsigned len1, unsigned len2, Bit8u imm)
 {
   unsigned aggregation_operation = (imm >> 2) & 3;
   unsigned num_elements = (imm & 0x1) ? 8 : 16;
@@ -240,7 +239,7 @@ static Bit16u aggregate(bx_bool BoolRes[16][16], unsigned len1, unsigned len2, B
           break;
         }
       }
- 
+
       if (res)
         result |= (1<<j);
     }
@@ -287,15 +286,15 @@ static Bit16u aggregate(bx_bool BoolRes[16][16], unsigned len1, unsigned len2, B
   return result;
 }
 
-#endif // (BX_SUPPORT_SSE >= 5) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
+#endif // (BX_SUPPORT_SSE > 4) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
 
 // for 3-byte opcodes
 #if (BX_SUPPORT_SSE >= 4) || (BX_SUPPORT_SSE >= 3 && BX_SUPPORT_SSE_EXTENSION > 0)
 
 /* 66 0F 3A 60 */
-void BX_CPU_C::PCMPESTRM_VdqWdqIb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::PCMPESTRM_VdqWdqIb(bxInstruction_c *i)
 {
-#if (BX_SUPPORT_SSE >= 5) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
+#if (BX_SUPPORT_SSE > 4) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
   BX_CPU_THIS_PTR prepareSSE();
 
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->nnn()), op2, result;
@@ -306,12 +305,13 @@ void BX_CPU_C::PCMPESTRM_VdqWdqIb(bxInstruction_c *i)
     op2 = BX_READ_XMM_REG(i->rm());
   }
   else {
+    bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
-    readVirtualDQwordAligned(i->seg(), RMAddr(i), (Bit8u *) &op2);
+    readVirtualDQwordAligned(i->seg(), eaddr, (Bit8u *) &op2);
   }
 
   // compare all pairs of Ai, Bj
-  bx_bool BoolRes[16][16];
+  Bit8u BoolRes[16][16];
   compare_strings(BoolRes, op1, op2, imm8);
   unsigned len1, len2, num_elements = (imm8 & 0x1) ? 8 : 16;
 
@@ -328,7 +328,7 @@ void BX_CPU_C::PCMPESTRM_VdqWdqIb(bxInstruction_c *i)
   }
   Bit16u result2 = aggregate(BoolRes, len1, len2, imm8);
 
-  // As defined by imm8[6], result2 is then either stored to the least 
+  // As defined by imm8[6], result2 is then either stored to the least
   // significant bits of XMM0 (zero extended to 128 bits) or expanded
   // into a byte/word-mask and then stored to XMM0
   if (imm8 & 0x40) {
@@ -354,18 +354,17 @@ void BX_CPU_C::PCMPESTRM_VdqWdqIb(bxInstruction_c *i)
     flags |= EFlagsOFMask;
   setEFlagsOSZAPC(flags);
 
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), result);
+  BX_WRITE_XMM_REG(0, result); /* store result XMM0 */
 #else
   BX_INFO(("PCMPESTRM_VdqWdqIb: required SSE4.2, use --enable-sse and --enable-sse-extension options"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
 /* 66 0F 3A 61 */
-void BX_CPU_C::PCMPESTRI_VdqWdqIb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::PCMPESTRI_VdqWdqIb(bxInstruction_c *i)
 {
-#if (BX_SUPPORT_SSE >= 5) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
+#if (BX_SUPPORT_SSE > 4) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
   BX_CPU_THIS_PTR prepareSSE();
 
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->nnn()), op2;
@@ -376,12 +375,13 @@ void BX_CPU_C::PCMPESTRI_VdqWdqIb(bxInstruction_c *i)
     op2 = BX_READ_XMM_REG(i->rm());
   }
   else {
+    bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
-    readVirtualDQwordAligned(i->seg(), RMAddr(i), (Bit8u *) &op2);
+    readVirtualDQwordAligned(i->seg(), eaddr, (Bit8u *) &op2);
   }
 
   // compare all pairs of Ai, Bj
-  bx_bool BoolRes[16][16];
+  Bit8u BoolRes[16][16];
   compare_strings(BoolRes, op1, op2, imm8);
   unsigned len1, len2, num_elements = (imm8 & 0x1) ? 8 : 16;
   int index;
@@ -424,14 +424,14 @@ void BX_CPU_C::PCMPESTRI_VdqWdqIb(bxInstruction_c *i)
 
 #else
   BX_INFO(("PCMPESTRI_VdqWdqIb: required SSE4.2, use --enable-sse and --enable-sse-extension options"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
 /* 66 0F 3A 62 */
-void BX_CPU_C::PCMPISTRM_VdqWdqIb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::PCMPISTRM_VdqWdqIb(bxInstruction_c *i)
 {
-#if (BX_SUPPORT_SSE >= 5) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
+#if (BX_SUPPORT_SSE > 4) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
   BX_CPU_THIS_PTR prepareSSE();
 
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->nnn()), op2, result;
@@ -442,12 +442,13 @@ void BX_CPU_C::PCMPISTRM_VdqWdqIb(bxInstruction_c *i)
     op2 = BX_READ_XMM_REG(i->rm());
   }
   else {
+    bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
-    readVirtualDQwordAligned(i->seg(), RMAddr(i), (Bit8u *) &op2);
+    readVirtualDQwordAligned(i->seg(), eaddr, (Bit8u *) &op2);
   }
 
   // compare all pairs of Ai, Bj
-  bx_bool BoolRes[16][16];
+  Bit8u BoolRes[16][16];
   compare_strings(BoolRes, op1, op2, imm8);
 
   unsigned num_elements = (imm8 & 0x1) ? 8 : 16;
@@ -455,7 +456,7 @@ void BX_CPU_C::PCMPISTRM_VdqWdqIb(bxInstruction_c *i)
   unsigned len2 = find_eos(op2, imm8);
   Bit16u result2 = aggregate(BoolRes, len1, len2, imm8);
 
-  // As defined by imm8[6], result2 is then either stored to the least 
+  // As defined by imm8[6], result2 is then either stored to the least
   // significant bits of XMM0 (zero extended to 128 bits) or expanded
   // into a byte/word-mask and then stored to XMM0
   if (imm8 & 0x40) {
@@ -481,18 +482,17 @@ void BX_CPU_C::PCMPISTRM_VdqWdqIb(bxInstruction_c *i)
     flags |= EFlagsOFMask;
   setEFlagsOSZAPC(flags);
 
-  /* now write result back to destination */
-  BX_WRITE_XMM_REG(i->nnn(), result);
+  BX_WRITE_XMM_REG(0, result); /* store result XMM0 */
 #else
   BX_INFO(("PCMPISTRM_VdqWdqIb: required SSE4.2, use --enable-sse and --enable-sse-extension options"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
 /* 66 0F 3A 63 */
-void BX_CPU_C::PCMPISTRI_VdqWdqIb(bxInstruction_c *i)
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::PCMPISTRI_VdqWdqIb(bxInstruction_c *i)
 {
-#if (BX_SUPPORT_SSE >= 5) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
+#if (BX_SUPPORT_SSE > 4) || (BX_SUPPORT_SSE >= 4 && BX_SUPPORT_SSE_EXTENSION > 0)
   BX_CPU_THIS_PTR prepareSSE();
 
   BxPackedXmmRegister op1 = BX_READ_XMM_REG(i->nnn()), op2;
@@ -503,12 +503,13 @@ void BX_CPU_C::PCMPISTRI_VdqWdqIb(bxInstruction_c *i)
     op2 = BX_READ_XMM_REG(i->rm());
   }
   else {
+    bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
     /* pointer, segment address pair */
-    readVirtualDQwordAligned(i->seg(), RMAddr(i), (Bit8u *) &op2);
+    readVirtualDQwordAligned(i->seg(), eaddr, (Bit8u *) &op2);
   }
 
   // compare all pairs of Ai, Bj
-  bx_bool BoolRes[16][16];
+  Bit8u BoolRes[16][16];
   compare_strings(BoolRes, op1, op2, imm8);
   unsigned num_elements = (imm8 & 0x1) ? 8 : 16;
   int index;
@@ -542,7 +543,7 @@ void BX_CPU_C::PCMPISTRI_VdqWdqIb(bxInstruction_c *i)
 
 #else
   BX_INFO(("PCMPISTRI_VdqWdqIb: required SSE4.2, use --enable-sse and --enable-sse-extension options"));
-  UndefinedOpcode(i);
+  exception(BX_UD_EXCEPTION, 0, 0);
 #endif
 }
 
