@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: stack16.cc,v 1.40 2008/05/08 18:02:21 sshwarts Exp $
+// $Id: stack16.cc,v 1.45.2.1 2009/06/07 07:49:10 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,7 +22,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA B 02110-1301 USA
 /////////////////////////////////////////////////////////////////////////
 
 #define NEED_CPU_REG_SHORTCUTS 1
@@ -72,64 +72,58 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::PUSH16_SS(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::POP16_DS(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR speculative_rsp = 1;
-  BX_CPU_THIS_PTR prev_rsp = RSP;
+  RSP_SPECULATIVE;
 
   Bit16u ds = pop_16();
   load_seg_reg(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS], ds);
 
-  BX_CPU_THIS_PTR speculative_rsp = 0;
+  RSP_COMMIT;
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::POP16_ES(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR speculative_rsp = 1;
-  BX_CPU_THIS_PTR prev_rsp = RSP;
+  RSP_SPECULATIVE;
 
   Bit16u es = pop_16();
   load_seg_reg(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES], es);
 
-  BX_CPU_THIS_PTR speculative_rsp = 0;
+  RSP_COMMIT;
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::POP16_FS(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR speculative_rsp = 1;
-  BX_CPU_THIS_PTR prev_rsp = RSP;
+  RSP_SPECULATIVE;
 
   Bit16u fs = pop_16();
   load_seg_reg(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS], fs);
 
-  BX_CPU_THIS_PTR speculative_rsp = 0;
+  RSP_COMMIT;
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::POP16_GS(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR speculative_rsp = 1;
-  BX_CPU_THIS_PTR prev_rsp = RSP;
+  RSP_SPECULATIVE;
 
   Bit16u gs = pop_16();
   load_seg_reg(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS], gs);
 
-  BX_CPU_THIS_PTR speculative_rsp = 0;
+  RSP_COMMIT;
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::POP16_SS(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR speculative_rsp = 1;
-  BX_CPU_THIS_PTR prev_rsp = RSP;
+  RSP_SPECULATIVE;
 
   Bit16u ss = pop_16();
   load_seg_reg(&BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS], ss);
 
-  BX_CPU_THIS_PTR speculative_rsp = 0;
+  RSP_COMMIT;
 
   // POP SS inhibits interrupts, debug exceptions and single-step
   // trap exceptions until the execution boundary following the
   // next instruction is reached.
   // Same code as MOV_SwEw()
-  BX_CPU_THIS_PTR inhibit_mask |=
-    BX_INHIBIT_INTERRUPTS | BX_INHIBIT_DEBUG;
+  BX_CPU_THIS_PTR inhibit_mask |= BX_INHIBIT_INTERRUPTS_BY_MOVSS;
   BX_CPU_THIS_PTR async_event = 1;
 }
 
@@ -140,19 +134,18 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::POP_RX(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::POP_EwM(bxInstruction_c *i)
 {
-  BX_CPU_THIS_PTR speculative_rsp = 1;
-  BX_CPU_THIS_PTR prev_rsp = RSP;
+  RSP_SPECULATIVE;
 
   Bit16u val16 = pop_16();
 
   // Note: there is one little weirdism here.  It is possible to use
   // SP in the modrm addressing. If used, the value of SP after the
   // pop is used to calculate the address.
-  BX_CPU_CALL_METHODR (i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  write_virtual_word(i->seg(), RMAddr(i), val16);
+  write_virtual_word(i->seg(), eaddr, val16);
 
-  BX_CPU_THIS_PTR speculative_rsp = 0;
+  RSP_COMMIT;
 }
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::PUSH_Iw(bxInstruction_c *i)
@@ -162,9 +155,9 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::PUSH_Iw(bxInstruction_c *i)
 
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::PUSH_EwM(bxInstruction_c *i)
 {
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit16u op1_16 = read_virtual_word(i->seg(), RMAddr(i));
+  Bit16u op1_16 = read_virtual_word(i->seg(), eaddr);
 
   push_16(op1_16);
 }
@@ -176,26 +169,26 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::PUSHAD16(bxInstruction_c *i)
 
   if (BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.d_b)
   {
-    write_virtual_word(BX_SEG_REG_SS, (Bit32u)(temp_ESP -  2), AX);
-    write_virtual_word(BX_SEG_REG_SS, (Bit32u)(temp_ESP -  4), CX);
-    write_virtual_word(BX_SEG_REG_SS, (Bit32u)(temp_ESP -  6), DX);
-    write_virtual_word(BX_SEG_REG_SS, (Bit32u)(temp_ESP -  8), BX);
-    write_virtual_word(BX_SEG_REG_SS, (Bit32u)(temp_ESP - 10), temp_SP);
-    write_virtual_word(BX_SEG_REG_SS, (Bit32u)(temp_ESP - 12), BP);
-    write_virtual_word(BX_SEG_REG_SS, (Bit32u)(temp_ESP - 14), SI);
-    write_virtual_word(BX_SEG_REG_SS, (Bit32u)(temp_ESP - 16), DI);
+    write_virtual_word_32(BX_SEG_REG_SS, (Bit32u)(temp_ESP -  2), AX);
+    write_virtual_word_32(BX_SEG_REG_SS, (Bit32u)(temp_ESP -  4), CX);
+    write_virtual_word_32(BX_SEG_REG_SS, (Bit32u)(temp_ESP -  6), DX);
+    write_virtual_word_32(BX_SEG_REG_SS, (Bit32u)(temp_ESP -  8), BX);
+    write_virtual_word_32(BX_SEG_REG_SS, (Bit32u)(temp_ESP - 10), temp_SP);
+    write_virtual_word_32(BX_SEG_REG_SS, (Bit32u)(temp_ESP - 12), BP);
+    write_virtual_word_32(BX_SEG_REG_SS, (Bit32u)(temp_ESP - 14), SI);
+    write_virtual_word_32(BX_SEG_REG_SS, (Bit32u)(temp_ESP - 16), DI);
     ESP -= 16;
   }
   else
   {
-    write_virtual_word(BX_SEG_REG_SS, (Bit16u)(temp_SP -  2), AX);
-    write_virtual_word(BX_SEG_REG_SS, (Bit16u)(temp_SP -  4), CX);
-    write_virtual_word(BX_SEG_REG_SS, (Bit16u)(temp_SP -  6), DX);
-    write_virtual_word(BX_SEG_REG_SS, (Bit16u)(temp_SP -  8), BX);
-    write_virtual_word(BX_SEG_REG_SS, (Bit16u)(temp_SP - 10), temp_SP);
-    write_virtual_word(BX_SEG_REG_SS, (Bit16u)(temp_SP - 12), BP);
-    write_virtual_word(BX_SEG_REG_SS, (Bit16u)(temp_SP - 14), SI);
-    write_virtual_word(BX_SEG_REG_SS, (Bit16u)(temp_SP - 16), DI);
+    write_virtual_word_32(BX_SEG_REG_SS, (Bit16u)(temp_SP -  2), AX);
+    write_virtual_word_32(BX_SEG_REG_SS, (Bit16u)(temp_SP -  4), CX);
+    write_virtual_word_32(BX_SEG_REG_SS, (Bit16u)(temp_SP -  6), DX);
+    write_virtual_word_32(BX_SEG_REG_SS, (Bit16u)(temp_SP -  8), BX);
+    write_virtual_word_32(BX_SEG_REG_SS, (Bit16u)(temp_SP - 10), temp_SP);
+    write_virtual_word_32(BX_SEG_REG_SS, (Bit16u)(temp_SP - 12), BP);
+    write_virtual_word_32(BX_SEG_REG_SS, (Bit16u)(temp_SP - 14), SI);
+    write_virtual_word_32(BX_SEG_REG_SS, (Bit16u)(temp_SP - 16), DI);
     SP -= 16;
   }
 }
@@ -207,27 +200,27 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::POPAD16(bxInstruction_c *i)
   if (BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.d_b)
   {
     Bit32u temp_ESP = ESP;
-    di = read_virtual_word(BX_SEG_REG_SS, (Bit32u)(temp_ESP +  0));
-    si = read_virtual_word(BX_SEG_REG_SS, (Bit32u)(temp_ESP +  2));
-    bp = read_virtual_word(BX_SEG_REG_SS, (Bit32u)(temp_ESP +  4));
-    dummy = read_virtual_word(BX_SEG_REG_SS, (Bit32u)(temp_ESP +  6));
-    bx = read_virtual_word(BX_SEG_REG_SS, (Bit32u)(temp_ESP +  8));
-    dx = read_virtual_word(BX_SEG_REG_SS, (Bit32u)(temp_ESP + 10));
-    cx = read_virtual_word(BX_SEG_REG_SS, (Bit32u)(temp_ESP + 12));
-    ax = read_virtual_word(BX_SEG_REG_SS, (Bit32u)(temp_ESP + 14));
+    di = read_virtual_word_32(BX_SEG_REG_SS, (Bit32u)(temp_ESP +  0));
+    si = read_virtual_word_32(BX_SEG_REG_SS, (Bit32u)(temp_ESP +  2));
+    bp = read_virtual_word_32(BX_SEG_REG_SS, (Bit32u)(temp_ESP +  4));
+    dummy = read_virtual_word_32(BX_SEG_REG_SS, (Bit32u)(temp_ESP +  6));
+    bx = read_virtual_word_32(BX_SEG_REG_SS, (Bit32u)(temp_ESP +  8));
+    dx = read_virtual_word_32(BX_SEG_REG_SS, (Bit32u)(temp_ESP + 10));
+    cx = read_virtual_word_32(BX_SEG_REG_SS, (Bit32u)(temp_ESP + 12));
+    ax = read_virtual_word_32(BX_SEG_REG_SS, (Bit32u)(temp_ESP + 14));
     ESP += 16;
   }
   else
   {
     Bit16u temp_SP = SP;
-    di = read_virtual_word(BX_SEG_REG_SS, (Bit16u)(temp_SP +  0));
-    si = read_virtual_word(BX_SEG_REG_SS, (Bit16u)(temp_SP +  2));
-    bp = read_virtual_word(BX_SEG_REG_SS, (Bit16u)(temp_SP +  4));
-    dummy = read_virtual_word(BX_SEG_REG_SS, (Bit16u)(temp_SP +  6));
-    bx = read_virtual_word(BX_SEG_REG_SS, (Bit16u)(temp_SP +  8));
-    dx = read_virtual_word(BX_SEG_REG_SS, (Bit16u)(temp_SP + 10));
-    cx = read_virtual_word(BX_SEG_REG_SS, (Bit16u)(temp_SP + 12));
-    ax = read_virtual_word(BX_SEG_REG_SS, (Bit16u)(temp_SP + 14));
+    di = read_virtual_word_32(BX_SEG_REG_SS, (Bit16u)(temp_SP +  0));
+    si = read_virtual_word_32(BX_SEG_REG_SS, (Bit16u)(temp_SP +  2));
+    bp = read_virtual_word_32(BX_SEG_REG_SS, (Bit16u)(temp_SP +  4));
+    dummy = read_virtual_word_32(BX_SEG_REG_SS, (Bit16u)(temp_SP +  6));
+    bx = read_virtual_word_32(BX_SEG_REG_SS, (Bit16u)(temp_SP +  8));
+    dx = read_virtual_word_32(BX_SEG_REG_SS, (Bit16u)(temp_SP + 10));
+    cx = read_virtual_word_32(BX_SEG_REG_SS, (Bit16u)(temp_SP + 12));
+    ax = read_virtual_word_32(BX_SEG_REG_SS, (Bit16u)(temp_SP + 14));
     SP += 16;
   }
 
@@ -246,8 +239,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ENTER16_IwIb(bxInstruction_c *i)
   Bit8u level = i->Ib2();
   level &= 0x1F;
 
-  BX_CPU_THIS_PTR speculative_rsp = 1;
-  BX_CPU_THIS_PTR prev_rsp = RSP;
+  RSP_SPECULATIVE;
 
   push_16(BP);
   Bit16u frame_ptr16 = SP;
@@ -259,7 +251,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ENTER16_IwIb(bxInstruction_c *i)
       /* do level-1 times */
       while (--level) {
         ebp -= 2;
-        Bit16u temp16 = read_virtual_word(BX_SEG_REG_SS, ebp);
+        Bit16u temp16 = read_virtual_word_32(BX_SEG_REG_SS, ebp);
         push_16(temp16);
       }
 
@@ -283,7 +275,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ENTER16_IwIb(bxInstruction_c *i)
       /* do level-1 times */
       while (--level) {
         bp -= 2;
-        Bit16u temp16 = read_virtual_word(BX_SEG_REG_SS, bp);
+        Bit16u temp16 = read_virtual_word_32(BX_SEG_REG_SS, bp);
         push_16(temp16);
       }
 
@@ -296,10 +288,28 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::ENTER16_IwIb(bxInstruction_c *i)
     // ENTER finishes with memory write check on the final stack pointer
     // the memory is touched but no write actually occurs
     // emulate it by doing RMW read access from SS:SP
-    read_RMW_virtual_word(BX_SEG_REG_SS, SP);
+    read_RMW_virtual_word_32(BX_SEG_REG_SS, SP);
   }
 
   BP = frame_ptr16;
 
-  BX_CPU_THIS_PTR speculative_rsp = 0;
+  RSP_COMMIT;
+}
+
+void BX_CPP_AttrRegparmN(1) BX_CPU_C::LEAVE16(bxInstruction_c *i)
+{
+  BX_ASSERT(BX_CPU_THIS_PTR cpu_mode != BX_MODE_LONG_64);
+
+  Bit16u value16;
+
+  if (BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].cache.u.segment.d_b) {
+    value16 = read_virtual_word_32(BX_SEG_REG_SS, EBP);
+    ESP = EBP + 2;
+  }
+  else {
+    value16 = read_virtual_word_32(BX_SEG_REG_SS, BP);
+    SP = BP + 2;
+  }
+
+  BP = value16;
 }
