@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: dis_decode.cc,v 1.46 2008/04/27 19:47:12 sshwarts Exp $
+// $Id: dis_decode.cc,v 1.49.2.1 2009/06/07 07:49:10 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 
 #include <stdio.h>
@@ -76,13 +76,13 @@ x86_insn disassembler::decode(bx_bool is_32, bx_bool is_64, bx_address base, bx_
 #define SSE_PREFIX_66   1
 #define SSE_PREFIX_F2   2
 #define SSE_PREFIX_F3   3      /* only one SSE prefix could be used */
-  unsigned sse_prefix = SSE_PREFIX_NONE;
-  unsigned rex_prefix = 0;
+  unsigned sse_prefix = SSE_PREFIX_NONE, sse_opcode = 0;
+  unsigned rex_prefix = 0, prefixes = 0;
 
   for(;;)
   {
     insn.b1 = fetch_byte();
-    insn.prefixes++;
+    prefixes++;
 
     switch(insn.b1) {
       case 0x40:     // rex
@@ -106,32 +106,16 @@ x86_insn disassembler::decode(bx_bool is_32, bx_bool is_64, bx_address base, bx_
         continue;
 
       case 0x26:     // ES:
-        if (! is_64) insn.seg_override = ES_REG;
-        rex_prefix = 0;
-        continue;
-
       case 0x2e:     // CS:
-        if (! is_64) insn.seg_override = CS_REG;
-        rex_prefix = 0;
-        continue;
-
       case 0x36:     // SS:
-        if (! is_64) insn.seg_override = SS_REG;
-        rex_prefix = 0;
-        continue;
-
       case 0x3e:     // DS:
-        if (! is_64) insn.seg_override = DS_REG;
+        if (! is_64) insn.seg_override = (insn.b1 >> 3) & 3;
         rex_prefix = 0;
         continue;
 
       case 0x64:     // FS:
-        insn.seg_override = FS_REG;
-        rex_prefix = 0;
-        continue;
-
       case 0x65:     // GS:
-        insn.seg_override = GS_REG;
+        insn.seg_override = insn.b1 & 0xf;
         rex_prefix = 0;
         continue;
 
@@ -152,11 +136,11 @@ x86_insn disassembler::decode(bx_bool is_32, bx_bool is_64, bx_address base, bx_
         continue;
 
       case 0xf2:     // repne
-        sse_prefix = SSE_PREFIX_F2;
+      case 0xf3:     // rep
+        sse_prefix = insn.b1 & 0xf;
         rex_prefix = 0;
         continue;
 
-      case 0xf3:     // rep
         sse_prefix = SSE_PREFIX_F3;
         rex_prefix = 0;
         continue;
@@ -166,7 +150,6 @@ x86_insn disassembler::decode(bx_bool is_32, bx_bool is_64, bx_address base, bx_
         break;
     }
 
-    insn.prefixes--;
     break;
   }
 
@@ -218,11 +201,11 @@ x86_insn disassembler::decode(bx_bool is_32, bx_bool is_64, bx_address base, bx_
   {
     switch(attr) {
        case _GROUPN:
-         entry = &(OPCODE_TABLE(entry)[insn.nnn]);
+         entry = &(OPCODE_TABLE(entry)[insn.nnn & 7]);
          break;
 
        case _GRPSSE:
-         if(sse_prefix) insn.prefixes--;
+         sse_opcode = 1;
          /* For SSE opcodes, look into another 4 entries table
             with the opcode prefixes (NONE, 0x66, 0xF2, 0xF3) */
          entry = &(OPCODE_TABLE(entry)[sse_prefix]);
@@ -233,13 +216,13 @@ x86_insn disassembler::decode(bx_bool is_32, bx_bool is_64, bx_address base, bx_
          break;
 
        case _GRPRM:
-         entry = &(OPCODE_TABLE(entry)[insn.rm]);
+         entry = &(OPCODE_TABLE(entry)[insn.rm & 7]);
          break;
 
        case _GRPFP:
          if(insn.mod != 3)
          {
-             entry = &(OPCODE_TABLE(entry)[insn.nnn]);
+             entry = &(OPCODE_TABLE(entry)[insn.nnn & 7]);
          } else {
              int index = (insn.b1-0xD8)*64 + (insn.modrm & 0x3f);
              entry = &(BxDisasmOpcodeInfoFP[index]);
@@ -273,7 +256,7 @@ x86_insn disassembler::decode(bx_bool is_32, bx_bool is_64, bx_address base, bx_
   unsigned branch_hint = 0;
 
   // print prefixes
-  for(unsigned i=0;i<insn.prefixes;i++)
+  for(unsigned i=0;i<prefixes;i++)
   {
     Bit8u prefix_byte = *(instr+i);
 
@@ -282,8 +265,11 @@ x86_insn disassembler::decode(bx_bool is_32, bx_bool is_64, bx_address base, bx_
       dis_sprintf("%s ", OPCODE(prefix)->IntelOpcode);
     }
 
+    if (insn.b1 == 0x90 && !insn.rex_b && prefix_byte == 0xF3)
+      continue;
+
     if (prefix_byte == 0xF3 || prefix_byte == 0xF2) {
-      if (attr != _GRPSSE) {
+      if (! sse_opcode) {
         const BxDisasmOpcodeTable_t *prefix = &(opcode_table[prefix_byte]);
         dis_sprintf("%s ", OPCODE(prefix)->IntelOpcode);
       }
@@ -305,8 +291,11 @@ x86_insn disassembler::decode(bx_bool is_32, bx_bool is_64, bx_address base, bx_
     opcode = &Ia_jecxz_Jb;
 
   // fix nop opcode
-  if (insn.b1 == 0x90 && !insn.rex_b) {
-    opcode = &Ia_nop;
+  if (insn.b1 == 0x90) {
+    if (sse_prefix == SSE_PREFIX_F3)
+      opcode = &Ia_pause;
+    else if (!insn.rex_b)
+      opcode = &Ia_nop;
   }
 
   // print instruction disassembly
