@@ -43,8 +43,8 @@
 #define IA_XSAVE            0x00800000        /* XSAVE/XRSTOR extensions instruction */
 #define IA_AES              0x01000000        /* AES instruction */
 #define IA_AVX              0x02000000        /* AVX instruction */
-#define IA_LEGACY           0x40000000        /* legacy instruction */
-#define IA_UNDOCUMENTED     0x80000000        /* instruction undocumented */
+#define IA_MOVBE            0x04000000        /* MOVBE Intel Atom(R) instruction */
+#define IA_LEGACY           0x80000000        /* legacy instruction */
 
 /* general purpose bit register */
 enum {
@@ -128,7 +128,7 @@ public:
   Bit8u extend8b;
   Bit8u rex_r, rex_x, rex_b;
   Bit8u seg_override;
-  unsigned b1, prefixes;
+  unsigned b1;
   unsigned ilen;
 
   Bit8u modrm, mod, nnn, rm;
@@ -160,7 +160,6 @@ BX_CPP_INLINE x86_insn::x86_insn(bx_bool is32, bx_bool is64)
   extend8b = 0;
   rex_r = rex_b = rex_x = 0;
   seg_override = NO_SEG_OVERRIDE;
-  prefixes = 0;
   ilen = 0;
   b1 = 0;
 
@@ -373,18 +372,18 @@ public:
   void Apd(const x86_insn *insn);
 
   // 8-bit general purpose registers
-  void AL(const x86_insn *insn);
-  void CL(const x86_insn *insn);
+  void AL_Reg(const x86_insn *insn);
+  void CL_Reg(const x86_insn *insn);
 
   // 16-bit general purpose registers
-  void AX(const x86_insn *insn);
-  void DX(const x86_insn *insn);
+  void AX_Reg(const x86_insn *insn);
+  void DX_Reg(const x86_insn *insn);
 
   // 32-bit general purpose registers
-  void EAX(const x86_insn *insn);
+  void EAX_Reg(const x86_insn *insn);
 
   // 64-bit general purpose registers
-  void RAX(const x86_insn *insn);
+  void RAX_Reg(const x86_insn *insn);
 
   // segment registers
   void CS(const x86_insn *insn);
@@ -409,7 +408,7 @@ public:
   void Dq(const x86_insn *insn);
 
   //  8-bit general purpose register
-  void R8(const x86_insn *insn);
+  void Reg8(const x86_insn *insn);
 
   // 16-bit general purpose register
   void RX(const x86_insn *insn);
