@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: fpu.cc,v 1.42 2008/05/19 20:00:42 sshwarts Exp $
+// $Id: fpu.cc,v 1.55 2009/04/27 14:00:55 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2003 Stanislav Shwartsman
@@ -17,7 +17,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 //
 /////////////////////////////////////////////////////////////////////////
 
@@ -29,34 +29,27 @@
 
 #include "iodev/iodev.h"
 
-#define UPDATE_LAST_OPCODE       1
 #define CHECK_PENDING_EXCEPTIONS 1
 
 #if BX_SUPPORT_FPU
-void BX_CPU_C::prepareFPU(bxInstruction_c *i,
-	bx_bool check_pending_exceptions, bx_bool update_last_instruction)
+void BX_CPU_C::prepareFPU(bxInstruction_c *i, bx_bool check_pending_exceptions)
 {
   if (BX_CPU_THIS_PTR cr0.get_EM() || BX_CPU_THIS_PTR cr0.get_TS())
     exception(BX_NM_EXCEPTION, 0, 0);
 
   if (check_pending_exceptions)
     BX_CPU_THIS_PTR FPU_check_pending_exceptions();
+}
 
-  if (update_last_instruction)
-  {
-    BX_CPU_THIS_PTR the_i387.foo = (((Bit32u)(i->b1()) << 8) | i->modrm()) & 0x7ff;
-    BX_CPU_THIS_PTR the_i387.fcs = BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value;
-    BX_CPU_THIS_PTR the_i387.fip = BX_CPU_THIS_PTR prev_rip;
+void BX_CPU_C::FPU_update_last_instruction(bxInstruction_c *i)
+{
+  BX_CPU_THIS_PTR the_i387.foo = (((Bit32u)(i->b1()) << 8) | i->modrm()) & 0x7ff;
+  BX_CPU_THIS_PTR the_i387.fcs = BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value;
+  BX_CPU_THIS_PTR the_i387.fip = BX_CPU_THIS_PTR prev_rip;
 
-    if (! i->modC0()) {
-       BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
-
-       BX_CPU_THIS_PTR the_i387.fds = BX_CPU_THIS_PTR sregs[i->seg()].selector.value;
-       BX_CPU_THIS_PTR the_i387.fdp = RMAddr(i);
-    } else {
-       BX_CPU_THIS_PTR the_i387.fds = BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].selector.value;
-       BX_CPU_THIS_PTR the_i387.fdp = 0;
-    }
+  if (! i->modC0()) {
+     BX_CPU_THIS_PTR the_i387.fds = BX_CPU_THIS_PTR sregs[i->seg()].selector.value;
+     BX_CPU_THIS_PTR the_i387.fdp = RMAddr(i);
   }
 }
 
@@ -82,8 +75,10 @@ void BX_CPU_C::FPU_check_pending_exceptions(void)
   }
 }
 
-int BX_CPU_C::fpu_save_environment(bxInstruction_c *i)
+bx_address BX_CPU_C::fpu_save_environment(bxInstruction_c *i)
 {
+    unsigned offset;
+
     /* read all registers in stack order and update x87 tag word */
     for(int n=0;n<8;n++) {
        // update tag only if it is not empty
@@ -93,7 +88,7 @@ int BX_CPU_C::fpu_save_environment(bxInstruction_c *i)
        }
     }
 
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+    bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
     if (protected_mode())  /* Protected Mode */
     {
@@ -102,43 +97,43 @@ int BX_CPU_C::fpu_save_environment(bxInstruction_c *i)
             Bit32u tmp;
 
             tmp = 0xffff0000 | BX_CPU_THIS_PTR the_i387.get_control_word();
-            write_virtual_dword(i->seg(), RMAddr(i), tmp);
+            write_virtual_dword(i->seg(), eaddr, tmp);
             tmp = 0xffff0000 | BX_CPU_THIS_PTR the_i387.get_status_word();
-            write_virtual_dword(i->seg(), RMAddr(i) + 0x04, tmp);
+            write_virtual_dword(i->seg(), eaddr + 0x04, tmp);
             tmp = 0xffff0000 | BX_CPU_THIS_PTR the_i387.get_tag_word();
-            write_virtual_dword(i->seg(), RMAddr(i) + 0x08, tmp);
+            write_virtual_dword(i->seg(), eaddr + 0x08, tmp);
             tmp = (Bit32u)(BX_CPU_THIS_PTR the_i387.fip);
-            write_virtual_dword(i->seg(), RMAddr(i) + 0x0c, tmp);
+            write_virtual_dword(i->seg(), eaddr + 0x0c, tmp);
             tmp  = (BX_CPU_THIS_PTR the_i387.fcs & 0xffff) |
                           ((Bit32u)(BX_CPU_THIS_PTR the_i387.foo)) << 16;
-            write_virtual_dword(i->seg(), RMAddr(i) + 0x10, tmp);
+            write_virtual_dword(i->seg(), eaddr + 0x10, tmp);
             tmp = (Bit32u)(BX_CPU_THIS_PTR the_i387.fdp);
-            write_virtual_dword(i->seg(), RMAddr(i) + 0x14, tmp);
+            write_virtual_dword(i->seg(), eaddr + 0x14, tmp);
             tmp = 0xffff0000 | (BX_CPU_THIS_PTR the_i387.fds);
-            write_virtual_dword(i->seg(), RMAddr(i) + 0x18, tmp);
+            write_virtual_dword(i->seg(), eaddr + 0x18, tmp);
 
-            return 0x1c;
+            offset = 0x1c;
         }
         else /* Protected Mode - 16 bit */
         {
             Bit16u tmp;
 
             tmp = BX_CPU_THIS_PTR the_i387.get_control_word();
-            write_virtual_word(i->seg(), RMAddr(i), tmp);
+            write_virtual_word(i->seg(), eaddr, tmp);
             tmp = BX_CPU_THIS_PTR the_i387.get_status_word();
-            write_virtual_word(i->seg(), RMAddr(i) + 0x02, tmp);
+            write_virtual_word(i->seg(), eaddr + 0x02, tmp);
             tmp = BX_CPU_THIS_PTR the_i387.get_tag_word();
-            write_virtual_word(i->seg(), RMAddr(i) + 0x04, tmp);
+            write_virtual_word(i->seg(), eaddr + 0x04, tmp);
             tmp = (Bit16u)(BX_CPU_THIS_PTR the_i387.fip) & 0xffff;
-            write_virtual_word(i->seg(), RMAddr(i) + 0x06, tmp);
+            write_virtual_word(i->seg(), eaddr + 0x06, tmp);
             tmp = (BX_CPU_THIS_PTR the_i387.fcs);
-            write_virtual_word(i->seg(), RMAddr(i) + 0x08, tmp);
+            write_virtual_word(i->seg(), eaddr + 0x08, tmp);
             tmp = (Bit16u)(BX_CPU_THIS_PTR the_i387.fdp) & 0xffff;
-            write_virtual_word(i->seg(), RMAddr(i) + 0x0a, tmp);
+            write_virtual_word(i->seg(), eaddr + 0x0a, tmp);
             tmp = (BX_CPU_THIS_PTR the_i387.fds);
-            write_virtual_word(i->seg(), RMAddr(i) + 0x0c, tmp);
+            write_virtual_word(i->seg(), eaddr + 0x0c, tmp);
 
-            return 0x0e;
+            offset = 0x0e;
         }
     }
     else   /* Real or V86 Mode */
@@ -148,56 +143,58 @@ int BX_CPU_C::fpu_save_environment(bxInstruction_c *i)
         Bit32u fp_dp = ((Bit32u)(BX_CPU_THIS_PTR the_i387.fds) << 4) +
               (Bit32u)(BX_CPU_THIS_PTR the_i387.fdp);
 
-        if (i->os32L() || i->os64L())
+        if (i->os32L())
         {
             Bit32u tmp;
 
             tmp = 0xffff0000 | BX_CPU_THIS_PTR the_i387.get_control_word();
-            write_virtual_dword(i->seg(), RMAddr(i), tmp);
+            write_virtual_dword(i->seg(), eaddr, tmp);
             tmp = 0xffff0000 | BX_CPU_THIS_PTR the_i387.get_status_word();
-            write_virtual_dword(i->seg(), RMAddr(i) + 0x04, tmp);
+            write_virtual_dword(i->seg(), eaddr + 0x04, tmp);
             tmp = 0xffff0000 | BX_CPU_THIS_PTR the_i387.get_tag_word();
-            write_virtual_dword(i->seg(), RMAddr(i) + 0x08, tmp);
+            write_virtual_dword(i->seg(), eaddr + 0x08, tmp);
             tmp = 0xffff0000 | (fp_ip & 0xffff);
-            write_virtual_dword(i->seg(), RMAddr(i) + 0x0c, tmp);
+            write_virtual_dword(i->seg(), eaddr + 0x0c, tmp);
             tmp = ((fp_ip & 0xffff0000) >> 4) | BX_CPU_THIS_PTR the_i387.foo;
-            write_virtual_dword(i->seg(), RMAddr(i) + 0x10, tmp);
+            write_virtual_dword(i->seg(), eaddr + 0x10, tmp);
             tmp = 0xffff0000 | (fp_dp & 0xffff);
-            write_virtual_dword(i->seg(), RMAddr(i) + 0x14, tmp);
+            write_virtual_dword(i->seg(), eaddr + 0x14, tmp);
             tmp = (fp_dp & 0xffff0000) >> 4;
-            write_virtual_dword(i->seg(), RMAddr(i) + 0x18, tmp);
+            write_virtual_dword(i->seg(), eaddr + 0x18, tmp);
 
-            return 0x1c;
+            offset = 0x1c;
         }
         else  /* Real or V86 Mode - 16 bit */
         {
             Bit16u tmp;
 
             tmp = BX_CPU_THIS_PTR the_i387.get_control_word();
-            write_virtual_word(i->seg(), RMAddr(i), tmp);
+            write_virtual_word(i->seg(), eaddr, tmp);
             tmp = BX_CPU_THIS_PTR the_i387.get_status_word();
-            write_virtual_word(i->seg(), RMAddr(i) + 0x02, tmp);
+            write_virtual_word(i->seg(), eaddr + 0x02, tmp);
             tmp = BX_CPU_THIS_PTR the_i387.get_tag_word();
-            write_virtual_word(i->seg(), RMAddr(i) + 0x04, tmp);
+            write_virtual_word(i->seg(), eaddr + 0x04, tmp);
             tmp = fp_ip & 0xffff;
-            write_virtual_word(i->seg(), RMAddr(i) + 0x06, tmp);
+            write_virtual_word(i->seg(), eaddr + 0x06, tmp);
             tmp = (Bit16u)((fp_ip & 0xf0000) >> 4) | BX_CPU_THIS_PTR the_i387.foo;
-            write_virtual_word(i->seg(), RMAddr(i) + 0x08, tmp);
+            write_virtual_word(i->seg(), eaddr + 0x08, tmp);
             tmp = fp_dp & 0xffff;
-            write_virtual_word(i->seg(), RMAddr(i) + 0x0a, tmp);
+            write_virtual_word(i->seg(), eaddr + 0x0a, tmp);
             tmp = (Bit16u)((fp_dp & 0xf0000) >> 4);
-            write_virtual_word(i->seg(), RMAddr(i) + 0x0c, tmp);
+            write_virtual_word(i->seg(), eaddr + 0x0c, tmp);
 
-            return 0x0e;
+            offset = 0x0e;
         }
     }
+
+    return (eaddr + offset);
 }
 
-int BX_CPU_C::fpu_load_environment(bxInstruction_c *i)
+bx_address BX_CPU_C::fpu_load_environment(bxInstruction_c *i)
 {
-    int offset;
+    unsigned offset;
 
-    BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+    bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
     if (protected_mode())  /* Protected Mode */
     {
@@ -205,21 +202,21 @@ int BX_CPU_C::fpu_load_environment(bxInstruction_c *i)
         {
             Bit32u tmp;
 
-            tmp = read_virtual_dword(i->seg(), RMAddr(i));
+            tmp = read_virtual_dword(i->seg(), eaddr);
             BX_CPU_THIS_PTR the_i387.cwd = tmp & 0xffff;
-            tmp = read_virtual_dword(i->seg(), RMAddr(i) + 0x04);
+            tmp = read_virtual_dword(i->seg(), eaddr + 0x04);
             BX_CPU_THIS_PTR the_i387.swd = tmp & 0xffff;
             BX_CPU_THIS_PTR the_i387.tos = (tmp >> 11) & 0x07;
-            tmp = read_virtual_dword(i->seg(), RMAddr(i) + 0x08);
+            tmp = read_virtual_dword(i->seg(), eaddr + 0x08);
             BX_CPU_THIS_PTR the_i387.twd = tmp & 0xffff;
-            tmp = read_virtual_dword(i->seg(), RMAddr(i) + 0x0c);
+            tmp = read_virtual_dword(i->seg(), eaddr + 0x0c);
             BX_CPU_THIS_PTR the_i387.fip = tmp;
-            tmp = read_virtual_dword(i->seg(), RMAddr(i) + 0x10);
+            tmp = read_virtual_dword(i->seg(), eaddr + 0x10);
             BX_CPU_THIS_PTR the_i387.fcs = tmp & 0xffff;
             BX_CPU_THIS_PTR the_i387.foo = (tmp >> 16) & 0x07ff;
-            tmp = read_virtual_dword(i->seg(), RMAddr(i) + 0x14);
+            tmp = read_virtual_dword(i->seg(), eaddr + 0x14);
             BX_CPU_THIS_PTR the_i387.fdp = tmp;
-            tmp = read_virtual_dword(i->seg(), RMAddr(i) + 0x18);
+            tmp = read_virtual_dword(i->seg(), eaddr + 0x18);
             BX_CPU_THIS_PTR the_i387.fds = tmp & 0xffff;
             offset = 0x1c;
         }
@@ -227,20 +224,20 @@ int BX_CPU_C::fpu_load_environment(bxInstruction_c *i)
         {
             Bit16u tmp;
 
-            tmp = read_virtual_word(i->seg(), RMAddr(i));
+            tmp = read_virtual_word(i->seg(), eaddr);
             BX_CPU_THIS_PTR the_i387.cwd = tmp;
-            tmp = read_virtual_word(i->seg(), RMAddr(i) + 0x2);
+            tmp = read_virtual_word(i->seg(), eaddr + 0x2);
             BX_CPU_THIS_PTR the_i387.swd = tmp;
             BX_CPU_THIS_PTR the_i387.tos = (tmp >> 11) & 0x07;
-            tmp = read_virtual_word(i->seg(), RMAddr(i) + 0x04);
+            tmp = read_virtual_word(i->seg(), eaddr + 0x04);
             BX_CPU_THIS_PTR the_i387.twd = tmp;
-            tmp = read_virtual_word(i->seg(), RMAddr(i) + 0x06);
+            tmp = read_virtual_word(i->seg(), eaddr + 0x06);
             BX_CPU_THIS_PTR the_i387.fip = tmp & 0xffff;
-            tmp = read_virtual_word(i->seg(), RMAddr(i) + 0x08);
+            tmp = read_virtual_word(i->seg(), eaddr + 0x08);
             BX_CPU_THIS_PTR the_i387.fcs = tmp;
-            tmp = read_virtual_word(i->seg(), RMAddr(i) + 0x0a);
+            tmp = read_virtual_word(i->seg(), eaddr + 0x0a);
             BX_CPU_THIS_PTR the_i387.fdp = tmp & 0xffff;
-            tmp = read_virtual_word(i->seg(), RMAddr(i) + 0x0c);
+            tmp = read_virtual_word(i->seg(), eaddr + 0x0c);
             BX_CPU_THIS_PTR the_i387.fds = tmp;
             /* opcode is defined to be zero */
             BX_CPU_THIS_PTR the_i387.foo = 0;
@@ -251,27 +248,27 @@ int BX_CPU_C::fpu_load_environment(bxInstruction_c *i)
     {
         Bit32u fp_ip = 0, fp_dp = 0;
 
-        if (i->os32L() || i->os64L())
+        if (i->os32L())
         {
             Bit32u tmp;
 
-            tmp = read_virtual_dword(i->seg(), RMAddr(i));
+            tmp = read_virtual_dword(i->seg(), eaddr);
             BX_CPU_THIS_PTR the_i387.cwd = tmp & 0xffff;
-            tmp = read_virtual_dword(i->seg(), RMAddr(i) + 0x04);
+            tmp = read_virtual_dword(i->seg(), eaddr + 0x04);
             BX_CPU_THIS_PTR the_i387.swd = tmp & 0xffff;
             BX_CPU_THIS_PTR the_i387.tos = (tmp >> 11) & 0x07;
-            tmp = read_virtual_dword(i->seg(), RMAddr(i) + 0x08);
+            tmp = read_virtual_dword(i->seg(), eaddr + 0x08);
             BX_CPU_THIS_PTR the_i387.twd = tmp & 0xffff;
-            tmp = read_virtual_dword(i->seg(), RMAddr(i) + 0x0c);
+            tmp = read_virtual_dword(i->seg(), eaddr + 0x0c);
             fp_ip = tmp & 0xffff;
-            tmp = read_virtual_dword(i->seg(), RMAddr(i) + 0x10);
+            tmp = read_virtual_dword(i->seg(), eaddr + 0x10);
             fp_ip = fp_ip | ((tmp & 0x0ffff000) << 4);
             BX_CPU_THIS_PTR the_i387.fip = fp_ip;
             BX_CPU_THIS_PTR the_i387.foo = tmp & 0x07ff;
             BX_CPU_THIS_PTR the_i387.fcs = 0;
-            tmp = read_virtual_dword(i->seg(), RMAddr(i) + 0x14);
+            tmp = read_virtual_dword(i->seg(), eaddr + 0x14);
             fp_dp = tmp & 0xffff;
-            tmp = read_virtual_dword(i->seg(), RMAddr(i) + 0x18);
+            tmp = read_virtual_dword(i->seg(), eaddr + 0x18);
             fp_dp = fp_dp | ((tmp & 0x0ffff000) << 4);
             BX_CPU_THIS_PTR the_i387.fdp = fp_dp;
             BX_CPU_THIS_PTR the_i387.fds = 0;
@@ -281,23 +278,23 @@ int BX_CPU_C::fpu_load_environment(bxInstruction_c *i)
         {
             Bit16u tmp;
 
-            tmp = read_virtual_word(i->seg(), RMAddr(i));
+            tmp = read_virtual_word(i->seg(), eaddr);
             BX_CPU_THIS_PTR the_i387.cwd = tmp;
-            tmp = read_virtual_word(i->seg(), RMAddr(i) + 0x2);
+            tmp = read_virtual_word(i->seg(), eaddr + 0x2);
             BX_CPU_THIS_PTR the_i387.swd = tmp;
             BX_CPU_THIS_PTR the_i387.tos = (tmp >> 11) & 0x07;
-            tmp = read_virtual_word(i->seg(), RMAddr(i) + 0x04);
+            tmp = read_virtual_word(i->seg(), eaddr + 0x04);
             BX_CPU_THIS_PTR the_i387.twd = tmp;
-            tmp = read_virtual_word(i->seg(), RMAddr(i) + 0x06);
+            tmp = read_virtual_word(i->seg(), eaddr + 0x06);
             fp_ip = tmp & 0xffff;
-            tmp = read_virtual_word(i->seg(), RMAddr(i) + 0x08);
+            tmp = read_virtual_word(i->seg(), eaddr + 0x08);
             fp_ip = fp_ip | ((tmp & 0xf000) << 4);
             BX_CPU_THIS_PTR the_i387.fip = fp_ip;
             BX_CPU_THIS_PTR the_i387.foo = tmp & 0x07ff;
             BX_CPU_THIS_PTR the_i387.fcs = 0;
-            tmp = read_virtual_word(i->seg(), RMAddr(i) + 0x0a);
+            tmp = read_virtual_word(i->seg(), eaddr + 0x0a);
             fp_dp = tmp & 0xffff;
-            tmp = read_virtual_word(i->seg(), RMAddr(i) + 0x0c);
+            tmp = read_virtual_word(i->seg(), eaddr + 0x0c);
             fp_dp = fp_dp | ((tmp & 0xf000) << 4);
             BX_CPU_THIS_PTR the_i387.fdp = fp_dp;
             BX_CPU_THIS_PTR the_i387.fds = 0;
@@ -321,18 +318,18 @@ int BX_CPU_C::fpu_load_environment(bxInstruction_c *i)
         FPU_PARTIAL_STATUS &= ~(FPU_SW_Summary | FPU_SW_Backward);
     }
 
-    return offset;
+    return (eaddr + offset);
 }
 
 /* D9 /5 */
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLDCW(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
-  BX_CPU_THIS_PTR prepareFPU(i, CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
+  BX_CPU_THIS_PTR prepareFPU(i, CHECK_PENDING_EXCEPTIONS);
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  Bit16u cwd = read_virtual_word(i->seg(), RMAddr(i));
+  Bit16u cwd = read_virtual_word(i->seg(), eaddr);
   FPU_CONTROL_WORD = (cwd & ~FPU_CW_Reserved_Bits) | 0x0040; // bit 6 is reserved as '1
 
   /* check for unmasked exceptions */
@@ -355,13 +352,13 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLDCW(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNSTCW(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
-  BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
+  BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS);
 
   Bit16u cwd = BX_CPU_THIS_PTR the_i387.get_control_word();
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  write_virtual_word(i->seg(), RMAddr(i), cwd);
+  write_virtual_word(i->seg(), eaddr, cwd);
 #else
   BX_INFO(("FNSTCW: required FPU, configure --enable-fpu"));
 #endif
@@ -371,13 +368,13 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNSTCW(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNSTSW(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
-  BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
+  BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS);
 
   Bit16u swd = BX_CPU_THIS_PTR the_i387.get_status_word();
 
-  BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
+  bx_address eaddr = BX_CPU_CALL_METHODR(i->ResolveModrm, (i));
 
-  write_virtual_word(i->seg(), RMAddr(i), swd);
+  write_virtual_word(i->seg(), eaddr, swd);
 #else
   BX_INFO(("FNSTSW: required FPU, configure --enable-fpu"));
 #endif
@@ -387,7 +384,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNSTSW(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNSTSW_AX(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
-  BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
+  BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS);
   AX = BX_CPU_THIS_PTR the_i387.get_status_word();
 #else
   BX_INFO(("FNSTSW_AX: required FPU, configure --enable-fpu"));
@@ -398,15 +395,17 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNSTSW_AX(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FRSTOR(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
-  BX_CPU_THIS_PTR prepareFPU(i, CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
+  BX_CPU_THIS_PTR prepareFPU(i, CHECK_PENDING_EXCEPTIONS);
 
-  int offset = fpu_load_environment(i);
+  bx_address offset = fpu_load_environment(i);
   floatx80 tmp;
 
   /* read all registers in stack order */
   for(int n=0;n<8;n++)
   {
-     read_virtual_tword(i->seg(), RMAddr(i) + offset + n*10, &tmp);
+     tmp.fraction = read_virtual_qword(i->seg(), offset + n*10);
+     tmp.exp      = read_virtual_word (i->seg(), offset + n*10 + 8);
+
      // update tag only if it is not empty
      BX_WRITE_FPU_REGISTER_AND_TAG(tmp,
               IS_TAG_EMPTY(n) ? FPU_Tag_Empty : FPU_tagof(tmp), n);
@@ -420,15 +419,16 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FRSTOR(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNSAVE(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
-  BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
+  BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS);
 
-  int offset = fpu_save_environment(i);
+  bx_address offset = fpu_save_environment(i);
 
   /* save all registers in stack order. */
   for(int n=0;n<8;n++)
   {
      floatx80 stn = BX_READ_FPU_REG(n);
-     write_virtual_tword(i->seg(), RMAddr(i) + offset + n*10, &stn);
+     write_virtual_qword(i->seg(), offset + n*10,     stn.fraction);
+     write_virtual_word (i->seg(), offset + n*10 + 8, stn.exp);
   }
 
   BX_CPU_THIS_PTR the_i387.init();
@@ -441,11 +441,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNSAVE(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNCLEX(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
-  BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
+  BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS);
 
   FPU_PARTIAL_STATUS &= ~(FPU_SW_Backward|FPU_SW_Summary|FPU_SW_Stack_Fault|FPU_SW_Precision|
-		   FPU_SW_Underflow|FPU_SW_Overflow|FPU_SW_Zero_Div|FPU_SW_Denormal_Op|
-		   FPU_SW_Invalid);
+                   FPU_SW_Underflow|FPU_SW_Overflow|FPU_SW_Zero_Div|FPU_SW_Denormal_Op|
+                   FPU_SW_Invalid);
 
   // do not update last fpu instruction pointer
 #else
@@ -457,7 +457,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNCLEX(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNINIT(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
-  BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
+  BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS);
   BX_CPU_THIS_PTR the_i387.init();
 #else
   BX_INFO(("FNINIT: required FPU, configure --enable-fpu"));
@@ -468,7 +468,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNINIT(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLDENV(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
-  BX_CPU_THIS_PTR prepareFPU(i, CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
+  BX_CPU_THIS_PTR prepareFPU(i, CHECK_PENDING_EXCEPTIONS);
   fpu_load_environment(i);
 
   /* read all registers in stack order and update x87 tag word */
@@ -488,7 +488,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FLDENV(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNSTENV(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
-  BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
+  BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS);
   fpu_save_environment(i);
   /* mask all floating point exceptions */
   FPU_CONTROL_WORD |= FPU_CW_Exceptions_Mask;
@@ -503,7 +503,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNSTENV(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNOP(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
-  BX_CPU_THIS_PTR prepareFPU(i, CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
+  BX_CPU_THIS_PTR prepareFPU(i, CHECK_PENDING_EXCEPTIONS);
 
   // Perform no FPU operation. This instruction takes up space in the
   // instruction stream but does not affect the FPU or machine
@@ -516,7 +516,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FNOP(bxInstruction_c *i)
 void BX_CPP_AttrRegparmN(1) BX_CPU_C::FPLEGACY(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
-  BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS, !UPDATE_LAST_OPCODE);
+  BX_CPU_THIS_PTR prepareFPU(i, !CHECK_PENDING_EXCEPTIONS);
 
   // FPU performs no specific operation and no internal x87 states
   // are affected
@@ -529,17 +529,22 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FPLEGACY(bxInstruction_c *i)
 
 #if BX_SUPPORT_FPU
 
+#include "softfloatx80.h"
+
 #include <math.h>
 
 void BX_CPU_C::print_state_FPU(void)
 {
   static double scale_factor = pow(2.0, -63.0);
-  static char* cw_round_control[] = {
+  static const char* cw_round_control[] = {
     "NEAREST", "DOWN", "UP", "CHOP"
   };
-  static char* cw_precision_control[] = {
+  static const char* cw_precision_control[] = {
     "32", "RES", "64", "80"
   };
+  static const char* fp_class[] = {
+    "ZERO", "xNAN", "-INF", "+INF", "DENORMAL", "NORMAL"
+  };
 
   Bit32u reg;
   reg = BX_CPU_THIS_PTR the_i387.get_status_word();
@@ -598,11 +603,13 @@ void BX_CPU_C::print_state_FPU(void)
 #else
     f *= fp.fraction*scale_factor;
 #endif
-    fprintf(stderr, "%sFP%d ST%d(%c):        raw 0x%04x:%08lx%08lx (%.10f)\n",
+    float_class_t f_class = floatx80_class(fp);
+    fprintf(stderr, "%sFP%d ST%d(%c):        raw 0x%04x:%08lx%08lx (%.10f) (%s)\n",
           i==tos?"=>":"  ", i, (i-tos)&7,
           "v0se"[tag],
           fp.exp & 0xffff, GET32H(fp.fraction), GET32L(fp.fraction),
-          f);
+          f, (f_class == float_NaN) ? (floatx80_is_signaling_nan(fp) ? "SNAN" : "QNAN") : fp_class[f_class]);
   }
 }
+
 #endif
