@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: fpu_misc.cc,v 1.14 2008/05/10 13:34:01 sshwarts Exp $
+// $Id: fpu_misc.cc,v 1.18.2.1 2009/06/07 07:49:11 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //   Copyright (c) 2003 Stanislav Shwartsman
@@ -17,7 +17,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 //
 /////////////////////////////////////////////////////////////////////////
 
@@ -35,6 +35,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FXCH_STi(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_update_last_instruction(i);
 
   int st0_tag = BX_CPU_THIS_PTR the_i387.FPU_gettagi(0);
   int sti_tag = BX_CPU_THIS_PTR the_i387.FPU_gettagi(i->rm());
@@ -46,11 +47,11 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FXCH_STi(bxInstruction_c *i)
 
   if (st0_tag == FPU_Tag_Empty || sti_tag == FPU_Tag_Empty)
   {
-      BX_CPU_THIS_PTR FPU_exception(FPU_EX_Stack_Underflow);
+      FPU_exception(FPU_EX_Stack_Underflow);
 
       if(BX_CPU_THIS_PTR the_i387.is_IA_masked())
       {
-	  /* Masked response */
+          /* Masked response */
           if (st0_tag == FPU_Tag_Empty)
               st0_reg = floatx80_default_nan;
 
@@ -72,16 +73,16 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FCHS(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_update_last_instruction(i);
 
   if (IS_TAG_EMPTY(0)) {
-      BX_CPU_THIS_PTR FPU_stack_underflow(0);
-      return;
+     FPU_stack_underflow(0);
+  }
+  else {
+     clear_C1();
+     floatx80 st0_reg = BX_READ_FPU_REG(0);
+     BX_WRITE_FPU_REG(floatx80_chs(st0_reg), 0);
   }
-
-  clear_C1();
-
-  floatx80 st0_reg = BX_READ_FPU_REG(0);
-  BX_WRITE_FPU_REG(floatx80_chs(st0_reg), 0);
 #else
   BX_INFO(("FCHS: required FPU, configure --enable-fpu"));
 #endif
@@ -92,16 +93,16 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FABS(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_update_last_instruction(i);
 
   if (IS_TAG_EMPTY(0)) {
-      BX_CPU_THIS_PTR FPU_stack_underflow(0);
-      return;
+     FPU_stack_underflow(0);
+  }
+  else {
+     clear_C1();
+     floatx80 st0_reg = BX_READ_FPU_REG(0);
+     BX_WRITE_FPU_REG(floatx80_abs(st0_reg), 0);
   }
-
-  clear_C1();
-
-  floatx80 st0_reg = BX_READ_FPU_REG(0);
-  BX_WRITE_FPU_REG(floatx80_abs(st0_reg), 0);
 #else
   BX_INFO(("FABS: required FPU, configure --enable-fpu"));
 #endif
@@ -112,6 +113,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FDECSTP(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_update_last_instruction(i);
 
   clear_C1();
 
@@ -126,6 +128,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FINCSTP(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_update_last_instruction(i);
 
   clear_C1();
 
@@ -140,6 +143,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FFREE_STi(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_update_last_instruction(i);
   BX_CPU_THIS_PTR the_i387.FPU_settagi(FPU_Tag_Empty, i->rm());
 #else
   BX_INFO(("FFREE_STi: required FPU, configure --enable-fpu"));
@@ -156,6 +160,7 @@ void BX_CPP_AttrRegparmN(1) BX_CPU_C::FFREEP_STi(bxInstruction_c *i)
 {
 #if BX_SUPPORT_FPU
   BX_CPU_THIS_PTR prepareFPU(i);
+  BX_CPU_THIS_PTR FPU_update_last_instruction(i);
   BX_CPU_THIS_PTR the_i387.FPU_settagi(FPU_Tag_Empty, i->rm());
   BX_CPU_THIS_PTR the_i387.FPU_pop();
 #else
