@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: gdbstub.cc,v 1.32 2008/02/15 19:03:53 sshwarts Exp $
+// $Id: gdbstub.cc,v 1.35.2.1 2009/06/07 07:49:09 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002-2006  The Bochs Project Team
@@ -95,12 +95,12 @@ static char get_debug_char(void)
 
 static const char hexchars[]="0123456789abcdef";
 
-static void put_reply(char* buffer)
+static void put_reply(const char* buffer)
 {
   unsigned char csum;
   int i;
 
-  BX_DEBUG (("put_buffer %s", buffer));
+  BX_DEBUG(("put_buffer '%s'", buffer));
 
   do {
     put_debug_char('$');
@@ -182,31 +182,25 @@ static void get_command(char* buffer)
 
 void hex2mem(char* buf, unsigned char* mem, int count)
 {
-  int i;
   unsigned char ch;
 
-  for (i = 0; i<count; i++)
+  for (int i = 0; i<count; i++)
   {
     ch = hex(*buf++) << 4;
     ch = ch + hex(*buf++);
-    *mem = ch;
-    mem++;
+    *mem++ = ch;
   }
 }
 
-char* mem2hex(char* mem, char* buf, int count)
+char* mem2hex(const Bit8u* mem, char* buf, int count)
 {
-  int i;
-  unsigned char ch;
+  Bit8u ch;
 
-  for (i = 0; i<count; i++)
+  for (int i = 0; i<count; i++)
   {
-    ch = *mem;
-    mem++;
-    *buf = hexchars[ch >> 4];
-    buf++;
-    *buf = hexchars[ch % 16];
-    buf++;
+    ch = *mem++;
+    *buf++ = hexchars[ch >> 4];
+    *buf++ = hexchars[ch % 16];
   }
   *buf = 0;
   return(buf);
@@ -243,7 +237,7 @@ static int other_thread = 0;
 #if !BX_SUPPORT_X86_64
 #define NUMREGS (16)
 #define NUMREGSBYTES (NUMREGS * 4)
-static int registers[NUMREGS];
+static Bit32u registers[NUMREGS];
 #endif
 
 #define MAX_BREAKPOINTS (255)
@@ -426,10 +420,10 @@ static int access_linear(Bit64u laddress,
   valid = BX_CPU(0)->dbg_xlate_linear2phy(laddress, (bx_phy_address*)&phys);
   if (!valid) return(0);
 
-  if (rw == BX_READ) {
-    valid = BX_MEM(0)->dbg_fetch_mem(BX_CPU(0), phys, len, data);
-  } else {
+  if (rw & 1) {
     valid = BX_MEM(0)->dbg_set_mem(phys, len, data);
+  } else {
+    valid = BX_MEM(0)->dbg_fetch_mem(BX_CPU(0), phys, len, data);
   }
 
   return(valid);
@@ -438,16 +432,14 @@ static int access_linear(Bit64u laddress,
 static void debug_loop(void)
 {
   char buffer[255];
-  char obuf[255];
-  int ne;
-  unsigned char mem[255];
-
-  ne = 0;
+  char obuf[1024];
+  int ne = 0;
+  Bit8u mem[255];
 
   while (ne == 0)
   {
     get_command(buffer);
-    BX_DEBUG(("get_buffer %s", buffer));
+    BX_DEBUG(("get_buffer '%s'", buffer));
 
     switch (buffer[0])
     {
@@ -565,7 +557,7 @@ static void debug_loop(void)
         BX_INFO(("addr %Lx len %x", addr, len));
 
         access_linear(addr, len, BX_READ, mem);
-        mem2hex((char *)mem, obuf, len);
+        mem2hex(mem, obuf, len);
         put_reply(obuf);
         break;
       }
@@ -577,25 +569,29 @@ static void debug_loop(void)
         char* ebuf;
 
         reg = strtoul(&buffer[1], &ebuf, 16);
-	++ebuf;
+        ++ebuf;
         value = read_little_endian_hex(ebuf);
 
         BX_INFO(("reg %d set to %Lx", reg, value));
-#if !BX_SUPPORT_X86_64
+#if BX_SUPPORT_X86_64 == 0
         switch (reg)
         {
-          case 1:
-          EAX = value;
-          break;
+          case 0:
+            EAX = value;
+            break;
 
-          case 2:
+          case 1:
             ECX = value;
             break;
 
-          case 3:
+          case 2:
             EBX = value;
             break;
 
+          case 3:
+            EDX = value;
+            break;
+
           case 4:
             ESP = value;
             break;
@@ -640,19 +636,19 @@ static void debug_loop(void)
             break;
 
           case 4:
-            RSI = value;
+            RSP = value;
             break;
 
           case 5:
-            RDI = value;
+            RBP = value;
             break;
 
           case 6:
-            ESP = value;
+            RSI = value;
             break;
 
           case 7:
-            RBP = value;
+            RDI = value;
             break;
 
           case 8:
@@ -702,41 +698,42 @@ static void debug_loop(void)
       }
 
       case 'g':
-#if !BX_SUPPORT_X86_64
-        registers[0] = EAX;
-        registers[1] = ECX;
-        registers[2] = EDX;
-        registers[3] = EBX;
-        registers[4] = ESP;
-        registers[5] = EBP;
-        registers[6] = ESI;
-        registers[7] = EDI;
+#if BX_SUPPORT_X86_64 == 0
+        WriteHostDWordToLittleEndian(registers + 0, EAX);
+        WriteHostDWordToLittleEndian(registers + 1, ECX);
+        WriteHostDWordToLittleEndian(registers + 2, EDX);
+        WriteHostDWordToLittleEndian(registers + 3, EBX);
+        WriteHostDWordToLittleEndian(registers + 4, ESP);
+        WriteHostDWordToLittleEndian(registers + 5, EBP);
+        WriteHostDWordToLittleEndian(registers + 6, ESI);
+        WriteHostDWordToLittleEndian(registers + 7, EDI);
         if (last_stop_reason == GDBSTUB_EXECUTION_BREAKPOINT)
         {
-          registers[8] = EIP + 1;
+          WriteHostDWordToLittleEndian(registers + 8, EIP + 1);
         }
         else
         {
-          registers[8] = EIP;
+          WriteHostDWordToLittleEndian(registers + 8, EIP);
         }
-        registers[9] = BX_CPU_THIS_PTR read_eflags();
-        registers[10] =
-          BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value;
-        registers[11] =
-          BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].selector.value;
-        registers[12] =
-          BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].selector.value;
-        registers[13] =
-          BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].selector.value;
-        registers[14] =
-          BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].selector.value;
-        registers[15] =
-          BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].selector.value;
-        mem2hex((char *)registers, obuf, NUMREGSBYTES);
+        WriteHostDWordToLittleEndian(registers + 9,
+                                     BX_CPU_THIS_PTR read_eflags());
+        WriteHostDWordToLittleEndian(registers + 10,
+          BX_CPU_THIS_PTR sregs[BX_SEG_REG_CS].selector.value);
+        WriteHostDWordToLittleEndian(registers + 11,
+          BX_CPU_THIS_PTR sregs[BX_SEG_REG_SS].selector.value);
+        WriteHostDWordToLittleEndian(registers + 12,
+          BX_CPU_THIS_PTR sregs[BX_SEG_REG_DS].selector.value);
+        WriteHostDWordToLittleEndian(registers + 13,
+          BX_CPU_THIS_PTR sregs[BX_SEG_REG_ES].selector.value);
+        WriteHostDWordToLittleEndian(registers + 14,
+          BX_CPU_THIS_PTR sregs[BX_SEG_REG_FS].selector.value);
+        WriteHostDWordToLittleEndian(registers + 15,
+          BX_CPU_THIS_PTR sregs[BX_SEG_REG_GS].selector.value);
+        mem2hex(registers, obuf, NUMREGSBYTES);
 #else
 #define PUTREG(buf, val, len) do { \
          Bit64u u = (val); \
-         (buf) = mem2hex((char*)&u, (buf), (len)); \
+         (buf) = mem2hex((Bit8u*)&u, (buf), (len)); \
       } while (0)
         char* buf;
         buf = obuf;
@@ -810,6 +807,10 @@ static void debug_loop(void)
                   SIM->get_param_num("bss_base", gdbstub_list)->get());
           put_reply(obuf);
         }
+        else if (strncmp(&buffer[1], "Supported", strlen("Supported")) == 0)
+        {
+          put_reply("");
+        }
         else
         {
           put_reply("Eff");
