@@ -1,10 +1,11 @@
 ////////////////////////////////////////////////////////////////////
-// $Id: wxdialog.h,v 1.69 2007/11/02 16:58:46 vruppert Exp $
+// $Id: wxdialog.h,v 1.76 2009/05/01 09:32:46 sshwarts Exp $
 ////////////////////////////////////////////////////////////////////
 //
 // wxWidgets dialogs for Bochs
 
 #include <wx/spinctrl.h>
+#include <wx/notebook.h>
 
 ////////////////////////////////////////////////////////////////////
 // text messages used in several places
@@ -45,7 +46,7 @@ wxChoice *makeLogOptionChoiceBox(wxWindow *parent, wxWindowID id, int evtype, bo
 
 ////////////////////////////////////////////////////////////////////
 // LogMsgAskDialog is a modal dialog box that shows the user a
-// simulation error message and asks if they want to continue or 
+// simulation error message and asks if they want to continue or
 // not.  It looks something like this:
 //
 // +----- PANIC ---------------------------------------------------+
@@ -69,8 +70,8 @@ class LogMsgAskDialog: public wxDialog
 {
 public:
   enum button_t {
-    CONT=0, DIE, DUMP, DEBUG, HELP, 
-    N_BUTTONS /* number of entries in enum */ 
+    CONT=0, DIE, DUMP, DEBUG, HELP,
+    N_BUTTONS /* number of entries in enum */
   };
 #define LOG_MSG_ASK_IDS \
   { ID_Continue, ID_Die, ID_DumpCore, ID_Debugger, wxHELP }
@@ -100,115 +101,6 @@ public:
 DECLARE_EVENT_TABLE()
 };
 
-////////////////////////////////////////////////////////////////////
-// FloppyConfigDialog is a modal dialog box that asks the user
-// what physical device or disk image should be used for emulation.
-//
-// +-----Configure Floppy Drive A----------------------------------+
-// |                                                               |
-// | Bochs can use a real floppy drive as Disk A, or use an        |
-// | image file.                                                   |
-// |                                                               |
-// |    [ ]  None/Disabled                                         |
-// |    [X]  Physical floppy drive A:                              |
-// |    [ ]  Physical floppy drive B:                              |
-// |    [ ]  Disk image: [_____________________________] [Browse]  |
-// |                                                               |
-// | What is the capacity of this disk? [1.44 MB]                  |
-// |                                                               |
-// | Hint: To create a disk image, choose the name and capacity    |
-// | above, then click Ok.                                         |
-// |                                                               |
-// |                   [ Help ] [ Cancel ] [ Create Image ] [ Ok ] |
-// +---------------------------------------------------------------+
-// To use this dialog:
-// After constructor, use AddRadio() to add radio buttons, SetFilename()
-// to fill in the disk image filename, SetCapacity() to set the capacity. 
-// Then call ShowModal() to display it.  Return value is wxID_OK or
-// wxID_CANCEL.  If you set a validation function, then it will be called when
-// ok is pressed, and will get a chance to veto the "Ok" if it returns false.
-// After ShowModal() returns, use GetFilename and GetCapacity to see what the
-// user did.  If the validation function sets parameters, this may be
-// unnecessary.
-//
-// Volker reminded me that I wasn't paying much attention to
-// the distinction between configuring the device (pre-boot) and 
-// configuring the media which can be done anytime.  Here's a proposal
-// to fix that...  -Bryce
-// +-----Configure Floppy Drive A----------------------------------+
-// |                                                               |
-// | +-- Device -----------------------------------------------+   |
-// | |                                                         |   |
-// | |  [ ] Enable Emulated Drive A                            |   |
-// | |                                                         |   |
-// | |  Drive capacity [1.44 MB]                               |   |
-// | |                                                         |   |
-// | +---------------------------------------------------------+   |
-// |                                                               |
-// | +-- Media: Where does the data come from? ----------------+   |
-// | |                                                         |   |
-// | | Bochs can use a physical floppy drive as the data       |   |
-// | | source, or use an image file.                           |   |
-// | |                                                         |   |
-// | |  [X]  Physical floppy drive A:                          |   |
-// | |  [ ]  Physical floppy drive B:                          |   |
-// | |  [ ]  Disk image: [_________________________] [Browse]  |   |
-// | |                                                         |   |
-// | | Media size [1.44 MB]                                    |   |
-// | |                                                         |   |
-// | | Hint: To create a disk image, choose the name and       |   |
-// | | capacity above, then click Ok.                          |   |
-// | |                                        [ Create Image ] |   |
-// | +---------------------------------------------------------+   |
-// |                                                               |
-// |                                    [ Help ] [ Cancel ] [ Ok ] |
-// +---------------------------------------------------------------+
-//////////////////////////////////////////////////////////////////////
-
-class FloppyConfigDialog: public wxDialog
-{
-public:
-#define FLOPPY_CONFIG_TITLE wxT("Configure ")
-#define FLOPPY_CONFIG_INSTRS wxT("Select the device or image to use when simulating ")
-#define FLOPPY_CONFIG_CAP wxT("What is the capacity of this disk?")
-#define FLOPPY_CONFIG_HINT wxT("To create a disk image, choose the file name and capacity, then click on \"Create Image\".\n\n" \
-                           "Clicking OK signals a media change for this drive.")
-#define FLOPPY_CONFIG_DISKIMG wxT("Disk image: ")
-private:
-  void Init();  // called automatically by ShowModal()
-  void ShowHelp();
-  wxStaticText *instr;
-#define FLOPPY_MAX_RBTNS 4
-  wxRadioButton *rbtn[FLOPPY_MAX_RBTNS];
-  wxString equivalentFilename[FLOPPY_MAX_RBTNS];
-  int n_rbtns;
-  wxButton *CreateBtn;
-  wxRadioButton *diskImageRadioBtn;
-  wxTextCtrl *filename;
-  wxChoice *capacity;
-  wxBoxSizer *vertSizer, *radioSizer, *diskImageSizer, *capacitySizer, *buttonSizer;
-  typedef bool (*validateFunc_t)(FloppyConfigDialog *dialog);
-  validateFunc_t validate;
-public:
-  FloppyConfigDialog(wxWindow* parent, wxWindowID id);
-  void OnEvent(wxCommandEvent& event);
-  void OnTextEvent(wxCommandEvent& event);
-  int ShowModal() { Init(); return wxDialog::ShowModal(); }
-  void SetRadio(int val);
-  void SetFilename(wxString f);
-  // Use char* instead of wxString because the array we use is already
-  // expressed as a char *[].
-  void SetCapacityChoices(const char *choices[]);
-  void SetCapacity(int cap);
-  int GetRadio();
-  int GetCapacity() { return capacity->GetSelection(); }
-  wxString GetFilename();
-  void SetDriveName(wxString name);
-  void SetValidateFunc(validateFunc_t v) { validate = v; }
-  void AddRadio(const wxString& description, const wxString& filename);
-DECLARE_EVENT_TABLE()
-};
-
 ////////////////////////////////////////////////////////////////////////////
 // AdvancedLogOptionsDialog
 ////////////////////////////////////////////////////////////////////////////
@@ -235,7 +127,7 @@ DECLARE_EVENT_TABLE()
 // |                                                         |
 // |                              [ Help ] [ Cancel ] [ Ok ] |
 // +-------------------------------------------------------+-+
-// 
+//
 class AdvancedLogOptionsDialog: public wxDialog
 {
 private:
@@ -378,7 +270,7 @@ struct AddParamContext {
 };
 
 
-class ParamDialog: public wxDialog 
+class ParamDialog: public wxDialog
 {
 private:
   void ShowHelp();
@@ -386,7 +278,6 @@ private:
   wxTextCtrl *serialDelay, *pasteDelay, *mappingFile;
   wxCheckBox *enableKeymap;
   int genId();
-  bool isGeneratedId(int id);
   bool isShowing;
   int nbuttons;
   bool runtime;
@@ -399,13 +290,12 @@ protected:
   // map parameter ID onto ParamStruct.
   wxHashTable *paramHash;
   virtual void EnableChanged();
-  void EnableParam(int param_id, bool enabled);
-  void EnableParam(const char *pname, bool enabled);
-  void EnableParam(const char *pname, bx_list_c *base, bool enabled);
-  void EnumChanged(ParamStruct *pstr);
-  void EnableChangedRecursive(bx_list_c *list, bool en, ParamStruct *pstrOfCheckbox);
   void EnableChanged(ParamStruct *pstr);
+  void EnableParam(int param_id, bool enabled);
+  void ProcessDependentList(ParamStruct *pstrChanged, bool enabled);
   bool CopyGuiToParam();
+  bool CopyGuiToParam(bx_param_c *param);
+  bool isGeneratedId(int id);
 public:
   ParamDialog(wxWindow* parent, wxWindowID id);
   virtual ~ParamDialog();
@@ -414,22 +304,40 @@ public:
   virtual void AddDefaultButtons();
   virtual void Init();  // called automatically by ShowModal()
   int ShowModal() {
-    Init(); 
+    Init();
     isShowing = true;
-    int ret = wxDialog::ShowModal(); 
+    int ret = wxDialog::ShowModal();
     isShowing = false;
     return ret;
   }
   bool Show(bool val) { isShowing = val; return wxDialog::Show(val); }
   void AddParam(bx_param_c *param, wxFlexGridSizer *sizer, bool plain = false);
   void AddParam(bx_param_c *param, bool plain = false, AddParamContext *context = NULL);
-  void AddParamList(char *nameList[], bx_param_c *base, wxFlexGridSizer *sizer = NULL, bool plain = false);
+  void AddParamList(const char *nameList[], bx_param_c *base, wxFlexGridSizer *sizer = NULL, bool plain = false);
   virtual void CopyParamToGui();
   bool IsShowing() { return isShowing; }
   void SetRuntimeFlag(bool val) { runtime = val; }
 DECLARE_EVENT_TABLE()
 };
 
+////////////////////////////////////////////////////////////////////////////
+// FloppyConfigDialog
+////////////////////////////////////////////////////////////////////////////
+//
+// the new FloppyConfigDialog is based on ParamDialog. It allows the user to
+// configure the floppy settings and to create a floppy image if necessary.
+class FloppyConfigDialog : public ParamDialog
+{
+private:
+  wxButton *createButton;
+  ParamStruct *pstrDevice, *pstrPath, *pstrMedia, *pstrStatus;
+public:
+  FloppyConfigDialog(wxWindow* parent, wxWindowID id);
+  void Setup(bx_list_c *list);
+  void OnEvent(wxCommandEvent& event);
+  DECLARE_EVENT_TABLE()
+};
+
 ////////////////////////////////////////////////////////////////////////////
 // LogOptionsDialog
 ////////////////////////////////////////////////////////////////////////////
@@ -448,8 +356,8 @@ private:
 #define LOG_OPTS_N_CHOICES_NORMAL 4
 #define LOG_OPTS_N_CHOICES 5   // number of choices, including "no change"
 #define LOG_OPTS_NO_CHANGE 4   // index of "no change"
-// normally all choices are available for all event types. The exclude 
-// expression allows some choices to be eliminated if they don't make any 
+// normally all choices are available for all event types. The exclude
+// expression allows some choices to be eliminated if they don't make any
 // sense.  For example, it would be stupid to ignore a panic.
 #define LOG_OPTS_EXCLUDE(type,choice)  ( \
    /* can't die or ask, on debug or info events */   \
@@ -470,7 +378,7 @@ public:
 ////////////////////////////////////////////////////////////////////////////
 // CpuRegistersDialog
 ////////////////////////////////////////////////////////////////////////////
-// 
+//
 // this would display the current values of all CPU registers, possibly you can
 // enable different groups like debug, FPU, MMX registers.  Certainly if you
 // interrupt the simulation, these would be updated.  we could update
@@ -478,7 +386,7 @@ public:
 // integrated with wxwidgets, you could single step and update the cpu
 // registers, with regs that change marked in a different color.  Modeless
 // dialog.
-// 
+//
 // +--- CPU Registers ---------------------------------------+
 // |                                                         |
 // |  EAX 0x00000000   EIP    0xffff   LDTR 0x00000000       |
@@ -541,7 +449,7 @@ class CpuRegistersDialog : public ParamDialog
   { "TR3", "TR4", "TR5", "TR6", "TR7", \
     NULL }
 #define CPU_REGS_CONTROL_REGS          \
-  { "CR0", "CR1", "CR2", "CR3", "CR4", \
+  { "CR0", "CR2", "CR3", "CR4", \
     NULL  }
 
   void Init();  // called automatically by ShowModal()
@@ -570,7 +478,7 @@ Everything else in here is a comment!
 
 
 ////////////////////////////////////////////////////////////////////////////
-// proposed dialogs, not implemented 
+// proposed dialogs, not implemented
 ////////////////////////////////////////////////////////////////////////////
 
 Here are some quick sketches of what different parts of the interface
@@ -695,7 +603,7 @@ CD-ROM button on the toolbar at runtime.
 ////////////////////////////////////////////////////////////////////////////
 // ChooseConfigDialog
 ////////////////////////////////////////////////////////////////////////////
-The idea is that you could choose from a set of configurations 
+The idea is that you could choose from a set of configurations
 (individual bochsrc files, basically).  When you first install
 Bochs, it would just have DLX Linux Demo, and Create New.
 As you create new configurations and save them, the list
@@ -754,7 +662,7 @@ let you go right to the configure screen for that disk drive.
 |          ++--++                                               |
 |                                                               |
 |          +-----+  C Drive                                     |
-|          |=====|  Hard Disk Image                     +----+  | 
+|          |=====|  Hard Disk Image                     +----+  |
 | [BOOT]   |    o|  C:\Bochs\Images\HD30meg.img         |Edit|  |
 |          +-----+                                      +----+  |
 |                                                               |
@@ -767,10 +675,10 @@ let you go right to the configure screen for that disk drive.
 +---------------------------------------------------------------+
 
 ////////////////////////////////////////////////////////////////////////////
-// KeymappingDialog      
+// KeymappingDialog
 ////////////////////////////////////////////////////////////////////////////
 more ambitious: create a button for each key on a standard keyboard, so that
-you can view/edit/load/save key mappings, produce any combination of keys 
+you can view/edit/load/save key mappings, produce any combination of keys
 (esp. ones that your OS or window manager won't allow)
 
 ////////////////////////////////////////////////////////////////////////////
@@ -791,41 +699,41 @@ complete.
 
 Suggestions from Greg Alexander:
 > I'm using O for radio buttons and [ ] for check boxes.
-> 
+>
 > ---------Basic Configure Timing Dialog--------
-> 
+>
 > Instructions per second:  [_________] (maybe have some default options
 >     and an "other")
-> 
+>
 > Select timing behavior desired:
-> 
+>
 > O Full Speed, Real Time
 >   (NOT Reproducible)
 > O Minimize CPU Use, Real Time
 > O Full speed, NOT Real Time
-> 
+>
 > [Advanced]
-> 
+>
 > -----------------------------------------------
 > The logic for the above would look like this:
 > All options get the New PIT.
 > Option 1 Gets you the Realtime PIT.
 > Option 2 Gets you the Slowdown Timer.
 > Option 3 Gets you neither.
-> 
+>
 > -------Advanced Configure Timing Dialog--------
-> 
+>
 > Instructions per second:  [_________]
-> 
+>
 > Select PIT Model:
 > O Old Model
 > O New Model
 > O Realtime PIT (not reproducible)
-> 
+>
 > Select Optional Realtime Hacks:
 > [ ] Slowdown Timer  Maxmult setting [_________]
 > [ ] Idle Hack (X Windows Only)
-> 
+>
 > ----------------------------------------------
 
 ////////////////////////////////////////////////////////////////////////////
@@ -887,9 +795,9 @@ all the devices.
 
 Try #2 for the advanced event configuration dialog.
 It shows the selection of the default actions again
-at the top, with some explanation.  Then at bottom, you 
-can select a device in the list box and edit the settings 
-for that device individually.  It would be possible to 
+at the top, with some explanation.  Then at bottom, you
+can select a device in the list box and edit the settings
+for that device individually.  It would be possible to
 allow selection of multiple devices and then edit several
 devices at once.
 
@@ -943,7 +851,7 @@ devices at once.
 | +------------------------------------------------------+  |
 |                                [ Help ] [ Cancel ] [ Ok ] |
 +-----------------------------------------------------------+
-                                                           
+
 +---- Configure events -------------------------------------+
 |  __________    ____________                               |
 | | Default  \  | Per Device \                              |
