@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: instrument.h,v 1.25 2008/04/19 10:12:09 sshwarts Exp $
+// $Id: instrument.h,v 1.33 2009/03/22 09:52:48 sshwarts Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,7 +22,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
 
 // possible types passed to BX_INSTR_TLB_CNTRL()
@@ -56,6 +56,10 @@
 
 class bxInstruction_c;
 
+void bx_instr_init_env(void);
+void bx_instr_exit_env(void);
+void bx_instr_initialize(unsigned cpu);
+
 // maximum size of an instruction
 #define MAX_OPCODE_SIZE 16
 
@@ -72,7 +76,6 @@ public:
 
   /* decoding */
   unsigned opcode_size;
-  unsigned nprefixes;
   Bit8u    opcode[MAX_OPCODE_SIZE];
   bx_bool  is32, is64;
 
@@ -100,7 +103,7 @@ public:
   void toggle_active() { active = !active; }
   bx_bool is_active() const { return active; }
 
-  void bx_instr_reset();
+  void bx_instr_reset(unsigned type);
   void bx_instr_new_instruction();
 
   void bx_instr_cnear_branch_taken(bx_address new_eip);
@@ -109,12 +112,9 @@ public:
   void bx_instr_far_branch(unsigned what, Bit16u new_cs, bx_address new_eip);
 
   void bx_instr_opcode(const Bit8u *opcode, unsigned len, bx_bool is32, bx_bool is64);
-  void bx_instr_fetch_decode_completed(bxInstruction_c *i);
-
-  void bx_instr_prefix(Bit8u prefix);
 
   void bx_instr_interrupt(unsigned vector);
-  void bx_instr_exception(unsigned vector);
+  void bx_instr_exception(unsigned vector, unsigned error_code);
   void bx_instr_hwinterrupt(unsigned vector, Bit16u cs, bx_address eip);
 
   void bx_instr_mem_data_access(unsigned seg, bx_address offset, unsigned len, unsigned rw);
@@ -127,133 +127,132 @@ void bx_instr_init(unsigned cpu);
 
 extern bxInstrumentation *icpu;
 
-/* simulation init, shutdown, reset */
-#  define BX_INSTR_INIT(cpu_id)	           bx_instr_init(cpu_id);
-#  define BX_INSTR_EXIT(cpu_id)
-#  define BX_INSTR_RESET(cpu_id)           icpu[cpu_id].bx_instr_reset()
-#  define BX_INSTR_HLT(cpu_id)
-#  define BX_INSTR_MWAIT(cpu_id, addr, len, flags)
+/* initialization/deinitialization of instrumentalization*/
+#define BX_INSTR_INIT_ENV() bx_instr_init_env()
+#define BX_INSTR_EXIT_ENV() bx_instr_exit_env()
 
-#  define BX_INSTR_NEW_INSTRUCTION(cpu_id) icpu[cpu_id].bx_instr_new_instruction()
+/* simulation init, shutdown, reset */
+#define BX_INSTR_INITIALIZE(cpu_id)	   bx_instr_initialize(cpu_id);
+#define BX_INSTR_EXIT(cpu_id)
+#define BX_INSTR_RESET(cpu_id, type)     icpu[cpu_id].bx_instr_reset(type)
+#define BX_INSTR_HLT(cpu_id)
+#define BX_INSTR_MWAIT(cpu_id, addr, len, flags)
+
+#define BX_INSTR_NEW_INSTRUCTION(cpu_id) icpu[cpu_id].bx_instr_new_instruction()
 
 /* called from command line debugger */
-#  define BX_INSTR_DEBUG_PROMPT()
-#  define BX_INSTR_START()
-#  define BX_INSTR_STOP()
-#  define BX_INSTR_PRINT()
+#define BX_INSTR_DEBUG_PROMPT()
+#define BX_INSTR_START()
+#define BX_INSTR_STOP()
+#define BX_INSTR_PRINT()
 
 /* branch resoultion */
-#  define BX_INSTR_CNEAR_BRANCH_TAKEN(cpu_id, new_eip)       icpu[cpu_id].bx_instr_cnear_branch_taken(new_eip)
-#  define BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(cpu_id)   icpu[cpu_id].bx_instr_cnear_branch_not_taken()
-#  define BX_INSTR_UCNEAR_BRANCH(cpu_id, what, new_eip)      icpu[cpu_id].bx_instr_ucnear_branch(what, new_eip)
-#  define BX_INSTR_FAR_BRANCH(cpu_id, what, new_cs, new_eip) icpu[cpu_id].bx_instr_far_branch(what, new_cs, new_eip)
+#define BX_INSTR_CNEAR_BRANCH_TAKEN(cpu_id, new_eip)       icpu[cpu_id].bx_instr_cnear_branch_taken(new_eip)
+#define BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(cpu_id)   icpu[cpu_id].bx_instr_cnear_branch_not_taken()
+#define BX_INSTR_UCNEAR_BRANCH(cpu_id, what, new_eip)      icpu[cpu_id].bx_instr_ucnear_branch(what, new_eip)
+#define BX_INSTR_FAR_BRANCH(cpu_id, what, new_cs, new_eip) icpu[cpu_id].bx_instr_far_branch(what, new_cs, new_eip)
 
 /* decoding completed */
-#  define BX_INSTR_OPCODE(cpu_id, opcode, len, is32, is64) \
+#define BX_INSTR_OPCODE(cpu_id, opcode, len, is32, is64) \
                        icpu[cpu_id].bx_instr_opcode(opcode, len, is32, is64)
-#  define BX_INSTR_FETCH_DECODE_COMPLETED(cpu_id, i) \
-                       icpu[cpu_id].bx_instr_fetch_decode_completed(i)
-
-/* prefix byte decoded */
-#  define BX_INSTR_PREFIX(cpu_id, prefix)  icpu[cpu_id].bx_instr_prefix(prefix)
 
 /* exceptional case and interrupt */
-#  define BX_INSTR_EXCEPTION(cpu_id, vector)            icpu[cpu_id].bx_instr_exception(vector)
-#  define BX_INSTR_INTERRUPT(cpu_id, vector)            icpu[cpu_id].bx_instr_interrupt(vector)
-#  define BX_INSTR_HWINTERRUPT(cpu_id, vector, cs, eip) icpu[cpu_id].bx_instr_hwinterrupt(vector, cs, eip)
+#define BX_INSTR_EXCEPTION(cpu_id, vector, error_code) \
+                       icpu[cpu_id].bx_instr_exception(vector, error_code)
+
+#define BX_INSTR_INTERRUPT(cpu_id, vector) icpu[cpu_id].bx_instr_interrupt(vector)
+#define BX_INSTR_HWINTERRUPT(cpu_id, vector, cs, eip) icpu[cpu_id].bx_instr_hwinterrupt(vector, cs, eip)
 
 /* TLB/CACHE control instruction executed */
-#  define BX_INSTR_CLFLUSH(cpu_id, laddr, paddr)
-#  define BX_INSTR_CACHE_CNTRL(cpu_id, what)
-#  define BX_INSTR_TLB_CNTRL(cpu_id, what, new_cr3)
-#  define BX_INSTR_PREFETCH_HINT(cpu_id, what, seg, offset)
+#define BX_INSTR_CLFLUSH(cpu_id, laddr, paddr)
+#define BX_INSTR_CACHE_CNTRL(cpu_id, what)
+#define BX_INSTR_TLB_CNTRL(cpu_id, what, new_cr3)
+#define BX_INSTR_PREFETCH_HINT(cpu_id, what, seg, offset)
 
 /* execution */
-#  define BX_INSTR_BEFORE_EXECUTION(cpu_id, i)
-#  define BX_INSTR_AFTER_EXECUTION(cpu_id, i)
-#  define BX_INSTR_REPEAT_ITERATION(cpu_id, i)
+#define BX_INSTR_BEFORE_EXECUTION(cpu_id, i)
+#define BX_INSTR_AFTER_EXECUTION(cpu_id, i)
+#define BX_INSTR_REPEAT_ITERATION(cpu_id, i)
 
 /* memory access */
-#  define BX_INSTR_LIN_ACCESS(cpu_id, lin, phy, len, rw)
+#define BX_INSTR_LIN_ACCESS(cpu_id, lin, phy, len, rw)
 
-#  define BX_INSTR_MEM_DATA_ACCESS(cpu_id, seg, offset, len, rw) \
+#define BX_INSTR_MEM_DATA_ACCESS(cpu_id, seg, offset, len, rw) \
                     icpu[cpu_id].bx_instr_mem_data_access(seg, offset, len, rw)
 
 /* called from memory object */
-#  define BX_INSTR_PHY_WRITE(cpu_id, addr, len)
-#  define BX_INSTR_PHY_READ(cpu_id, addr, len)
+#define BX_INSTR_PHY_WRITE(cpu_id, addr, len)
+#define BX_INSTR_PHY_READ(cpu_id, addr, len)
 
 /* feedback from device units */
-#  define BX_INSTR_INP(addr, len)
-#  define BX_INSTR_INP2(addr, len, val)
-#  define BX_INSTR_OUTP(addr, len)
-#  define BX_INSTR_OUTP2(addr, len, val)
+#define BX_INSTR_INP(addr, len)
+#define BX_INSTR_INP2(addr, len, val)
+#define BX_INSTR_OUTP(addr, len, val)
 
 /* wrmsr callback */
-#  define BX_INSTR_WRMSR(cpu_id, addr, value)
+#define BX_INSTR_WRMSR(cpu_id, addr, value)
 
-#else
+#else // BX_INSTRUMENTATION
+
+/* initialization/deinitialization of instrumentalization */
+#define BX_INSTR_INIT_ENV()
+#define BX_INSTR_EXIT_ENV()
 
 /* simulation init, shutdown, reset */
-#  define BX_INSTR_INIT(cpu_id)
-#  define BX_INSTR_EXIT(cpu_id)
-#  define BX_INSTR_RESET(cpu_id)
-#  define BX_INSTR_HLT(cpu_id)
-#  define BX_INSTR_MWAIT(cpu_id, addr, len, flags)
-#  define BX_INSTR_NEW_INSTRUCTION(cpu_id)
+#define BX_INSTR_INITIALIZE(cpu_id)
+#define BX_INSTR_EXIT(cpu_id)
+#define BX_INSTR_RESET(cpu_id, type)
+#define BX_INSTR_HLT(cpu_id)
+#define BX_INSTR_MWAIT(cpu_id, addr, len, flags)
+#define BX_INSTR_NEW_INSTRUCTION(cpu_id)
 
 /* called from command line debugger */
-#  define BX_INSTR_DEBUG_PROMPT()
-#  define BX_INSTR_START()
-#  define BX_INSTR_STOP()
-#  define BX_INSTR_PRINT()
+#define BX_INSTR_DEBUG_PROMPT()
+#define BX_INSTR_START()
+#define BX_INSTR_STOP()
+#define BX_INSTR_PRINT()
 
 /* branch resoultion */
-#  define BX_INSTR_CNEAR_BRANCH_TAKEN(cpu_id, new_eip)
-#  define BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(cpu_id)
-#  define BX_INSTR_UCNEAR_BRANCH(cpu_id, what, new_eip)
-#  define BX_INSTR_FAR_BRANCH(cpu_id, what, new_cs, new_eip)
+#define BX_INSTR_CNEAR_BRANCH_TAKEN(cpu_id, new_eip)
+#define BX_INSTR_CNEAR_BRANCH_NOT_TAKEN(cpu_id)
+#define BX_INSTR_UCNEAR_BRANCH(cpu_id, what, new_eip)
+#define BX_INSTR_FAR_BRANCH(cpu_id, what, new_cs, new_eip)
 
 /* decoding completed */
-#  define BX_INSTR_OPCODE(cpu_id, opcode, len, is32, is64)
-#  define BX_INSTR_FETCH_DECODE_COMPLETED(cpu_id, i)
-
-/* prefix byte decoded */
-#  define BX_INSTR_PREFIX(cpu_id, prefix)
+#define BX_INSTR_OPCODE(cpu_id, opcode, len, is32, is64)
 
 /* exceptional case and interrupt */
-#  define BX_INSTR_EXCEPTION(cpu_id, vector)
-#  define BX_INSTR_INTERRUPT(cpu_id, vector)
-#  define BX_INSTR_HWINTERRUPT(cpu_id, vector, cs, eip)
+#define BX_INSTR_EXCEPTION(cpu_id, vector, error_code)
+#define BX_INSTR_INTERRUPT(cpu_id, vector)
+#define BX_INSTR_HWINTERRUPT(cpu_id, vector, cs, eip)
 
 /* TLB/CACHE control instruction executed */
-#  define BX_INSTR_CLFLUSH(cpu_id, laddr, paddr)
-#  define BX_INSTR_CACHE_CNTRL(cpu_id, what)
-#  define BX_INSTR_TLB_CNTRL(cpu_id, what, new_cr3)
-#  define BX_INSTR_PREFETCH_HINT(cpu_id, what, seg, offset)
+#define BX_INSTR_CLFLUSH(cpu_id, laddr, paddr)
+#define BX_INSTR_CACHE_CNTRL(cpu_id, what)
+#define BX_INSTR_TLB_CNTRL(cpu_id, what, new_cr3)
+#define BX_INSTR_PREFETCH_HINT(cpu_id, what, seg, offset)
 
 /* execution */
-#  define BX_INSTR_BEFORE_EXECUTION(cpu_id, i)
-#  define BX_INSTR_AFTER_EXECUTION(cpu_id, i)
-#  define BX_INSTR_REPEAT_ITERATION(cpu_id, i)
+#define BX_INSTR_BEFORE_EXECUTION(cpu_id, i)
+#define BX_INSTR_AFTER_EXECUTION(cpu_id, i)
+#define BX_INSTR_REPEAT_ITERATION(cpu_id, i)
 
 /* memory access */
-#  define BX_INSTR_LIN_ACCESS(cpu_id, lin, phy, len, rw)
+#define BX_INSTR_LIN_ACCESS(cpu_id, lin, phy, len, rw)
 
 /* memory access */
-#  define BX_INSTR_MEM_DATA_ACCESS(cpu_id, seg, offset, len, rw)
+#define BX_INSTR_MEM_DATA_ACCESS(cpu_id, seg, offset, len, rw)
 
 /* called from memory object */
-#  define BX_INSTR_PHY_WRITE(cpu_id, addr, len)
-#  define BX_INSTR_PHY_READ(cpu_id, addr, len)
+#define BX_INSTR_PHY_WRITE(cpu_id, addr, len)
+#define BX_INSTR_PHY_READ(cpu_id, addr, len)
 
 /* feedback from device units */
-#  define BX_INSTR_INP(addr, len)
-#  define BX_INSTR_INP2(addr, len, val)
-#  define BX_INSTR_OUTP(addr, len)
-#  define BX_INSTR_OUTP2(addr, len, val)
+#define BX_INSTR_INP(addr, len)
+#define BX_INSTR_INP2(addr, len, val)
+#define BX_INSTR_OUTP(addr, len, val)
 
 /* wrmsr callback */
-#  define BX_INSTR_WRMSR(cpu_id, addr, value)
+#define BX_INSTR_WRMSR(cpu_id, addr, value)
 
-#endif
+#endif // BX_INSTRUMENTATION
