@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: instrument.cc,v 1.26 2008/04/19 10:12:09 sshwarts Exp $
+// $Id: instrument.cc,v 1.32 2009/02/09 10:35:55 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,16 +22,19 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
 
 #include "bochs.h"
 
 #if BX_INSTRUMENTATION
 
-void bx_instr_init(unsigned cpu) {}
+void bx_instr_init_env(void) {}
+void bx_instr_exit_env(void) {}
+
+void bx_instr_initialize(unsigned cpu) {}
 void bx_instr_exit(unsigned cpu) {}
-void bx_instr_reset(unsigned cpu) {}
+void bx_instr_reset(unsigned cpu, unsigned type) {}
 void bx_instr_hlt(unsigned cpu) {}
 void bx_instr_mwait(unsigned cpu, bx_phy_address addr, unsigned len, Bit32u flags) {}
 void bx_instr_new_instruction(unsigned cpu) {}
@@ -47,12 +50,9 @@ void bx_instr_ucnear_branch(unsigned cpu, unsigned what, bx_address new_eip) {}
 void bx_instr_far_branch(unsigned cpu, unsigned what, Bit16u new_cs, bx_address new_eip) {}
 
 void bx_instr_opcode(unsigned cpu, const Bit8u *opcode, unsigned len, bx_bool is32, bx_bool is64) {}
-void bx_instr_fetch_decode_completed(unsigned cpu, bxInstruction_c *i) {}
-
-void bx_instr_prefix(unsigned cpu, Bit8u prefix) {}
 
 void bx_instr_interrupt(unsigned cpu, unsigned vector) {}
-void bx_instr_exception(unsigned cpu, unsigned vector) {}
+void bx_instr_exception(unsigned cpu, unsigned vector, unsigned error_code) {}
 void bx_instr_hwinterrupt(unsigned cpu, unsigned vector, Bit16u cs, bx_address eip) {}
 
 void bx_instr_tlb_cntrl(unsigned cpu, unsigned what, bx_phy_address new_cr3) {}
@@ -65,9 +65,8 @@ void bx_instr_after_execution(unsigned cpu, bxInstruction_c *i) {}
 void bx_instr_repeat_iteration(unsigned cpu, bxInstruction_c *i) {}
 
 void bx_instr_inp(Bit16u addr, unsigned len) {}
-void bx_instr_outp(Bit16u addr, unsigned len) {}
 void bx_instr_inp2(Bit16u addr, unsigned len, unsigned val) {}
-void bx_instr_outp2(Bit16u addr, unsigned len, unsigned val) {}
+void bx_instr_outp(Bit16u addr, unsigned len, unsigned val) {}
 
 void bx_instr_mem_data_access(unsigned cpu, unsigned seg, bx_address offset, unsigned len, unsigned rw) {}
 void bx_instr_lin_access(unsigned cpu, bx_address lin, bx_address phy, unsigned len, unsigned rw) {}
