@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: eth.cc,v 1.24 2005/12/10 18:37:35 vruppert Exp $
+// $Id: eth.cc,v 1.31 2009/04/19 17:25:40 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,7 +22,8 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+//
 
 // eth.cc  - helper code to find and create pktmover classes
 
@@ -30,10 +31,10 @@
 // NE2000/ether stuff.
 
 // Define BX_PLUGGABLE in files that can be compiled into plugins.  For
-// platforms that require a special tag on exported symbols, BX_PLUGGABLE 
+// platforms that require a special tag on exported symbols, BX_PLUGGABLE
 // is used to know when we are exporting symbols and when we are importing.
 #define BX_PLUGGABLE
- 
+
 #define NO_DEVICE_INCLUDES
 #include "iodev.h"
 
@@ -47,7 +48,7 @@ eth_locator_c *eth_locator_c::all;
 
 //
 // Each pktmover module has a static locator class that registers
-// here 
+// here
 //
 eth_locator_c::eth_locator_c(const char *type)
 {
@@ -88,13 +89,14 @@ extern class bx_vnet_locator_c bx_vnet_match;
 //
 eth_pktmover_c *
 eth_locator_c::create(const char *type, const char *netif,
-		      const char *macaddr,
-		      eth_rx_handler_t rxh, void *rxarg, char *script)
+                      const char *macaddr,
+                      eth_rx_handler_t rxh, bx_devmodel_c *dev,
+                      const char *script)
 {
 #ifdef eth_static_constructors
   for (eth_locator_c *p = all; p != NULL; p = p->next) {
     if (strcmp(type, p->type) == 0)
-      return (p->allocate(netif, macaddr, rxh, rxarg, script));
+      return (p->allocate(netif, macaddr, rxh, dev, script));
   }
 #else
   eth_locator_c *ptr = 0;
@@ -108,36 +110,36 @@ eth_locator_c::create(const char *type, const char *netif,
 #ifdef ETH_NULL
   {
     if (!strcmp(type, "null"))
-      ptr = (eth_locator_c *) &bx_null_match; 
+      ptr = (eth_locator_c *) &bx_null_match;
   }
 #endif
 #ifdef ETH_FBSD
   {
-    if (!strcmp(type, "fbsd"))    
+    if (!strcmp(type, "fbsd"))
       ptr = (eth_locator_c *) &bx_fbsd_match;
   }
 #endif
 #ifdef ETH_LINUX
   {
-    if (!strcmp(type, "linux"))    
+    if (!strcmp(type, "linux"))
       ptr = (eth_locator_c *) &bx_linux_match;
   }
 #endif
 #if HAVE_TUNTAP
   {
-    if (!strcmp(type, "tuntap"))    
+    if (!strcmp(type, "tuntap"))
       ptr = (eth_locator_c *) &bx_tuntap_match;
   }
 #endif
 #if HAVE_VDE
   {
-    if (!strcmp(type, "vde"))    
+    if (!strcmp(type, "vde"))
       ptr = (eth_locator_c *) &bx_vde_match;
   }
 #endif
 #if HAVE_ETHERTAP
   {
-    if (!strcmp(type, "tap"))    
+    if (!strcmp(type, "tap"))
       ptr = (eth_locator_c *) &bx_tap_match;
   }
 #endif
@@ -148,11 +150,11 @@ eth_locator_c::create(const char *type, const char *netif,
   }
 #endif
   {
-    if (!strcmp(type, "vnet"))    
+    if (!strcmp(type, "vnet"))
       ptr = (eth_locator_c *) &bx_vnet_match;
   }
   if (ptr)
-    return (ptr->allocate(netif, macaddr, rxh, rxarg, script));
+    return (ptr->allocate(netif, macaddr, rxh, dev, script));
 #endif
 
   return (NULL);
@@ -164,27 +166,24 @@ extern "C" {
 #include <sys/wait.h>
 };
 
-#undef LOG_THIS
-#define LOG_THIS bx_devices.pluginNE2kDevice->
-
 // This is a utility script used for tuntap or ethertap
-int execute_script( char* scriptname, char* arg1 )
+int execute_script(bx_devmodel_c *netdev, const char* scriptname, char* arg1)
 {
   int pid,status;
 
   if (!(pid=fork())) {
     char filename[BX_PATHNAME_LEN];
-    if ( scriptname[0]=='/' ) {
-      strcpy (filename, scriptname);
+    if (scriptname[0]=='/') {
+      strcpy(filename, scriptname);
     }
     else {
-      getcwd (filename, BX_PATHNAME_LEN);
-      strcat (filename, "/");
-      strcat (filename, scriptname);
+      getcwd(filename, BX_PATHNAME_LEN);
+      strcat(filename, "/");
+      strcat(filename, scriptname);
     }
 
     // execute the script
-    BX_INFO(("Executing script '%s %s'",filename,arg1));
+    netdev->info("Executing script '%s %s'",filename,arg1);
     execle(filename, scriptname, arg1, NULL, NULL);
 
     // if we get here there has been a problem
@@ -200,4 +199,23 @@ int execute_script( char* scriptname, char* arg1 )
 
 #endif // (HAVE_ETHERTAP==1) || (HAVE_TUNTAP==1)
 
+void write_pktlog_txt(FILE *pktlog_txt, const Bit8u *buf, unsigned len, bx_bool host_to_guest)
+{
+  Bit8u *charbuf = (Bit8u *)buf;
+  unsigned n;
+
+  if (!host_to_guest) {
+    fprintf(pktlog_txt, "a packet from guest to host, length %u\n", len);
+  } else {
+    fprintf(pktlog_txt, "a packet from host to guest, length %u\n", len);
+  }
+  for (n = 0; n < len; n++) {
+    if (((n % 16) == 0) && (n > 0))
+      fprintf(pktlog_txt, "\n");
+    fprintf(pktlog_txt, "%02x ", (unsigned)charbuf[n]);
+  }
+  fprintf(pktlog_txt, "\n--\n");
+  fflush(pktlog_txt);
+}
+
 #endif /* if BX_NETWORKING */
