@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: eth_linux.cc,v 1.23 2008/01/26 22:24:01 sshwarts Exp $
+// $Id: eth_linux.cc,v 1.27 2009/04/13 13:33:11 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,7 +22,8 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
+//
 
 // Peter Grehan (grehan@iprg.nokia.com) coded all of this
 // NE2000/ether stuff.
@@ -52,7 +53,7 @@
 
 #include "eth.h"
 
-#define LOG_THIS bx_devices.pluginNE2kDevice->
+#define LOG_THIS netdev->
 
 extern "C" {
 #include <errno.h>
@@ -97,10 +98,10 @@ static const struct sock_filter promiscfilter[] = {
 class bx_linux_pktmover_c : public eth_pktmover_c {
 public:
   bx_linux_pktmover_c(const char *netif,
-		     const char *macaddr,
-		     eth_rx_handler_t rxh,
-		     void *rxarg,
-		     char *script);
+                      const char *macaddr,
+                      eth_rx_handler_t rxh,
+                      bx_devmodel_c *dev,
+                      const char *script);
   void sendpkt(void *buf, unsigned io_len);
 
 private:
@@ -123,10 +124,10 @@ public:
   bx_linux_locator_c(void) : eth_locator_c("linux") {}
 protected:
   eth_pktmover_c *allocate(const char *netif,
-			   const char *macaddr,
-			   eth_rx_handler_t rxh,
-			   void *rxarg, char *script) {
-    return (new bx_linux_pktmover_c(netif, macaddr, rxh, rxarg, script));
+                           const char *macaddr,
+                           eth_rx_handler_t rxh,
+                           bx_devmodel_c *dev, const char *script) {
+    return (new bx_linux_pktmover_c(netif, macaddr, rxh, dev, script));
   }
 } bx_linux_match;
 
@@ -138,16 +139,17 @@ protected:
 // the constructor
 //
 bx_linux_pktmover_c::bx_linux_pktmover_c(const char *netif,
-				       const char *macaddr,
-				       eth_rx_handler_t rxh,
-				       void *rxarg,
-				       char *script)
+                                         const char *macaddr,
+                                         eth_rx_handler_t rxh,
+                                         bx_devmodel_c *dev,
+                                         const char *script)
 {
   struct sockaddr_ll sll;
   struct packet_mreq mr;
   struct ifreq ifr;
   struct sock_fprog fp;
 
+  this->netdev = dev;
   memcpy(linux_macaddr, macaddr, 6);
 
   // Open packet socket
@@ -228,11 +230,10 @@ bx_linux_pktmover_c::bx_linux_pktmover_c(const char *netif,
   // Start the rx poll
   this->rx_timer_index =
     bx_pc_system.register_timer(this, this->rx_timer_handler, BX_PACKET_POLL,
-				1, 1, "eth_linux"); // continuous, active
+                                1, 1, "eth_linux"); // continuous, active
 
   this->rxh   = rxh;
-  this->rxarg = rxarg;
-  BX_INFO(("eth_linux: enabled NE2K emulation on interface %s", netif));
+  BX_INFO(("linux network driver initialized: using interface %s", netif));
 }
 
 // the output routine - called with pre-formatted ethernet frame.
@@ -284,7 +285,7 @@ bx_linux_pktmover_c::rx_timer(void)
   // let through broadcast, multicast, and our mac address
 //  if ((memcmp(rxbuf, broadcast_macaddr, 6) == 0) || (memcmp(rxbuf, this->linux_macaddr, 6) == 0) || rxbuf[0] & 0x01) {
     BX_DEBUG(("eth_linux: got packet: %d bytes, dst=%x:%x:%x:%x:%x:%x, src=%x:%x:%x:%x:%x:%x\n", nbytes, rxbuf[0], rxbuf[1], rxbuf[2], rxbuf[3], rxbuf[4], rxbuf[5], rxbuf[6], rxbuf[7], rxbuf[8], rxbuf[9], rxbuf[10], rxbuf[11]));
-    (*rxh)(rxarg, rxbuf, nbytes);
+    (*rxh)(netdev, rxbuf, nbytes);
 //  }
 }
 #endif /* if BX_NETWORKING && defined ETH_LINUX */
