@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: logio.cc,v 1.69 2008/05/23 18:06:50 sshwarts Exp $
+// $Id: logio.cc,v 1.73 2009/04/10 08:15:25 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,7 +22,7 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 
 #include "bochs.h"
@@ -61,7 +61,6 @@ void iofunctions::init(void)
   init_log(stderr);
   log = new logfunc_t(this);
   log->put("IO");
-  log->settype(IOLOG);
   log->ldebug("Init(log file: '%s').",logfn);
 }
 
@@ -161,18 +160,18 @@ void iofunctions::set_log_prefix(const char* prefix)
   strcpy(logprefix, prefix);
 }
 
-//  iofunctions::out(class, level, prefix, fmt, ap)
+//  iofunctions::out(level, prefix, fmt, ap)
 //  DO NOT nest out() from ::info() and the like.
 //    fmt and ap retained for direct printinf from iofunctions only!
 
-void iofunctions::out(int f, int l, const char *prefix, const char *fmt, va_list ap)
+void iofunctions::out(int level, const char *prefix, const char *fmt, va_list ap)
 {
   char c=' ', *s;
   assert(magic==MAGIC_LOGNUM);
   assert(this != NULL);
   assert(logfd != NULL);
 
-  switch(l) {
+  switch (level) {
     case LOGLEV_INFO: c='i'; break;
     case LOGLEV_PANIC: c='p'; break;
     case LOGLEV_PASS: c='s'; break;
@@ -217,9 +216,9 @@ void iofunctions::out(int f, int l, const char *prefix, const char *fmt, va_list
 
   fprintf(logfd," ");
 
-  if(l==LOGLEV_PANIC)
+  if(level==LOGLEV_PANIC)
     fprintf(logfd, ">>PANIC<< ");
-  if(l==LOGLEV_PASS)
+  if(level==LOGLEV_PASS)
     fprintf(logfd, ">>PASS<< ");
 
   vfprintf(logfd, fmt, ap);
@@ -276,7 +275,6 @@ logfunctions::logfunctions(void)
 {
   prefix = NULL;
   put(" ");
-  settype(GENLOG);
   if (io == NULL && Allocio == 0) {
     Allocio = 1;
     io = new iofunc_t(stderr);
@@ -292,7 +290,6 @@ logfunctions::logfunctions(iofunc_t *iofunc)
 {
   prefix = NULL;
   put(" ");
-  settype(GENLOG);
   setio(iofunc);
   // BUG: unfortunately this can be called before the bochsrc is read,
   // which means that the bochsrc has no effect on the actions.
@@ -302,70 +299,57 @@ logfunctions::logfunctions(iofunc_t *iofunc)
 
 logfunctions::~logfunctions()
 {
-  this->logio->remove_logfn(this);
+  logio->remove_logfn(this);
   if (prefix) free(prefix);
 }
 
 void logfunctions::setio(iofunc_t *i)
 {
   // add pointer to iofunction object to use
-  this->logio = i;
+  logio = i;
   // give iofunction a pointer to me
   i->add_logfn(this);
 }
 
 void logfunctions::put(const char *p)
 {
-  char * tmpbuf=strdup("[     ]");   // if we ever have more than 32 chars,
-                                     // we need to rethink this
+  char *tmpbuf=strdup("[     ]");   // if we ever have more than 32 chars,
+                                    // we need to rethink this
 
   if (tmpbuf == NULL)
-  {
-    return;                          // allocation not successful
-  }
+    return;                         // allocation not successful
 
-  if (this->prefix != NULL)
+  if (prefix != NULL)
   {
-    free(this->prefix);              // free previously allocated memory
+    free(prefix);             // free previously allocated memory
     prefix = NULL;
   }
 
   size_t len=strlen(p);
-  for(size_t i=1;i<len+1;i++) {
+  if (len > (strlen(tmpbuf) - 2)) {
+    len = strlen(tmpbuf) - 2;
+  }
+  for(size_t i=1;i <= len;i++) {
     tmpbuf[i]=p[i-1];
   }
 
-  switch(len) {
-  case  1: tmpbuf[2]=' ';
-  case  2: tmpbuf[3]=' ';
-  case  3: tmpbuf[4]=' ';
-  case  4: tmpbuf[5]=' ';
-  default: tmpbuf[6]=']'; tmpbuf[7]='\0'; break;
-  }
-
-  prefix=tmpbuf;
-}
-
-void logfunctions::settype(int t)
-{
-  type=t;
+  prefix = tmpbuf;
 }
 
 void logfunctions::info(const char *fmt, ...)
 {
   va_list ap;
 
-  assert(this != NULL);
-  assert(this->logio != NULL);
+  assert(logio != NULL);
 
   if(!onoff[LOGLEV_INFO]) return;
 
   va_start(ap, fmt);
-  this->logio->out(this->type,LOGLEV_INFO,this->prefix, fmt, ap);
+  logio->out(LOGLEV_INFO, prefix, fmt, ap);
   if (onoff[LOGLEV_INFO] == ACT_ASK)
-    ask(LOGLEV_INFO, this->prefix, fmt, ap);
+    ask(LOGLEV_INFO, prefix, fmt, ap);
   if (onoff[LOGLEV_INFO] == ACT_FATAL)
-    fatal(this->prefix, fmt, ap, 1);
+    fatal(prefix, fmt, ap, 1);
   va_end(ap);
 }
 
@@ -373,17 +357,16 @@ void logfunctions::error(const char *fmt, ...)
 {
   va_list ap;
 
-  assert(this != NULL);
-  assert(this->logio != NULL);
+  assert(logio != NULL);
 
   if(!onoff[LOGLEV_ERROR]) return;
 
   va_start(ap, fmt);
-  this->logio->out(this->type,LOGLEV_ERROR,this->prefix, fmt, ap);
+  logio->out(LOGLEV_ERROR, prefix, fmt, ap);
   if (onoff[LOGLEV_ERROR] == ACT_ASK)
-    ask(LOGLEV_ERROR, this->prefix, fmt, ap);
+    ask(LOGLEV_ERROR, prefix, fmt, ap);
   if (onoff[LOGLEV_ERROR] == ACT_FATAL)
-    fatal(this->prefix, fmt, ap, 1);
+    fatal(prefix, fmt, ap, 1);
   va_end(ap);
 }
 
@@ -391,24 +374,23 @@ void logfunctions::panic(const char *fmt, ...)
 {
   va_list ap;
 
-  assert(this != NULL);
-  assert(this->logio != NULL);
+  assert(logio != NULL);
 
   // Special case for panics since they are so important.  Always print
   // the panic to the log, no matter what the log action says.
   //if(!onoff[LOGLEV_PANIC]) return;
 
   va_start(ap, fmt);
-  this->logio->out(this->type,LOGLEV_PANIC,this->prefix, fmt, ap);
+  logio->out(LOGLEV_PANIC, prefix, fmt, ap);
 
   // This fixes a funny bug on linuxppc where va_list is no pointer but a struct
   va_end(ap);
   va_start(ap, fmt);
 
   if (onoff[LOGLEV_PANIC] == ACT_ASK)
-    ask(LOGLEV_PANIC, this->prefix, fmt, ap);
+    ask(LOGLEV_PANIC, prefix, fmt, ap);
   if (onoff[LOGLEV_PANIC] == ACT_FATAL)
-    fatal(this->prefix, fmt, ap, 1);
+    fatal(prefix, fmt, ap, 1);
   va_end(ap);
 }
 
@@ -416,24 +398,23 @@ void logfunctions::pass(const char *fmt, ...)
 {
   va_list ap;
 
-  assert(this != NULL);
-  assert(this->logio != NULL);
+  assert(logio != NULL);
 
   // Special case for panics since they are so important.  Always print
   // the panic to the log, no matter what the log action says.
   //if(!onoff[LOGLEV_PASS]) return;
 
   va_start(ap, fmt);
-  this->logio->out(this->type,LOGLEV_PASS,this->prefix, fmt, ap);
+  logio->out(LOGLEV_PASS, prefix, fmt, ap);
 
   // This fixes a funny bug on linuxppc where va_list is no pointer but a struct
   va_end(ap);
   va_start(ap, fmt);
 
   if (onoff[LOGLEV_PASS] == ACT_ASK)
-    ask(LOGLEV_PASS, this->prefix, fmt, ap);
+    ask(LOGLEV_PASS, prefix, fmt, ap);
   if (onoff[LOGLEV_PASS] == ACT_FATAL)
-    fatal(this->prefix, fmt, ap, 101);
+    fatal(prefix, fmt, ap, 101);
   va_end(ap);
 }
 
@@ -441,17 +422,16 @@ void logfunctions::ldebug(const char *fmt, ...)
 {
   va_list ap;
 
-  assert(this != NULL);
-  assert(this->logio != NULL);
+  assert(logio != NULL);
 
   if(!onoff[LOGLEV_DEBUG]) return;
 
   va_start(ap, fmt);
-  this->logio->out(this->type,LOGLEV_DEBUG,this->prefix, fmt, ap);
+  logio->out(LOGLEV_DEBUG, prefix, fmt, ap);
   if (onoff[LOGLEV_DEBUG] == ACT_ASK)
-    ask(LOGLEV_DEBUG, this->prefix, fmt, ap);
+    ask(LOGLEV_DEBUG, prefix, fmt, ap);
   if (onoff[LOGLEV_DEBUG] == ACT_FATAL)
-    fatal(this->prefix, fmt, ap, 1);
+    fatal(prefix, fmt, ap, 1);
   va_end(ap);
 }
 
@@ -477,7 +457,6 @@ void logfunctions::ask(int level, const char *prefix, const char *fmt, va_list a
   // the reentry check above.
   if (SIM->get_init_done()) DEV_vga_refresh();
 
-#if !BX_EXTERNAL_DEBUGGER
   // ensure the text screen is showing
   SIM->set_display_mode(DISP_MODE_CONFIG);
   int val = SIM->log_msg(prefix, level, buf1);
@@ -528,9 +507,6 @@ void logfunctions::ask(int level, const char *prefix, const char *fmt, va_list a
       // in gui/control.cc.
       fprintf(stderr, "WARNING: log_msg returned unexpected value %d\n", val);
   }
-#else
-  // external debugger ask code goes here
-#endif
   // return to simulation mode
   SIM->set_display_mode(DISP_MODE_SIM);
   in_ask_already = 0;
