@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: osdep.cc,v 1.17 2006/08/07 17:59:15 vruppert Exp $
+// $Id: osdep.cc,v 1.19.2.1 2009/06/07 07:49:09 vruppert Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2001  MandrakeSoft S.A.
@@ -22,22 +22,23 @@
 //
 //  You should have received a copy of the GNU Lesser General Public
 //  License along with this library; if not, write to the Free Software
-//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+/////////////////////////////////////////////////////////////////////////
 
 //
 // osdep.cc
-// 
+//
 // Provide definition of library functions that are missing on various
 // systems.  The only reason this is a .cc file rather than a .c file
 // is so that it can include bochs.h.  Bochs.h includes all the required
-// system headers, with appropriate #ifdefs for different compilers and 
+// system headers, with appropriate #ifdefs for different compilers and
 // platforms.
 //
 
 #include "bochs.h"
 
 //////////////////////////////////////////////////////////////////////
-// Missing library functions.  These should work on any platform 
+// Missing library functions.  These should work on any platform
 // that needs them.
 //////////////////////////////////////////////////////////////////////
 
@@ -66,16 +67,15 @@ int bx_vsnprintf (char *s, size_t maxlen, const char *format, va_list arg)
 #endif /* !BX_HAVE_VSNPRINTF*/
 
 #if (!BX_HAVE_STRTOULL && !BX_HAVE_STRTOUQ)
-/* taken from glibc-2.2.2: strtod.c, and stripped down a lot.  There are 
-   still a few leftover references to decimal points and exponents, 
+/* taken from glibc-2.2.2: strtod.c, and stripped down a lot.  There are
+   still a few leftover references to decimal points and exponents,
    but it works for bases 10 and 16 */
 
 #define RETURN(val,end)							      \
     do { if (endptr != NULL) *endptr = (char *) (end);		      \
 	 return val; } while (0)
 
-Bit64u
-bx_strtoull (const char *nptr, char **endptr, int baseignore)
+Bit64u bx_strtoull (const char *nptr, char **endptr, int baseignore)
 {
   int negative;			/* The sign of the number.  */
   int exponent;			/* Exponent of the number.  */
@@ -113,29 +113,29 @@ bx_strtoull (const char *nptr, char **endptr, int baseignore)
 
   /* Get sign of the result.  */
   if (c == '-')
-    {
-      negative = 1;
-      c = *++cp;
-    }
+  {
+     negative = 1;
+     c = *++cp;
+  }
   else if (c == '+')
     c = *++cp;
 
   if (c < '0' || c > '9')
-    {
-      /* It is really a text we do not recognize.  */
-      RETURN (0, nptr);
-    }
+  {
+     /* It is really a text we do not recognize.  */
+     RETURN (0, nptr);
+  }
 
   /* First look whether we are faced with a hexadecimal number.  */
   if (c == '0' && tolower (cp[1]) == 'x')
-    {
-      /* Okay, it is a hexa-decimal number.  Remember this and skip
-	 the characters.  BTW: hexadecimal numbers must not be
-	 grouped.  */
-      base = 16;
-      cp += 2;
-      c = *cp;
-    }
+  {
+     /* Okay, it is a hexa-decimal number.  Remember this and skip
+        the characters.  BTW: hexadecimal numbers must not be
+        grouped.  */
+     base = 16;
+     cp += 2;
+     c = *cp;
+  }
 
   /* Record the start of the digits, in case we will check their grouping.  */
   start_of_digits = startp = cp;
@@ -162,20 +162,20 @@ bx_strtoull (const char *nptr, char **endptr, int baseignore)
   startp = cp;
   dig_no = 0;
   while (1)
-    {
+  {
       if ((c >= '0' && c <= '9')
 	  || (base == 16 && tolower (c) >= 'a' && tolower (c) <= 'f'))
 	++dig_no;
       else
 	break;
       c = *++cp;
-    }
+  }
 
   /* The whole string is parsed.  Store the address of the next character.  */
   if (endptr)
     *endptr = (char *) cp;
 
-  if (dig_no == 0) 
+  if (dig_no == 0)
     return 0;
 
   for (p=start_of_digits; p!=cp; p++) {
@@ -202,7 +202,7 @@ int main (int argc, char **argv)
     l = strtoul (buf, &endbuf, 10);
     printf ("As a long, %ld\n", l);
     printf ("Endbuf is at buf[%d]\n", endbuf-buf);
-    ll = bx_strtoull (buf, &endbuf, 10);
+    ll = bx_strtoull(buf, &endbuf, 10);
     printf ("As a long long, %lld\n", ll);
     printf ("Endbuf is at buf[%d]\n", endbuf-buf);
   }
@@ -214,14 +214,12 @@ int main (int argc, char **argv)
 /* XXX use real strdup */
 char *bx_strdup(const char *str)
 {
-	char *temp;
-	
-	temp = (char*)malloc(strlen(str)+1);
-	sprintf(temp, "%s", str);
-	return temp;
-	
-	// Well, I'm sure this isn't how strdup is REALLY implemented,
-	// but it works...
+  char *temp = (char*)malloc(strlen(str)+1);
+  sprintf(temp, "%s", str);
+  return temp;
+
+  // Well, I'm sure this isn't how strdup is REALLY implemented,
+  // but it works...
 }
 #endif  /* !BX_HAVE_STRDUP */
 
@@ -270,51 +268,48 @@ int bx_mkstemp(char *tpl)
 
 int fd_read(char *buffer, Bit32u offset, Bit32u bytes)
 {
-	OSErr err;
-	IOParam param;
-	
-	param.ioRefNum=-5; // Refnum of the floppy disk driver
-	param.ioVRefNum=1;
-	param.ioPosMode=fsFromStart;
-	param.ioPosOffset=offset;
-	param.ioBuffer=buffer;
-	param.ioReqCount=bytes;
-	err = PBReadSync((union ParamBlockRec *)(&param));
-	return param.ioActCount;
+  OSErr err;
+  IOParam param;
+
+  param.ioRefNum=-5; // Refnum of the floppy disk driver
+  param.ioVRefNum=1;
+  param.ioPosMode=fsFromStart;
+  param.ioPosOffset=offset;
+  param.ioBuffer=buffer;
+  param.ioReqCount=bytes;
+  err = PBReadSync((union ParamBlockRec *)(&param));
+  return param.ioActCount;
 }
 
 int fd_write(char *buffer, Bit32u offset, Bit32u bytes)
 {
-	OSErr		err;
-	IOParam	param;
-	
-	param.ioRefNum=-5; // Refnum of the floppy disk driver
-	param.ioVRefNum=1;
-	param.ioPosMode=fsFromStart;
-	param.ioPosOffset=offset;
-	param.ioBuffer=buffer;
-	param.ioReqCount=bytes;
-	err = PBWriteSync((union ParamBlockRec *)(&param));
-	return param.ioActCount;
+  OSErr   err;
+  IOParam param;
+
+  param.ioRefNum=-5; // Refnum of the floppy disk driver
+  param.ioVRefNum=1;
+  param.ioPosMode=fsFromStart;
+  param.ioPosOffset=offset;
+  param.ioBuffer=buffer;
+  param.ioReqCount=bytes;
+  err = PBWriteSync((union ParamBlockRec *)(&param));
+  return param.ioActCount;
 }
 
 int fd_stat(struct stat *buf)
 {
-	OSErr		err;
-	DrvSts	status;
-	int			result;
-	
-	result = 0;
-	err = DriveStatus(1, &status);
-	if (status.diskInPlace <1 || status.diskInPlace > 2)
-		result = -1;
-	buf->st_mode = S_IFCHR;
-	return result;
+  OSErr   err;
+  DrvSts  status;
+  int     result = 0;
+
+  err = DriveStatus(1, &status);
+  if (status.diskInPlace <1 || status.diskInPlace > 2)
+    result = -1;
+  buf->st_mode = S_IFCHR;
+  return result;
 }
 #endif /* BX_WITH_MACOS */
 
-
-
 //////////////////////////////////////////////////////////////////////
 // New functions to replace library functions
 //   with OS-independent versions
